#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
import sys
import optparse
import xlwt
import re
from PIL import Image
from commons.core.seq.BioseqDB import BioseqDB
from Gnome_tools.Taxonomy import Taxonomy
from sets import Set

class CreateAnalysisCardForMetaGAnalysis(object):
    def __init__(self):
        self._initialFileName = ""
        self._pyrocleanerFileName = ""
        self._ITSFileName = ""
        self._OTU_tableFileName = ""
        self._OTU_FileName = ""
        self._RepFileName = ""
        self._ReclusteringDir = None

    def setInitialFileName(self, initialFileName):
        self._initialFileName = initialFileName

    def setPyrocleanerFileName(self, file):
        self._pyrocleanerFileName = file

    def setITSFileName(self, file):
        self._ITSFileName = file

    def setOTUTableFileName(self, file):
        self._OTU_tableFileName = file

    def setOTUFileName(self, file):
        self._OTU_FileName = file

    def setRepFileName(self, file):
        self._RepFileName = file

    def setReclusteringDir(self, directory):
        self._ReclusteringDir = directory

    def setAttributesFromCommandLine(self):
        parser = optparse.OptionParser()
        parser.add_option("-i", "--input", dest="input", help="file with raw data")
        parser.add_option("-p", "--pyrocleaner", dest="pyrocleanerFileName", help="file with cleaned data (software like pyrocleaner)")
        parser.add_option("-e", "--extractITS", dest="ITSFileName", help="file with extracted ITS sequences, in analysis_directory/FungalITSExtractor directory")
        parser.add_option("-t", "--OTUTableFile", dest="OTUTableFileName", help="file with OTUs, in analysis_directory/ITSX_otu_table.txt")
        parser.add_option("-o", "--OTUFile", dest="OTUFileName", help="file with picked OTUs, in analysis_directory/uclust_picked_otus/ITS1_otus/ITS1_otus.txt")
        parser.add_option("-r", "--repFile", dest="RepFileName", help="file with rep seq, in analysis_directory/ITSX_X_rep_set.fasta")
        parser.add_option("-c", "--reclusteringDir", dest="ReclusteringDir", help="reclustering directory")

        options, args = parser.parse_args()
        self.setInitialFileName(options.input)
        self.setPyrocleanerFileName(options.pyrocleanerFileName)
        self.setITSFileName(options.ITSFileName)
        self.setOTUTableFileName(options.OTUTableFileName)
        self.setOTUFileName(options.OTUFileName)
        self.setRepFileName(options.RepFileName)
        self.setReclusteringDir(options.ReclusteringDir)

    def checkOptions(self):
        exception = "Argument error"
        if self._initialFileName == None:
            err =  "missing raw data file"
            raise exception, err
        if self._pyrocleanerFileName == None:
            err =  "missing cleaned file"
            raise exception, err
        if self._ITSFileName == None:
            err =  "missing ITS file"
            raise exception, err
        if self._OTU_tableFileName == None:
            err =  "missing OTUs file"
            raise exception, err
        if self._RepFileName == None:
            err =  "missing rep seq file"
            raise exception, err

    def getSampleName(self):
	regexp = re.compile("(.*)(.fasta|.fsa|.fa|.tfa)")
	basename  = os.path.basename(self._initialFileName)
        motif = regexp.search(basename)
        sampleToWrite = motif.group(1)
        return sampleToWrite

    def getNbSequences(self, file):
        iBioseq = BioseqDB()
        iBioseq.load(file)
        nbSequence = iBioseq.getSize()
        return nbSequence

    def getTaxonomyFoundByAnalysis(self, line):
        index = 1
        taxonomy = line.split("\t")[2].split(";")[-1].rstrip()
        while taxonomy == "Other":
            index += 1
            taxonomy = line.split("\t")[2].split(";")[-index].rstrip()
        return taxonomy

    def createClusterDict(self, lSeqRef):
        dictRefSeq2LSeq = {}
        f = open(self._OTU_FileName, "r")

        line = f.readline()
        while line:
            line = line.rstrip()
            cluster = line.split("\t")[1:]
            for seq in lSeqRef:
                if seq in cluster:
                    cluster.remove(seq)
                    dictRefSeq2LSeq[seq] = cluster
                    lSeqRef.remove(seq)
                    break
            line = f.readline()
        f.close()
        return  dictRefSeq2LSeq

    def run(self):
        try:
            self.checkOptions()
        except Exception, val:
            print val

        sampleToWrite = self.getSampleName()
        nbInitialSequence = self.getNbSequences(self._initialFileName)
        nbPyrocleanerFileSequence = self.getNbSequences(self._pyrocleanerFileName)
        nbITSFile = self.getNbSequences(self._ITSFileName)

        self._createPNGFiles()

        lTax = self._getlITaxonomyFromOTUTableFile()

        basename  = os.path.basename(self._initialFileName)
        outputFile = "%s_QIIME_analysis.xls" % basename

        try:
            book = xlwt.Workbook()

            feuil1 = book.add_sheet('ANALYSIS RESUME')

            feuil1.write(0,0,"SAMPLE:")
            feuil1.write(0,1, sampleToWrite)
            feuil1.write(0,2,"SEQUENCE COUNT:")
            feuil1.write(0,3,nbInitialSequence)

            feuil1.write(2, 1, "raw file:")
            feuil1.write(2, 2, self._initialFileName)
            feuil1.write(2, 3, "%s sequences" % nbInitialSequence )

            basename  = os.path.basename(self._initialFileName)
            self._insertPNGFile(feuil1, basename, 3, 0)

            feuil1.write(25, 1, "cleaned file:")
            feuil1.write(25, 2, self._pyrocleanerFileName)
            feuil1.write(25, 3, "%s sequences" % nbPyrocleanerFileSequence )

            basename  = os.path.basename(self._pyrocleanerFileName)
            self._insertPNGFile(feuil1, basename, 27, 0)

            feuil1.write(50, 1, "ITS file:")
            feuil1.write(50, 2, self._ITSFileName)
            feuil1.write(50, 3, "%s sequences" % nbITSFile)

            basename  = os.path.basename(self._ITSFileName)
            self._insertPNGFile(feuil1, basename, 52, 0)

            feuil1.write(75, 1, "TAXONOMY:")
            feuil1.write(75, 2, "NBR OF SEQUENCES:")
            feuil1.write(75, 3, "MAIN SEQUENCE OF THE CLUSTER ")

            lTaxSorted = sorted(lTax, key=lambda taxonomy : taxonomy.getTaxonomyFound())

            lRefSeq = []
            row=75
            for iTax in lTaxSorted:
                row = row + 2
                feuil1.write(row, 1, iTax.getTaxonomyFound())
                feuil1.write(row, 2, int(iTax.getNbrSequence()))

                ReferenceSequenceName = iTax.getSequenceHeader()
                feuil1.write(row, 3, ">%s" % ReferenceSequenceName)
                lRefSeq.append(ReferenceSequenceName)
                feuil1.write(row, 4, iTax.getRefSeq())

            lRefSeqToProcess = []
            lRefSeqToProcess.extend(lRefSeq)
            dRefSeq2lCluster = self.createClusterDict(lRefSeqToProcess)

            if self._ReclusteringDir != None:
                clusters = self._extactClustersFromReclustering(lRefSeq)

            for key in lRefSeq:
                row = 0
                sheet = 2
                iBioseqDB = BioseqDB()
                iBioseqDB.load(self._ITSFileName)
                keySheet = book.add_sheet(key.split("_")[1])

                if self._ReclusteringDir != None:
                    keySheet.write(row, 0, "ref sequence:")
                    iBioseq = iBioseqDB.fetch(key)
                    seq = iBioseq.getSequence()
                    keySheet.write(row, 1, ">%s" % key)
                    keySheet.write(row, 2, seq)
                    row = 2
                    keySheet.write(row, 0, "reclustering")

                    clusterDict = {}
                    for reclustering in sorted(clusters[key]):
                        clusterDict[reclustering] = 0
                        for seqName in clusters[key][reclustering]:
                            row = row + 1
                            keySheet.write(row, 0, reclustering)
                            keySheet.write(row, 1, ">%s" % seqName)
                            iBioseq = iBioseqDB.fetch(seqName)
                            seq = iBioseq.getSequence()
                            keySheet.write(row, 2, seq)
                            clusterDict[reclustering] = clusterDict[reclustering] + 1

                    row = row + 2
                    keySheet.write(row, 0, "summary reclustering:")
                    row = row + 1
                    keySheet.write(row, 0, "cluster index")
                    keySheet.write(row, 1, "nb reads")
                    # write nb_reads / cluster in descending order
                    for reclustering in sorted(clusterDict.items(), key=lambda x: x[1], reverse=True):
                        row = row + 1
                        keySheet.write(row, 0, reclustering[0])
                        keySheet.write(row, 1, reclustering[1])

                else:
                    keySheet.write(row, 0, ">%s" % key)
                    iBioseq = iBioseqDB.fetch(key)
                    seq = iBioseq.getSequence()
                    keySheet.write(row, 1, seq)

                    for seq in dRefSeq2lCluster[key]:
                        row = row + 1
                        keySheet.write(row, 0, ">%s" % seq)
                        iBioseq = iBioseqDB.fetch(seq)
                        seq = iBioseq.getSequence()
                        keySheet.write(row, 1, seq)

                sheet = sheet + 1
        finally:
            book.save(outputFile)

    def _extactClustersFromReclustering(self, lRefSeq):
        if os.path.isdir(self._ReclusteringDir):
            listDir = os.listdir(self._ReclusteringDir)
            clusterDict = {}

            for elmt in listDir:
                path = "%s/%s" % (self._ReclusteringDir, elmt)

                if os.path.isdir(path):
                    suffix = path.split("_")[-1]
                    nb = suffix.split(".")[0]
                    allSeqs = set()
                    f = open("%s/%s/%s_otus.txt" % (self._ReclusteringDir, elmt, nb), 'r')

                    line = f.readline()
                    while line:
                        line = line.rstrip()
                        clusters = line.split("\t")

                        if clusterDict.has_key(nb):
                            clusterDict[nb][clusters[0]] = clusters[1:]
                        else:
                            clusterDict[nb] = {clusters[0]: clusters[1:]}
                        allSeqs.update(clusters[1:])
                        line = f.readline()
                    f.close()

                    # get intersection = seqRef
                    clusterSeqRef = allSeqs & set(lRefSeq)
                    # replace cluster nb by seqref in dict
                    clusterDict[list(clusterSeqRef)[0]] = clusterDict.pop(nb)

            return clusterDict

        else:
            sys.exit("ERROR : Reclustering directory \"%s\" not found" % self._ReclusteringDir)

    def _LaunchGetSizeCmd(self, file):
        workingDir = os.getcwd()
        basename  = os.path.basename(file)
        cmd = "python %s/SMART/Java/Python/getSizes.py -f fasta -i %s -o %s/%s.png" % (os.getenv("REPET_PATH"), file, workingDir, basename)
        os.system(cmd)

    def _createPNGFiles(self):
        self._LaunchGetSizeCmd(self._initialFileName)
        self._LaunchGetSizeCmd(self._ITSFileName)
        self._LaunchGetSizeCmd(self._pyrocleanerFileName)

    def _insertPNGFile(self, feuil1, file, line, col):
        file_in = "%s.png" % file
        img = Image.open(file_in).convert("RGB")
        file_out = "%s.bmp" % file
        img.resize((500, 300)).save(file_out)
        feuil1.insert_bitmap(file_out, line, col)
        return file_in, file_out, img

    def _getlITaxonomyFromOTUTableFile(self):
        lTax = []
        f = open(self._OTU_tableFileName, "r")
        line = f.readline()
        while line:
            if not line.startswith("#"):
                taxonomy = self.getTaxonomyFoundByAnalysis(line)
                referenceId = line.split("\t")[0]
                nbSequence2ref = line.split("\t")[1]
                iTax = Taxonomy(taxonomy, referenceId, nbSequence2ref)
                iTax.getReferenceSequence(self._RepFileName)
                lTax.append(iTax)
            line = f.readline()

        f.close()
        return lTax

if __name__ == "__main__":
    iCSAC =  CreateAnalysisCardForMetaGAnalysis()
    iCSAC.setAttributesFromCommandLine()
    iCSAC.run()
