#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import getopt
import sys
import time
import os

from Bio import SeqIO
from Bio import Entrez

class DownloadEntriesFromGenbank():

	verbose = True

	def __init__(self):
		self._webenv = None
		self._batch_size = 100
		self._query_key = None
		self._retstart = None
		self._nbEntries = None
		self._email = None

	def setAttributesFromCmdLine(self):
		try:
			opts, args = getopt.getopt(sys.argv[1:],"ve:b:w:q:s:n:",["email=","verbose","help","batchsize=","webenv=","querykey=","retstart=","nbentries="])
		except getopt.GetoptError, err:
			print str(err)
			self.help()
		for o,a in opts:
			if o in ("-v","--verbose"):
				verbose = True
			elif o in ("-e","--email"):
				self._email = a
			elif o in ("-b","--batchsize"):
				self._batch_size = int(a)
			elif o in ("-w","--webenv"):
				self._webenv = a
			elif o in ("-q","--querykey"):
				self._query_key = a
			elif o in ("-s","--retstart"):
				self._retstart = int(a)
			elif o in ("-n","--nbentries"):
				self._nbEntries = int(a)
			elif o in ("--help","--usage"):
				self.help()
				sys.exit()
			else:
				assert False, "unhandled option"
				self.help()

		if not self._email:
			print "\n ERROR: missing parameter -e or --email"
			self.help()

		if (self._retstart or self._nbEntries or self._webenv or self._query_key) and (not self._webenv or not self._query_key or not self._retstart or not self._nbEntries):
			print "\n ERROR: nbentries, webenv and querykey must be defined when using restart and option"
			self.help()


	def help(self):
		print
		print "usage: %s [ options ]" % (sys.argv[0])
		print
		print "args (mandatory):"
		print "  -e or --email"
		print
		print "args (optional):"
		print "  -b or --batchsize [int]"
		print "  -w or --webenv"
		print "  -q or --querykey"
		print "  -s or --retstart [int]"
		print "  -n or --nbentries [int]"
		print "  -v or --verbose"
		print "  --help, --usage"
		sys.exit()


	def downloadEntries(self, start, end):

		filename = "%i_to_%i.gb" % (start+1, end)

		if self.verbose:
			print "Going to download record %i to %i" % (start+1, end)

		if os.path.isfile(filename):
			print "%s exists, will be remove and replace" % (filename)

		try:
			# Downloading...
			net_handle = Entrez.efetch(db="nucleotide",rettype="gb",webenv=self._webenv, query_key=self._query_key, retstart=start, retmax=self._batch_size,usehistory="y")
			out_handle = open(filename, "w")
			out_handle.write(net_handle.read())
			out_handle.close()
			net_handle.close()
			print "Saved"
		except:
			raise Exception

		try:
			self.getNumberOfRecords(filename)
		except:
			raise Exception


	def getNumberOfRecords(self,filename):

		try:
			records = list(SeqIO.parse(filename, "genbank"))
			if len(records) < self._batch_size:
				raise Exception
		except:
			print "cannot read filename not genbank file or missing entries"
			raise Exception

	def getIdList(self):

		Entrez.email = self._email
		# get the number of entries
		handle = Entrez.egquery(term="Fungi [Organism] AND (200 [SLEN] : 3000 [SLEN])) AND (((ITS1 [titl] OR ITS2 [titl]) OR 5.8S [titl]) OR internal transcribed spacer [titl] OR internal transcribed spacers [titl]))")
		record = Entrez.read(handle)
		for row in record["eGQueryResult"]:
			if row["DbName"]=="nuccore":
				nbEntries = row["Count"]

		if self.verbose:
			print "# %s entries found for request #" %(nbEntries)

		# get Ids list
		handle = Entrez.esearch(db="nuccore",term="Fungi [Organism] AND (200 [SLEN] : 3000 [SLEN])) AND (((ITS1 [titl] OR ITS2 [titl]) OR 5.8S [titl]) OR internal transcribed spacer [titl] OR internal transcribed spacers [titl]))", retmax=nbEntries, usehistory="y")
		record = Entrez.read(handle)
		lIds = record["IdList"]

		# post Ids list at NCBI
		search_results = Entrez.read(Entrez.epost("nuccore", id=",".join(lIds)))
		self._webenv = search_results["WebEnv"]
		self._query_key = search_results["QueryKey"]

		# nb Entries
		self._nbEntries = int(nbEntries)

		print "webenv : %s " % (self._webenv)
		print "query_key : %s " % (self._query_key)


	def exe(self):

		if not self._nbEntries and not self._retstart and not self._webenv and not self._query_key:
			# get Id List
			self.getIdList()
		else:
			Entrez.email = self._email

		# get data
		if self._retstart:
			intial_start = self._retstart
		else:
			intial_start = 0

		#print "%d %d %d" % (intial_start,self._nbEntries-intial_start,self._batch_size)

		for start in range(intial_start,self._nbEntries-intial_start,self._batch_size):
			end = min(self._nbEntries-intial_start, start+self._batch_size)
			success = False
			tries = 0
			while success is False:
				try:
					self.downloadEntries(start, end)
					success = True
				except Exception, err:
					tries = tries + 1
					print "Error in download from %s to %s" % (start, end)
					#print "ERROR: %s\n" % str(err)
					if tries > 4:
						print "Failed to download after 4 tries"
						sys.exit()


if __name__=="__main__":
	iDownloadEntriesFromGenbank = DownloadEntriesFromGenbank()
	iDownloadEntriesFromGenbank.setAttributesFromCmdLine()
	iDownloadEntriesFromGenbank.exe()

