#! /usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
import glob
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser
from commons.core.seq.BioseqDB import BioseqDB
from commons.core.checker.CheckerUtils import CheckerUtils
from commons.core.sql.DbFactory import DbFactory
from commons.core.sql.TableJobAdaptatorFactory import TableJobAdaptatorFactory
from commons.core.launcher.Launcher import Launcher
from commons.core.utils.RepetConfigParser import RepetConfigParser
import shutil
import uuid

class MetaGPipe(object):

    def __init__(self, inputFileName = "", referenceFileName = "", taxonomyFileName = "", workingDir = "", sequenceType = "", configFileName="", reclustering=None, extraction=True):
        self._options = None
        self._inputFileName = inputFileName
        self._referenceFileName = referenceFileName
        self._taxonomyFileName = taxonomyFileName
        self._workingDir = workingDir
        self._sequenceType = sequenceType
        self._configFileName = configFileName
        self._reclustering = reclustering
        self._outputNewHFileName = "%s_newH.fasta" % (os.path.splitext(self._inputFileName)[0])
        self._extraction = extraction
        #self._queue = "test=TRUE"
        self._queue = ""

    def setAttributesFromCmdLine(self):
        description = "MetaGPipe is a pipeline for metagenomic data.\n" 
        usage = "MetaGPipe.py -i <input file> -r <reference file> -t <taxonomy file> -w <directory> -C <configuration file> -s <sequence type to analyse> [--noextraction]\n"
        parser = RepetOptionParser(description = description, usage = usage)
        parser.add_option('-i', "--input", dest="inputFileName", action="store", type="string", help="input file (fasta type)", default="")
        parser.add_option('-r', "--reference", dest="reference", action="store", type="string", help="reference file ", default="")
        parser.add_option('-t', "--taxonomy", dest="taxonomy", action="store", type="string", help="taxonomy file (optional)", default="")
        parser.add_option('-w', "--workingDir", dest="workingDir", action="store", type="string", help="output directory", default="")
        parser.add_option('-C', "--configFileName", dest="configFileName", action="store", type="string", help="configuration file", default="")
        parser.add_option('-s', "--sequenceType", dest="sequenceType", action="store", type="string", help="sequence type (ITS1/ITS2/Both/ITS1Complete)", default="")
        parser.add_option('-c', "--reclustering", dest="reclustering", action="store", type="float", help="% similarity for reclustering, accepted values between [0.97-1]", default=None)
        parser.add_option("--noextraction", dest="extraction", action="store_false", help="ITS Extraction", default=True)

        options, args = parser.parse_args()
        self._setAttributesFromOptions(options)

    def _setAttributesFromOptions(self, options):
        self.setWorkingDir(options.workingDir)
        self.setInputFileName(options.inputFileName)
        self.setReferenceFileName(options.reference)
        self.setTaxonomyFileName(options.taxonomy)
        self.setConfigurationFileName(options.configFileName)
        self.setSequenceType(options.sequenceType)
        self.setReclustering(options.reclustering)
        self.setExtraction(options.extraction)
        
        self._outputNewHFileName = "%s/%s_newH.fasta" % (self._workingDir, os.path.splitext(os.path.basename(self._inputFileName))[0])
        self._outputITSFileName = "%s_ITS.fasta" % (os.path.splitext(self._outputNewHFileName)[0])
        self._outputClusterFileName = "%s_otu.txt" % (os.path.splitext(self._outputITSFileName)[0])
        self._outputMajoritySequenceFileName = "%s_ref_set.fasta" % (os.path.splitext(self._outputClusterFileName)[0])
        self._outputAssignationFileName = "%s_tax_assignments.txt" % (os.path.splitext(self._outputMajoritySequenceFileName)[0])
        
    def setWorkingDir(self, workingDir):
        self._workingDir = os.path.abspath(workingDir)
        
    def setInputFileName(self, inputFileName):
        self._inputFileName = os.path.abspath(inputFileName)
        
    def setReferenceFileName(self, referenceFileName):
        self._referenceFileName = os.path.abspath(referenceFileName)
        
    def setTaxonomyFileName(self, taxonomyFileName):
        self._taxonomyFileName = os.path.abspath(taxonomyFileName)
        
    def setSequenceType(self, sequenceType):
        self._sequenceType = sequenceType
        
    def setConfigurationFileName(self, configurationFileName):
        self._configFileName = os.path.abspath(configurationFileName)

    def setReclustering(self, reclustering):
        self._reclustering = reclustering

    def setExtraction(self, extraction):
        self._extraction = extraction
        
    def checkExecutables(self):
        if not CheckerUtils.isExecutableInUserPath("uclust"):
            raise Exception("ERROR: uclust must be in your path")
        
        if not CheckerUtils.isExecutableInUserPath("blastall"):
            raise Exception("ERROR: blastall (blast) must be in your path")
        
        if not CheckerUtils.isExecutableInUserPath("assign_taxonomy.py"):
            raise Exception("ERROR: Qiime must be in your path")
        
    def checkOptions(self):       
        if self._workingDir == "":
            raise Exception("ERROR: No specified -w option!")
        
        if self._inputFileName != "":
            if not FileUtils.isRessourceExists(self._inputFileName):
                raise Exception("ERROR: Input file %s does not exist!" % self._inputFileName)
        else:
            raise Exception("ERROR: No specified -i option!")
        
        if self._referenceFileName != "":
            if not FileUtils.isRessourceExists(self._referenceFileName):
                raise Exception("ERROR: reference file does not exist!" % self._referenceFileName)
        else:
            raise Exception("ERROR: No specified -r option!")
        
        if self._taxonomyFileName != "":
            if not FileUtils.isRessourceExists(self._taxonomyFileName):
                raise Exception("ERROR: taxonomy mapping file %s does not exist!" % self._taxonomyFileName)
        else:
            raise Exception("ERROR: No specified -t option!")
        
        if self._sequenceType != "":
            lSequenceTypeOptions = ["ITS1", "ITS2", "Both", "ITS1Complete"]
            if self._sequenceType not in lSequenceTypeOptions:
                raise Exception("ERROR: This sequence type %s is not allowed" %  self._sequenceType)
        else:    
            raise Exception("ERROR: No specified -s option!")
        
        if self._configFileName != "":
            if not FileUtils.isRessourceExists(self._configFileName):
                raise Exception("ERROR: Config file %s does not exist" %  self._configFileName)
        else:    
            raise Exception("ERROR: No specified -C option!")

        if self._reclustering != None:
            if self._reclustering < 0.97 or self._reclustering > 1:
                err = "ERROR: similarity percentage for reclustering range is [0.97-1]"
                raise Exception(err) 
            
    def renameHeaders(self):
        tag = self._inputFileName.split(".")[1]
        index = 0
        iBioseqDB = BioseqDB(self._inputFileName)
        for iBioseq in iBioseqDB.db:
            index += 1
            iBioseq.setHeader("%s_%s%s" % (tag, index, iBioseq.getHeader()))
        iBioseqDB.save(self._outputNewHFileName, "w")    
                             
    def run(self):
        sectionName ="MetaG_env"
        self.checkExecutables()
        self.checkOptions()
        
        if not os.path.exists(self._workingDir):
            os.makedirs(self._workingDir)
        else:
            raise Exception("WARNING: directory %s exists ! To perform new analyses, please remove or rename existing directory." % self._workingDir)

        os.chdir(self._workingDir)
        os.symlink(self._configFileName, "%s/%s" % (self._workingDir, os.path.basename(self._configFileName)))
        os.symlink(self._inputFileName, "%s/%s" % (self._workingDir, os.path.basename(self._inputFileName)))
        
        iconfigParser = RepetConfigParser()
        iconfigParser.readfp(open(self._configFileName))
        
        os.environ["REPET_HOST"] = iconfigParser.get("MetaG_env", "repet_host")
        os.environ["REPET_USER"] = iconfigParser.get("MetaG_env", "repet_user")
        os.environ["REPET_PW"] = iconfigParser.get("MetaG_env", "repet_pw")
        os.environ["REPET_DB"] = iconfigParser.get("MetaG_env", "repet_db")
        os.environ["REPET_PORT"] = iconfigParser.get("MetaG_env", "repet_port")
        os.environ["REPET_JOB_MANAGER"] = iconfigParser.get("MetaG_env", "repet_job_manager")
        os.environ["METAGPIPE"] = iconfigParser.get("MetaG_env", "metagpipe")  
        os.environ["PROFILES_PATH"] = iconfigParser.get("MetaG_env", "profile_path") 
        
        self.renameHeaders()
        
        cDir = os.getcwd()
        
        if iconfigParser.get(sectionName, "tmpDir") != "":
            tmpDir = iconfigParser.get(sectionName, "tmpDir")
        else:
            tmpDir = cDir
        
        uu = uuid.uuid4()
        self._uu = uu

        if(self._extraction == True):
	        self._launchSTEP1(tmpDir)
	        self._launchSTEP2(tmpDir)
            
	        lDirs = glob.glob('%s/batch*' % self._workingDir)
	        for dir in lDirs:
          	  shutil.rmtree( "%s" % dir)
        else:
            self._launchSTEP1bis(tmpDir)
        
        self._launchSTEP3(tmpDir)
        self._launchSTEP4(tmpDir)
        self._launchSTEP5(tmpDir)
        
        os.system("make_otu_table.py -i %s/uclust_picked_otus/%s_otus.txt -o %s/%s_otu_table.txt -t '%s/%s_%s_tax_assignments.txt'" % (self._workingDir, self._sequenceType, self._workingDir, self._sequenceType, self._workingDir, os.path.splitext(os.path.basename(self._inputFileName))[0], self._sequenceType))

        if self._reclustering != None:
            self._launchSTEP6(tmpDir)
        
    def _createDbSplitCommand(self, iLauncher):
        lArgs = []
        lArgs.append("-i %s" % os.path.basename(self._outputNewHFileName))
        lArgs.append("-n 50")
        lArgs.append("-d")
        return iLauncher.getSystemCommand("dbSplit.py", lArgs)
    
    def _createFungalITSExtractorCommand(self, inFile, iLauncher):
        lArgs = []
        lArgs.append("-i %s" % inFile)
        lArgs.append("-w %s" % (os.path.splitext(inFile)[0]))
        lArgs.append("-p profile_dir")
        return iLauncher.getSystemCommand("LaunchFungalITSExtractor.py", lArgs)
    
    def _createPickOTUWithUclustCommand(self, inFile, similarity, iLauncher):
        lArgs = []
        lArgs.append("-i %s" % os.path.basename(inFile))
        lArgs.append("-s %s" % str(similarity))
        return iLauncher.getSystemCommand("LaunchPickOTUWihtUclust.py", lArgs)
    
    def _createPickRepSetCommand(self, inFile, clusterFile, iLauncher):
        lArgs = []
        lArgs.append("-i %s" % os.path.basename(inFile))
        lArgs.append("-c %s" % os.path.basename(clusterFile))
        return iLauncher.getSystemCommand("LaunchPickRepSet.py", lArgs)
    
    def _createAssignTaxonomyCommand(self, inFile, referenceFile, taxonomyFile, iLauncher):
        lArgs = []
        lArgs.append("-i %s" % os.path.basename(inFile))
        lArgs.append("-r %s" % os.path.basename(referenceFile))
        lArgs.append("-t %s" % os.path.basename(taxonomyFile))
        return iLauncher.getSystemCommand("LaunchAssignTaxonomyWithBlast.py", lArgs)
    
    def _launchSTEP1(self, tmpDir):
        groupid = "DBSplit_%s" % self._uu
        acronym = "LaunchDBSplit"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, cdir=self._workingDir, tmpdir=tmpDir, groupid=groupid, acro=acronym, queue=self._queue)
        lCmds = []
        lCmds.append(self._createDbSplitCommand(iLauncher))
        lCmdStart = []
        lCmdStart.append("os.symlink('%s', '%s')" % (self._outputNewHFileName, os.path.basename(self._outputNewHFileName)))
        lCmdFinish = []
        lCmdFinish.append("shutil.move('batches', '%s/batches')\n" % self._workingDir)
        lCmdsTuples = []
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples)

    def _launchSTEP2(self, tmpDir):
        groupid = "FungalITSExtractor_%s" % self._uu
        acronym = "LaunchFungalITSExtractor"
        os.mkdir("%s/%s" % (self._workingDir, "FungalITSExtractor"))
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, cdir=self._workingDir, tmpdir=tmpDir, groupid=groupid, acro=acronym, queue=self._queue)
        lFiles = glob.glob("%s/batches/*.fa" % self._workingDir)
        lCmdsTuples = []
        for file in lFiles:
            lCmds = []
            lCmds.append(self._createFungalITSExtractorCommand(os.path.basename(file), iLauncher))
            lCmdStart = []
            lCmdStart.append("os.symlink('%s', '%s')\n" % (file, os.path.basename(file)))
            lCmdStart.append("os.symlink(os.environ['PROFILES_PATH'], 'profile_dir')\n")
            lCmdFinish = []
            outputDir = os.path.splitext(os.path.basename(file))[0]
            lCmdFinish.append("lDir = os.listdir('%s')\n" % outputDir)
            lCmdFinish.append("absOutputDir = os.path.abspath('%s')\n" % outputDir)
            lCmdFinish.append("for dir in lDir:\n")
            lCmdFinish.append("\tdir = '%s/%s' % (absOutputDir,dir)")
            lCmdFinish.append("\tshutil.move(dir, '%s')\n" % self._workingDir)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, True)
        lResultFileName = ["Both.fasta", "Both.txt", "ITS1.fasta", "ITS1Complete.fasta", "ITS1CompleteNegativeStrand.fasta", "ITS1CompletePositiveStrand.fasta", "ITS2.fasta", "NoITS1.fasta", "NoITS2.fasta", "None.txt"]
        for i in lResultFileName:
            lFiles = glob.glob("%s/batch_*/%s" % (self._workingDir, i))
            FileUtils.catFilesFromList(lFiles, "%s/%s/%s" % (self._workingDir,"FungalITSExtractor" , i))

    def _launchSTEP1bis(self, tmpDir):
        os.mkdir("%s/%s" % (self._workingDir, "FungalITSExtractor"))
        os.symlink(self._outputNewHFileName, "%s/%s/%s.fasta" % (self._workingDir,"FungalITSExtractor" , self._sequenceType))
        
    def _launchSTEP3(self, tmpDir):
        groupid = "PickOTUWithUclust_%s" % self._uu
        acronym = "LaunchPickOTUWithUclust"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, cdir=self._workingDir, tmpdir=tmpDir, groupid=groupid, acro=acronym, queue=self._queue)
        lCmds = []
        lCmds.append(self._createPickOTUWithUclustCommand("FungalITSExtractor/%s.fasta" % self._sequenceType, 0.97, iLauncher))
        lCmdStart = []
        lCmdStart.append("os.symlink('%s/FungalITSExtractor/%s.fasta', '%s.fasta')" % (self._workingDir, self._sequenceType, self._sequenceType))
        lCmdFinish = []
        lCmdFinish.append("shutil.move('uclust_picked_otus', '%s/')\n" % self._workingDir)
        lCmdsTuples = []
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples)

    def _launchSTEP4(self, tmpDir):
        groupid = "PickRepSet_%s" % self._uu
        acronym = "LaunchPickRepSet"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, cdir=self._workingDir, tmpdir=tmpDir, groupid=groupid, acro=acronym, queue=self._queue)
        lCmds = []
        lCmds.append(self._createPickRepSetCommand("FungalITSExtractor/%s.fasta" % (self._sequenceType), 'uclust_picked_otus/%s_otus.txt' % self._sequenceType, iLauncher))
        lCmdStart = []
        lCmdStart.append("os.symlink('%s/FungalITSExtractor/%s.fasta', '%s.fasta')" % (self._workingDir, self._sequenceType, self._sequenceType))
        lCmdStart.append("os.symlink('%s/uclust_picked_otus/%s_otus.txt', '%s_otus.txt')" % (self._workingDir, self._sequenceType, self._sequenceType))
        lCmdFinish = []
        lCmdFinish.append("shutil.move('%s.fasta_rep_set.fasta', '%s/%s.fasta_rep_set.fasta')\n" % (self._sequenceType, self._workingDir, self._sequenceType))
        lCmdsTuples = []
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples)

    def _launchSTEP5(self, tmpDir):
        groupid = "AssignTaxonomy_%s" % self._uu
        acronym = "LaunchAssignTaxonomy"
        os.symlink(self._referenceFileName, "%s/%s" % (self._workingDir, os.path.basename(self._referenceFileName)))
        os.symlink(self._taxonomyFileName, "%s/%s" % (self._workingDir, os.path.basename(self._taxonomyFileName)))
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, cdir=self._workingDir, tmpdir=tmpDir, groupid=groupid, acro=acronym, queue=self._queue)
        lCmds = []
        lCmds.append(self._createAssignTaxonomyCommand("%s.fasta_rep_set.fasta" % self._sequenceType, self._referenceFileName, self._taxonomyFileName, iLauncher))
        lCmdStart = []
        lCmdStart.append("os.symlink('%s/%s.fasta_rep_set.fasta', '%s.fasta_rep_set.fasta')" % (self._workingDir, self._sequenceType, self._sequenceType))
        lCmdStart.append("os.symlink('%s/%s', '%s')" % (self._workingDir, os.path.basename(self._referenceFileName), os.path.basename(self._referenceFileName)))
        lCmdStart.append("os.symlink('%s/%s', '%s')" % (self._workingDir, os.path.basename(self._taxonomyFileName), os.path.basename(self._taxonomyFileName)))
        lCmdFinish = []
        lCmdFinish.append("shutil.move('blast_assigned_taxonomy/%s.fasta_rep_set_tax_assignments.txt', '%s/%s_%s_tax_assignments.txt')\n" % (self._sequenceType, self._workingDir, os.path.splitext(os.path.basename(self._inputFileName))[0], self._sequenceType))
        lCmdsTuples = []
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples)

    def _launchSTEP6(self, tmpDir):
        clusters = self.createClusterDict()
        folder = "reclustering"
        iBioseqDB = BioseqDB()
        iBioseqDB.load("%s/FungalITSExtractor/%s.fasta" % (self._workingDir, self._sequenceType))
        # write dir
        if not os.path.isdir(folder):
            os.system("mkdir %s" % folder)
        # write file
        lClusterFiles = []
        for key in clusters:
            clusterFileName = "%s.fasta" % (key)
            clusterFile = open("%s/%s" % (folder, clusterFileName), "w")
            for seqName in clusters[key]:
                iBioseq = iBioseqDB.fetch(seqName)
                seq = iBioseq.getSequence()
                clusterFile.write(">%s\n%s\n" % (seqName, seq))                     
            clusterFile.close()
            lClusterFiles.append(clusterFileName)

        # Launcher
        groupid = "Reclustering_%s" % self._uu
        acronym = "LaunchPickOTUWithUclust"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, cdir=self._workingDir, tmpdir=tmpDir, groupid=groupid, acro=acronym, queue=self._queue)
        lCmdsTuples = []
        for clusterFile in lClusterFiles:
            lCmds = []
            lCmds.append(self._createPickOTUWithUclustCommand(clusterFile, self._reclustering, iLauncher))
            lCmdStart = []
            lCmdStart.append("os.symlink('%s/%s/%s', '%s')" % (self._workingDir, folder, clusterFile, clusterFile))
            lCmdFinish = []
            lCmdFinish.append("shutil.move('uclust_picked_otus', '%s/%s/reclustering_%s_%s')\n" % (self._workingDir, folder, str(self._reclustering), clusterFile))
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, True)

    def createClusterDict(self):
        dictClusters = {}
        f = open("%s/uclust_picked_otus/%s_otus.txt" % (self._workingDir, self._sequenceType), "r")
        line = f.readline()
        while line:
            line = line.rstrip()
            cluster = line.split("\t")
            dictClusters[cluster[0]] = cluster[1:]
            line = f.readline()
        f.close()
        return dictClusters
            
if __name__ == "__main__":
    iLaunch = MetaGPipe()
    iLaunch.setAttributesFromCmdLine()
    iLaunch.run()
