import unittest
from commons.core.coord.CountOverlapping import CountOverlapping
from commons.core.coord.Range import Range
from commons.core.checker.RepetException import RepetException
from commons.core.coord.Map import Map

class Test_CountOverlapping(unittest.TestCase):

    def test_CountOverlapping_simple_data_no_interval(self):
        lRanges = []
        lRanges.append(Range("seq1", 1, 4))
        lRanges.append(Range("seq1", 28, 33))
        lRanges.append(Range("seq1", 3, 16))
        lRanges.append(Range("seq1", 27, 33))
        lRanges.append(Range("seq1", 5, 20))
        lRanges.append(Range("seq1", 17, 22))
        lRanges.append(Range("seq1", 31, 35))
        lRanges.append(Range("seq1", 25, 30))
        queryInterval = Range("seq1", 23, 24)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(0, iCO.count(queryInterval))

    def test_CountOverlapping_simple_data_no_interval_different_seqnames(self):
        lRanges = []
        lRanges.append(Range("seq1", 1, 4))
        lRanges.append(Range("seq1", 28, 33))
        queryInterval = Range("seq2", 23, 24)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(0, iCO.count(queryInterval))
        
    def test_CountOverlapping_simple_data_with_all_interval(self):
        lRanges = []
        lRanges.append(Range("seq1", 5, 20))
        lRanges.append(Range("seq1", 28, 33))
        lRanges.append(Range("seq1", 17, 22))
        lRanges.append(Range("seq1", 25, 30))
        lRanges.append(Range("seq1", 31, 35))
        queryInterval = Range("seq1", 1, 40)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(5, iCO.count(queryInterval))
        
    def test_CountOverlapping_sorting_check(self):
        lRanges = []
        lRanges.append(Range("seq1", 1, 10))
        lRanges.append(Range("seq1", 2, 5))
        lRanges.append(Range("seq1", 15, 20))
        queryInterval = Range("seq1", 6, 10)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(1, iCO.count(queryInterval))
        
    def test_CountOverlapping_simple_data_with_exact_intersection(self):
        lRanges = []
        lRanges.append(Range("seq1", 31, 35))
        lRanges.append(Range("seq1", 5, 20))
        lRanges.append(Range("seq1", 17, 22))
        lRanges.append(Range("seq1", 28, 33))
        lRanges.append(Range("seq1", 25, 30))
        queryInterval = Range("seq1", 5, 20)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(2, iCO.count(queryInterval))
    
    def test_CountOverlapping_simple_data_with_exact_intersection_maps_and_range(self):
        lRanges = []
        lRanges.append(Map("region1", "seq1", 31, 35))
        lRanges.append(Map("region1", "seq1", 5, 20))
        lRanges.append(Range("seq1", 17, 22))
        lRanges.append(Map("region2", "seq1", 28, 33))
        lRanges.append(Map("region3", "seq1", 25, 30))
        queryInterval = Range("seq1", 5, 20)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(2, iCO.count(queryInterval))
    
    
# ======= Test with border cases =======
    
    def test_CountOverlapping_empty_data(self):
        lRanges = []
        queryInterval = Range("seq1", 5, 20)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(0, iCO.count(queryInterval))
        
    def test_CountOverlapping_reverse_strand(self):
        lRanges = []
        lRanges.append(Range("seq1", 1, 10))
        lRanges.append(Range("seq1", 5, 2))
        lRanges.append(Range("seq1", 15, 20))
        queryInterval = Range("seq1", 10, 2)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(2, iCO.count(queryInterval))
        
    def test_CountOverlapping_reverse_strand_bad_results(self):
        lRanges = []
        lRanges.append(Range("seq1", 1, 10))
        lRanges.append(Range("seq1", 5, 2))
        lRanges.append(Range("seq1", 15, 20))
        queryInterval = Range("seq1", 2, 10)
        
        # areFeaturesOnDirectStrandsOnly is wrongly set to True: results will be incorrect
        iCO = CountOverlapping(lRanges, areFeaturesOnDirectStrandsOnly = True)
        
        # thus we do not expect the good result 
        self.assertNotEqual(2, iCO.count(queryInterval))
        
    def test_CountOverlapping_1_base_interval(self):
        lRanges = []
        lRanges.append(Range("seq1", 31, 35))
        lRanges.append(Range("seq1", 5, 20))
        lRanges.append(Range("seq1", 17, 22))
        lRanges.append(Range("seq1", 28, 33))
        lRanges.append(Range("seq1", 25, 30))
        queryInterval = Range("seq1", 5, 5)
        
        iCO = CountOverlapping(lRanges)
        
        self.assertEqual(1, iCO.count(queryInterval))
    
    def test_CountOverlapping_RaiseException_different_seqnames(self):
        lRanges = []
        lRanges.append(Map("region2", "seq1", 17, 22))
        lRanges.append(Range("seq2", 25, 30))
        lRanges.append(Range("seq1", 31, 35))
        
        obsMessage = ""
        isExceptionRaised = False
        try:
            CountOverlapping(lRanges)
        except RepetException as e:
            obsMessage = e.getMessage()
            isExceptionRaised = True
        
        expMessage = "ERROR: different sequence names in input features list"
        
        self.assertTrue(isExceptionRaised)
        self.assertEqual(expMessage, obsMessage)
        
if __name__ == "__main__":
    unittest.main()