import os
import time
import shutil
import unittest
from commons.launcher.LaunchDSK import LaunchDSK
from commons.core.utils.FileUtils import FileUtils

class Test_F_LaunchDSK(unittest.TestCase):

    def setUp(self):
        self._inFastaFileName = "%s/Tools/DmelChr4.fa" % os.environ["REPET_DATA"]
        self.obsFileName = "DmelChr4.dsk.gff"
        self.testDir = os.path.join(os.getcwd(), "TestLaunchDSK%s" % time.strftime("%Y%m%d%H%M%S"))
        os.makedirs(self.testDir)
        os.chdir(self.testDir)
        
    def tearDown(self):
        os.chdir(self.testDir)
        os.chdir("..")
        shutil.rmtree(self.testDir)

    def test_run_output_as_gff_cleaned(self):
        iLaunchDSK = LaunchDSK(self._inFastaFileName, clean = True, verbosity = 2)
        iLaunchDSK.run()

        expFileName = "%s/Tools/DmelChr4.fa.dsk_k20.gff" % os.environ["REPET_DATA"]

        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self.obsFileName))

    def test_run_with_remote_dir(self):
        os.makedirs("sourceDir")
        changedInFastaFileName = os.path.basename(self._inFastaFileName)
        changedInFastaFilepath = os.path.join("sourceDir", changedInFastaFileName)
        shutil.copy2(self._inFastaFileName, changedInFastaFilepath)
        absolutePathFastaFile = os.path.join(os.getcwd(), changedInFastaFilepath)

        iLaunchDSK = LaunchDSK(absolutePathFastaFile, clean = True, verbosity = 2)
        iLaunchDSK.run()

        expFileName = "%s/Tools/DmelChr4.fa.dsk_k20.gff" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.gff"

        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        obsSourceDirFiles = os.listdir("sourceDir")
        expSourceDirFiles = [changedInFastaFileName, ]
        self.assertEquals(expSourceDirFiles, obsSourceDirFiles)

    def test_run_output_as_gff_cleaned_changed_minOccs2(self):
        iLaunchDSK = LaunchDSK(self._inFastaFileName, minOccs = 2, clean = True, verbosity = 2)
        iLaunchDSK.run()

        expFileName = "%s/Tools/DmelChr4.fa.dsk_minOcc2.gff" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.gff"

        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))

    def test_run_output_as_gff_cleaned_relative_input(self):
        srcFastaFileName = self._inFastaFileName
        dstFastaFileName = os.path.basename(self._inFastaFileName)
        shutil.copy2(srcFastaFileName, dstFastaFileName)
        iLaunchDSK = LaunchDSK(dstFastaFileName, clean = True, verbosity = 2)
        iLaunchDSK.run()

        expFileName = "%s/Tools/DmelChr4.fa.dsk_k20.gff" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.gff"

        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))

    def test_run_as_script_output_as_gff_cleaned_relative_input(self):
        srcFastaFileName = self._inFastaFileName
        dstFastaFileName = os.path.basename(self._inFastaFileName)
        shutil.copy2(srcFastaFileName, dstFastaFileName)

        cmd = 'LaunchDSK.py -i %s -s 20 -c -v 3' % (self._inFastaFileName)
        os.system(cmd)
        expFileName = "%s/Tools/DmelChr4.fa.dsk_k20.gff" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.gff"

        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))

    def test_run_output_as_wig_cleaned(self):
        iLaunchDSK = LaunchDSK(self._inFastaFileName, outputFormats = "wig", clean = False)
        iLaunchDSK.run()

        expFileName = "%s/Tools/DmelChr4.fa.dsk_k20.wig" % os.environ["REPET_DATA"]
        self.obsFileName = "DmelChr4.dsk.wig"
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self.obsFileName))

    def test_run_output_as_gff_and_wig_k15_cleaned(self):
        iLaunchDSK = LaunchDSK(self._inFastaFileName, merSize = 15, outputFormats = "wig,gff3", clean = True)
        iLaunchDSK.run()

        expFileName = "%s/Tools/DmelChr4.fa.dsk_k15.wig" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.wig"
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))

        expFileName = "%s/Tools/DmelChr4.fa.dsk_k15.gff3" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.gff3"
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))

    def test_run_as_script_run_output_as_gff_and_wig_k15_cleaned(self):
        logFileName = "test.log"
        cmd = 'LaunchDSK.py -i %s -s 15 -f gff,wig,vero -c -v 3 > %s' % (self._inFastaFileName, logFileName)
        os.system(cmd)
               
        expFileName = "%s/Tools/DmelChr4.fa.dsk_k15.wig" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.wig"
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        
        expFileName = "%s/Tools/DmelChr4.fa.dsk_k15.gff3" % os.environ["REPET_DATA"]
        obsFileName = "DmelChr4.dsk.gff"
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        
        expLogFileName = "exp.log"
        with open(expLogFileName, "w") as f:
            f.write(" LaunchDSK - WARNING - Warning: ignoring invalid formats: <vero>\n")
            f.write(" LaunchDSK - INFO - Starting to run dsk search of sequence DmelChr4.fa \n")
            f.write("Total RAM: 96679 MB\n")
            f.write("\n")
            f.write("Saved 10760 solid kmers\n")
            f.write(" LaunchDSK - INFO - Finished running dsk scan of sequence DmelChr4.fa \n")
            f.write(" LaunchDSK - INFO - Starting to dump the number of kmer occurence \n")
            f.write(" LaunchDSK - INFO - Starting dumping the number of kmer occurence \n")
            f.write(" LaunchDSK - INFO - Starting to run dsk occurences to map conversion\n")
            f.write(" LaunchDSK - INFO - sorting map data\n")
            f.write(" LaunchDSK - INFO - finished sorting map data\n")
            f.write(" LaunchDSK - INFO - Finished dsk occurences to map conversion\n")
            f.write(" LaunchDSK - INFO - Generating wig file\n")
            f.write(" LaunchDSK - INFO - Generating gff file\n")
        obsLogFileName = "obs.log"
        os.system("cut -d'-' -f4,5,6 %s > %s" % (logFileName, obsLogFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expLogFileName, obsLogFileName))
        
if __name__ == "__main__":
    unittest.main()