import os
import time
import glob
import shutil
import unittest
from commons.launcher.LaunchMCL import LaunchMCL
from commons.core.utils.FileUtils import FileUtils

class Test_F_LaunchMCL(unittest.TestCase):

    def setUp(self):
        self._testDir = os.path.join(os.getcwd(), "test_F_LaunchMCL_%s" % time.strftime("%Y%m%d%H%M%S"))
        os.mkdir(self._testDir)
        os.chdir(self._testDir)
        
        self._inputFileName = "DmelChr4_LTRharvest_expected_with_default_params.fa"
        os.symlink("%s/Tools/%s" % (os.environ["REPET_DATA"], self._inputFileName), self._inputFileName)
        self._outputFileName = "%s_MCL.fa" % os.path.splitext(self._inputFileName)[0]
        
        self._verbosity = 3
        
    def tearDown(self):
        os.chdir(self._testDir)
        os.chdir("..")
        try:
            shutil.rmtree(self._testDir)
        except:
            pass
        
    def test_run(self):
        iLaunchMCL = LaunchMCL(self._inputFileName, doClean = False, verbosity = self._verbosity)
        iLaunchMCL.run()
        expFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_expected.fa" % os.environ["REPET_DATA"]
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self._outputFileName))
         
    def test_run_with_map_file(self):
        iLaunchMCL = LaunchMCL(self._inputFileName, isCluster2Map = True, doClean = False, verbosity = self._verbosity)
        iLaunchMCL.run()
        expFastaFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_expected.fa" % os.environ["REPET_DATA"]
        expMapFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_expected.map" % os.environ["REPET_DATA"]
        obsMapFileName = "%s.map" % os.path.splitext(self._outputFileName)[0]
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFastaFileName, self._outputFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expMapFileName, obsMapFileName))
         
    def test_run_with_map_file_changed_coverage_threshold(self):
        iLaunchMCL = LaunchMCL(self._inputFileName, isCluster2Map = True, doClean = False, verbosity = self._verbosity)
        iLaunchMCL.setCoverageThreshold(0.06)
        iLaunchMCL.run()
        expFastaFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_cov5perc_expected.fa" % os.environ["REPET_DATA"]
        expMapFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_cov5perc_expected.map" % os.environ["REPET_DATA"]
        obsMapFileName = "%s.map" % os.path.splitext(self._outputFileName)[0]
        
        self.assertTrue(FileUtils.are2FilesIdentical(expFastaFileName, self._outputFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expMapFileName, obsMapFileName))
        
    def test_run_clusterConsensus_headers(self):
        os.remove(self._inputFileName)
        self._inputFileName = "DmelChr4_LTRharvest_MCL_ClusterConsHeaders_input.fa"
        os.symlink("%s/Tools/%s" % (os.environ["REPET_DATA"], self._inputFileName), self._inputFileName)
        iLaunchMCL = LaunchMCL(self._inputFileName, isClusterConsensusHeaders = True, doClean = True, verbosity = self._verbosity)
        iLaunchMCL.run()
        expFastaFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_ClusterConsHeaders_expected.fa" % os.environ["REPET_DATA"]
        self._outputFileName = "%s_MCL.fa" % os.path.splitext(self._inputFileName)[0]
        self.assertTrue(FileUtils.are2FilesIdentical(expFastaFileName, self._outputFileName))

    def test_run_as_script(self):
        cmd = "LaunchMCL.py -i %s -j -v 6 -v %d" % (self._inputFileName, self._verbosity)
        os.system(cmd)
        expFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_expected.fa" % os.environ["REPET_DATA"]
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self._outputFileName))
        self.assertTrue(len(glob.glob("MCLtmpDirectory*")) == 1)

    def test_run_as_script_allParams(self):
        cmd = "LaunchMCL.py -i %s -j -I 1.5 -T 0.0 -m -v %d" % (self._inputFileName, self._verbosity)
        os.system(cmd)
        expFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_expected.fa" % os.environ["REPET_DATA"]
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self._outputFileName))
        self.assertTrue(len(glob.glob("MCLtmpDirectory*")) == 1)

    def test_run_as_script_without_join(self):
        cmd = "LaunchMCL.py -i %s -v %d" % (self._inputFileName, self._verbosity)
        os.system(cmd)
        expFileName = "%s/Tools/DmelChr4_LTRharvest_MCL_expected.fa" % os.environ["REPET_DATA"]
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, self._outputFileName))
        self.assertTrue(len(glob.glob("MCLtmpDirectory*")) == 1)
        
if __name__ == "__main__":
    unittest.main()