import os
import time
import shutil
import unittest
from commons.launcher.LaunchQuast import LaunchQuast
from commons.core.utils.FileUtils import FileUtils
from commons.core.checker.RepetException import RepetException

class Test_F_LaunchQuast(unittest.TestCase):
    
    def setUp(self):
        self._curTestDir = os.getcwd()
        self.testFolder = "test_F_LaunchQuast_%s" % time.strftime("%Y%m%d%H%M%S")
        
        self.inputAssemblyFilePath = "%s/commons/DmelChr4_contigs.fa" % os.environ["REPET_DATA"]
        self.inputAssemblyFileName = os.path.basename(self.inputAssemblyFilePath)
        
        self.inputReferenceFilePath = "%s/commons/DmelChr4.fa" % os.environ["REPET_DATA"]
        self.inputReferenceFileName = os.path.basename(self.inputReferenceFilePath)
        
        self.expReportFileName = "expQuastReport.txt"
        
        os.makedirs(self.testFolder)
        os.chdir(self.testFolder)
        
        os.symlink(self.inputAssemblyFilePath, self.inputAssemblyFileName)
        os.symlink(self.inputReferenceFilePath, self.inputReferenceFileName)
    
    def tearDown(self):
        os.chdir(self._curTestDir)
        try:
            shutil.rmtree(self.testFolder)
        except:pass

    def test_run(self):
        iLaunchQuast = LaunchQuast(assemblyFileName = self.inputAssemblyFileName, referenceFileName = self.inputReferenceFileName, minContigSize = 200, nbThreads = 2, verbosity = 5)
        obsQuastStats = iLaunchQuast.run()
        expQuastStats = {"N50": 505, "NG50": 370, "coverage": 71.895}
        self.assertEquals(obsQuastStats, expQuastStats)

    def test_run_without_reference(self):
        iLaunchQuast = LaunchQuast(assemblyFileName = self.inputAssemblyFileName, minContigSize = 200, nbThreads = 2, verbosity = 5)
        obsQuastStats = iLaunchQuast.run()
        expQuastStats = {"N50": 505}
        self.assertEquals(obsQuastStats, expQuastStats)
        
    def test_runAsScript(self):
        self._writeExpReportFile()
        cmd = "LaunchQuast.py -a %s -r %s -m 200 -t 2 -v 5" % (self.inputAssemblyFileName, self.inputReferenceFileName)
        os.system(cmd)
        self.assertTrue(FileUtils.are2FilesIdentical(self.expReportFileName, "quast_results/latest/report.txt"))
        
    def test_run_inconsistent_deletions_sizes(self):
        iLaunchQuast = LaunchQuast()
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            iLaunchQuast.run()
        except RepetException as e:
            obsMessage = e.getMessage()
            isExceptionRaised = True
            
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, "ERROR: Missing input assembly file name")
        
    def _writeExpReportFile(self):
        #Good expected for QUAST 2.2
        with open(self.expReportFileName,'w') as f:
            f.write("All statistics are based on contigs of size >= 200 bp, unless otherwise noted (e.g., \"# contigs (>= 0 bp)\" and \"Total length (>= 0 bp)\" include all contigs).\n")
            f.write("\n")
            f.write("Assembly                     DmelChr4_contigs\n")
            f.write("# contigs (>= 0 bp)          3369            \n")
            f.write("# contigs (>= 1000 bp)       84              \n")
            f.write("Total length (>= 0 bp)       1125642         \n")
            f.write("Total length (>= 1000 bp)    105254          \n")
            f.write("# contigs                    2117            \n")
            f.write("Total length                 938562          \n")
            f.write("Largest contig               2233            \n")
            f.write("Reference length             1281640         \n")
            f.write("GC (%)                       35.09           \n")
            f.write("Reference GC (%)             35.14           \n")
            f.write("N50                          505             \n")
            f.write("NG50                         370             \n")
            f.write("N75                          331             \n")
            f.write("L50                          627             \n")
            f.write("LG50                         1024            \n")
            f.write("L75                          1204            \n")
            f.write("# misassemblies              1               \n")
            f.write("Misassembled contigs length  1240            \n")
            f.write("# local misassemblies        2               \n")
            f.write("# unaligned contigs          0 + 2 part      \n")
            f.write("Unaligned length             118             \n")
            f.write("Genome fraction (%)          71.895          \n")
            f.write("Duplication ratio            1.019           \n")
            f.write("# N's per 100 kbp            0.00            \n")
            f.write("# mismatches per 100 kbp     196.87          \n")
            f.write("# indels per 100 kbp         1.19            \n")
            f.write("Largest alignment            2233            \n")
            f.write("NA50                         505             \n")
            f.write("NGA50                        370             \n")
            f.write("NA75                         331             \n")
            f.write("LA50                         628             \n")
            f.write("LGA50                        1026            \n")
            f.write("LA75                         1206            \n")
        
if __name__ == "__main__":
    unittest.main()