#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
import shutil
import subprocess
from glob import glob
from commons.core.LoggerFactory import LoggerFactory
from commons.core.checker.ConfigChecker import ConfigChecker
from commons.core.checker.ConfigChecker import ConfigRules
from commons.core.launcher.Launcher import Launcher
from commons.core.seq.FastaUtils import FastaUtils
from commons.core.sql.DbFactory import DbFactory
from commons.core.sql.TableJobAdaptatorFactory import TableJobAdaptatorFactory
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser

LOG_DEPTH = "commons.tools"
LOG_FORMAT = "%(message)s"

## Detect TE features
#
class DetectTEFeatures_parallelized(object):
    
    def __init__(self, fastaFileName = "", projectName = "", configFileName = "", doClean = False, verbosity = 0):
        self._fastaFileName = fastaFileName
        self._projectName = projectName
        self._configFileName = configFileName
        self._doClean = doClean
        self._verbosity = verbosity
        self._log = LoggerFactory.createLogger("%s.%s" % (LOG_DEPTH, self.__class__.__name__), self._verbosity + 2, LOG_FORMAT)
        self._doTerminalRepeatSearch = True
        self._doTandemRepeatSearch = True
        self._doPolyAtailSearch = True
        self._doORFSearch = True
        self._doHmmer = True
        self._doTEBLRn = True
        self._doTEBLRtx = True
        self._doTEBLRx = True
        self._doHGBLRn = True
        self._dorDNABLRn = True
        self._dotRNASearch = True
        #TODO: doTEclass ? was difficult to install...
        self._blastType = "ncbi"
        self._profilesBank = ""
        self._knownTEnuclBank = ""
        self._knownTEprotBank = ""
        self._HGbank = ""
        self._rDNAbank = ""
        self._maxProfilesEvalue = "10"
        self._sectionName = "detect_features"
        self._resources = ""
        self._tmpDir = ""
        
    def setAttributesFromCmdLine(self):
        usage = "DetectTEFeatures_parallelized.py [options]"
        description = "Detect TE features on the input sequences file (fasta format).\n"
        description += "These features are terminal repeats, tandem repeats, poly A tail, ORF, profiles, TE in banks, rDNA, tRNA etc...\n"
        description += "Launch all detections in parallel\n"
        epilog = "\nExample 1: launch with default parameters\n"
        epilog += "\t$ DetectTEFeatures_parallelized.py -i consensus.fa -P project -C TEclassifier.cfg"
        epilog += "\n"
        #WARNING: blast coverage is different if it's not parallelized
        parser = RepetOptionParser(description = description, epilog = epilog, usage = usage)
        parser.add_option("-i", "--fasta",      dest = "fastaFileName", action = "store",       type = "string", help = "input fasta file name [compulsory] [format: fasta]", default = "")
        parser.add_option("-P", "--project",    dest = "projectName",   action = "store",       type = "string", help = "project name [compulsory] - not from config", default = "")
        parser.add_option("-C", "--config",     dest = "configFileName",action = "store",       type = "string", help = "configuration file name (e.g. TEdenovo.cfg)", default = "")
        parser.add_option("-c", "--clean",      dest = "doClean",       action = "store_true",                   help = "clean temporary files [optional] [default: False]", default = False)
        parser.add_option("-v", "--verbosity",  dest = "verbosity",     action = "store",       type = "int",    help = "verbosity [optional] [default: 1]", default = 1)
        options = parser.parse_args()[0]
        self._setAttributesFromOptions(options)
        
    def _setAttributesFromOptions(self, options):
        self.setFastaFileName(options.fastaFileName)
        self.setProjectName(options.projectName)
        self.setConfigFileName(options.configFileName)
        self.setDoClean(options.doClean)
        self.setVerbosity(options.verbosity)

    def _checkConfig(self):
        iConfigRules = ConfigRules()
        iConfigRules.addRuleSection(section=self._sectionName, mandatory=True)
        iConfigRules.addRuleOption(section=self._sectionName, option ="clean", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="resources", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="tmpDir", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="term_rep", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="tand_rep", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="polyA", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="orf", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_HMMER", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_BLRn", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_BLRtx", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_BLRx", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="HG_BLRn", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="rDNA_BLRn", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="blast", mandatory=True, type="string", set = ("ncbi", "blastplus", "wu"))
        iConfigRules.addRuleOption(section=self._sectionName, option ="tRNA_scan", mandatory=True, type="bool")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_nucl_bank", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_prot_bank", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="HG_nucl_bank", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="rDNA_bank", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_HMM_profiles", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TE_HMMER_evalue", mandatory=True, type="string")
        iConfigRules.addRuleOption(section=self._sectionName, option ="TRFmaxPeriod", mandatory=True, type="int")
#                    process = subprocess.Popen("blastn -h", shell=True, stdout=subprocess.PIPE)
#                    output = process.communicate()[0]
#                    if "washu" in output.lower():
#                        print "ERROR: you are trying to use NCBI BlastPlus, but the blast executable is pointing to WuBlast. Please check your system path."
#                        sys.exit(1)                 
        iConfigChecker = ConfigChecker(self._configFileName, iConfigRules)
        iConfig = iConfigChecker.getConfig()
        self._setAttributesFromConfig(iConfig)
        
    def _setAttributesFromConfig(self, iConfig):
        self.setDoClean(iConfig.get(self._sectionName, "clean"))
        self._resources = iConfig.get(self._sectionName, "resources")
        self._tmpDir = iConfig.get(self._sectionName, "tmpDir")
        self._doTerminalRepeatSearch = iConfig.get(self._sectionName, "term_rep")
        self._doTandemRepeatSearch = iConfig.get(self._sectionName, "tand_rep")
        self._doPolyAtailSearch = iConfig.get(self._sectionName, "polyA")
        self._doORFSearch = iConfig.get(self._sectionName, "orf")
        self._doHmmer = iConfig.get(self._sectionName, "TE_HMMER")
        self._doTEBLRn = iConfig.get(self._sectionName, "TE_BLRn")
        self._doTEBLRtx = iConfig.get(self._sectionName, "TE_BLRtx")
        self._doTEBLRx = iConfig.get(self._sectionName, "TE_BLRx")
        self._doHGBLRn = iConfig.get(self._sectionName, "HG_BLRn")
        self._dorDNABLRn = iConfig.get(self._sectionName, "rDNA_BLRn")
        self._dotRNASearch = iConfig.get(self._sectionName, "tRNA_scan")
        self._blastType = iConfig.get(self._sectionName, "blast")
        
        if self._doTEBLRn or self._doTEBLRtx:
            self._knownTEnuclBank = iConfig.get(self._sectionName, "TE_nucl_bank")
            self._knownTEnuclBank, lBanksNT = FileUtils.concatenateFileNamesFromString(self._knownTEnuclBank, splitSep = ",", joinSep = "_")
            if len(lBanksNT) > 1:
                FileUtils.catFilesFromList(lBanksNT, self._knownTEnuclBank)
        
        if self._doTEBLRx:
            self._knownTEprotBank = iConfig.get(self._sectionName, "TE_prot_bank")
            self._knownTEprotBank, lBanksAA = FileUtils.concatenateFileNamesFromString(self._knownTEprotBank, splitSep = ",", joinSep = "_")
            if len(lBanksAA) > 1:
                FileUtils.catFilesFromList(lBanksAA, self._knownTEprotBank)
        
        self._HGbank = iConfig.get(self._sectionName, "HG_nucl_bank")
        self._rDNAbank = iConfig.get(self._sectionName, "rDNA_bank")
        
        if self._doHmmer:
            self._profilesBank = iConfig.get(self._sectionName, "TE_HMM_profiles")
            self._profilesBank, lProfilesBanks = FileUtils.concatenateFileNamesFromString(self._profilesBank, splitSep = ",", joinSep = "_")
            if len(lProfilesBanks) > 1:
                FileUtils.catFilesFromList(lProfilesBanks, self._profilesBank)
        
        self._maxProfilesEvalue = iConfig.get(self._sectionName, "TE_HMMER_evalue")
        self._TRFmaxPeriod=iConfig.get(self._sectionName, "TRFmaxPeriod")
    
    def setFastaFileName(self, fastaFileName):
        self._fastaFileName = fastaFileName
        
    def setProjectName(self, projectName):
        self._projectName = projectName
        
    def setConfigFileName(self, configFileName):
        self._configFileName = configFileName
        
    def setDoClean(self, doClean):
        self._doClean = doClean
        
    def setVerbosity(self, verbosity):
        self._verbosity = verbosity
        
    def _checkOptions(self):
        if self._configFileName == "" or not FileUtils.isRessourceExists(self._configFileName):
            self._logAndRaise("ERROR: Missing configuration file")
        if self._fastaFileName == "" or not FileUtils.isRessourceExists(self._fastaFileName):
            self._logAndRaise("ERROR: Missing input fasta file")
            
    def _logAndRaise(self, errorMsg):
        self._log.error(errorMsg)
        raise Exception(errorMsg)
        
    def run(self):
        LoggerFactory.setLevel(self._log, self._verbosity + 2)
        self._checkOptions()
        self._checkConfig()
        self._log.info("START DetectTEFeatures_parallelized")
        self._log.debug("Fasta file name: %s" % self._fastaFileName)
        
        self._log.info("split the sequences in batches")
        FastaUtils.splitFastaFileInBatches(self._fastaFileName, 4000)
        
        cDir = os.getcwd()
        if not self._tmpDir:
            self._tmpDir = cDir
        self._log.info("launch the programs on each batch")
        
        groupid = "%s_detectTEFeaturesParallelized" % self._projectName
        acronym = "DetectFeaturesParallelized"
        iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, cDir, self._tmpDir, self._resources, groupid, acronym)
        lCmdsTuples = []
        
        lFiles = glob("%s/batches/*" % os.getcwd())
        for fileName in lFiles:
            cmd_start, cmd_finish = self._detectFeatures(fileName, cDir)
            lCmds = [cmd_start]
            lCmdFinish = [cmd_finish]
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds = lCmds, lCmdFinish = lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._doClean)
         
        if self._doClean:
            lFilesToRemove = ["formatdb.log", "last_time_stamp.log", "%s.h3f" % self._profilesBank, "%s.h3i" % self._profilesBank, "%s.h3m" % self._profilesBank, "%s.h3p" % self._profilesBank]
            FileUtils.removeFilesFromListIfExist(lFilesToRemove)
            shutil.rmtree("batches")
        
        self._log.info("collect the results and load them into tables")
        self._collectResults()
        
        if not self._checkIfFeaturesTablesExist():
            self._logAndRaise("ERROR: no table recording TE features exists")
            
        self._log.info("END DetectTEFeatures_parallelized")
        
    def _detectFeatures(self, inFileName, cDir):
        prefix = os.path.basename(inFileName)
        cmd_start = ""
        cmd_start += "if not os.path.exists( \"%s\" ):\n" % prefix
        cmd_start += "\tlog = os.system( \"ln -s %s .\" )\n" % inFileName
        cmd_start += "\tif log != 0:\n"
        cmd_start += "\t\traise Exception(\"ERROR: can't link '%s' in temporary directory\")\n" % inFileName
        cmd_finish = ""
        cmd_finish += "if os.path.exists( \"%s/%s\" ):\n" % (self._tmpDir, prefix)
        cmd_finish += "\tos.system( \"rm -f %s/%s\" )\n" % (self._tmpDir, prefix)
        
        launch_1 = "log = os.system( \""
        launch_2 = "\" )\n"
        launch_2 += "if log != 0:\n"
        launch_2 += "\traise Exception(\"ERROR in job : 'os.system' return \" + str(log))\n"
    
        if self._doTerminalRepeatSearch:
            cmd_start += self._detectTRsearch(prefix, launch_1, launch_2, cDir)
        if self._doPolyAtailSearch:
            cmd_start += self._detectPolyA(prefix, launch_1, launch_2, cDir)
        if self._doTandemRepeatSearch:
            cmd_start += self._detectTRF(prefix, launch_1, launch_2, cDir)
        if self._doORFSearch:
            cmd_start += self._detectORFs(prefix, launch_1, launch_2, cDir)
        if self._dotRNASearch:
            cmd_start += self._detectTRNA(prefix, launch_1, launch_2, cDir)
        if self._doTEBLRn:
            cmd_start += self._detectBlaster(prefix, self._knownTEnuclBank, "blastn", "TE_BLRn", launch_1, launch_2, cDir)
        if self._doTEBLRtx:
            cmd_start += self._detectBlaster(prefix, self._knownTEnuclBank, "tblastx", "TE_BLRtx", launch_1, launch_2, cDir)
        if self._doTEBLRx:
            cmd_start += self._detectBlaster(prefix, self._knownTEprotBank, "blastx", "TE_BLRx", launch_1, launch_2, cDir)
        if self._doHGBLRn:
            cmd_start += self._detectBlaster(prefix, self._HGbank, "blastn", "HG_BLRn", launch_1, launch_2, cDir)
        if self._dorDNABLRn:
            cmd_start += self._detectBlaster(prefix, self._rDNAbank, "blastn", "rDNA_BLRn", launch_1, launch_2, cDir)
        if self._doHmmer:
            cmd_start += self._detectHmmProfiles(prefix, launch_1, launch_2, cDir)
        return cmd_start, cmd_finish
    
    def _detectTRsearch(self, inFileName, launch_1, launch_2, cDir):
        cmd = ""
        cmd += launch_1
        cmd += "TRsearch"
        cmd += " %s" % inFileName
        cmd += launch_2
        cmd += "if not os.path.exists( \"%s/%s.TR.set\" ):\n" % (cDir, inFileName)
        cmd += "\tos.system( \"mv %s.TR.set %s\" )\n" % (inFileName, cDir)
        return cmd
    
    def _detectPolyA(self, inFileName, launch_1, launch_2, cDir):
        cmd = ""
        cmd += launch_1
        cmd += "polyAtail"
        cmd += " %s" % inFileName
        cmd += launch_2
        cmd += "if not os.path.exists( \"%s/%s.polyA.set\" ):\n" % (cDir, inFileName)
        cmd += "\tos.system( \"mv %s.polyA.set %s\" )\n" % (inFileName, cDir)
        return cmd
    
    def _detectTRF(self, inFileName, launch_1, launch_2, cDir):
        cmd = ""
        cmd += launch_1
        cmd += "LaunchTRF.py"
        cmd += " -i %s" % inFileName
        cmd += " -o %s.SSR.set" % inFileName
        cmd += " -m %d" % self._TRFmaxPeriod
        cmd += " -c"
        cmd += " -v 1"
        cmd += launch_2
        cmd += "if not os.path.exists( \"%s/%s.SSR.set\" ):\n" % (cDir, inFileName)
        cmd += "\tos.system( \"mv %s.SSR.set %s\" )\n" % (inFileName, cDir)
        return cmd
    
    def _detectORFs(self, inFileName, launch_1, launch_2, cDir):
        cmd = ""
        cmd += "from commons.core.seq.FastaUtils import FastaUtils\n"
        cmd += "FastaUtils.dbORF(\"%s\", 10, 30, verbose=%s)\n" % (inFileName, self._verbosity)
        cmd += "if not os.path.exists( \"%s/%s.ORF.map\" ):\n" % (cDir, inFileName)
        cmd += "\tos.system( \"mv %s.ORF.map %s\" )\n" % (inFileName, cDir)
        return cmd
    
    def _detectBlaster(self, inFileName, bank, blast, acronym, launch_1, launch_2, cDir):
        if not os.path.exists("%s_cut" % bank):
            self._log.info("prepare bank '%s'..." % bank)
            prg = "blaster"
            cmd = prg
            cmd += " -s %s" % bank
            cmd += " -n %s" % blast
            if self._blastType == "ncbi": 
                cmd += " -N"
                cmd += " -p '-a 1'"
            elif self._blastType == "wu": 
                cmd += " -W"
                cmd += " -p -cpus=1"
            elif self._blastType == "blastplus":
                cmd += " -X"
                cmd += " -p '-num_threads 1'"
            cmd += " -r"
            cmd += " -P"
            cmd += " -v %i >> prepareBanksForBlaster.log" % (self._verbosity - 1)
            process = subprocess.Popen(cmd, shell = True)
            process.communicate()
            if process.returncode != 0:
                self._logAndRaise("ERROR when launching '%s'" % cmd)
            FileUtils.removeFilesByPattern("%s-%s-*.param" % (bank, blast))
            
        cmd = ""
        cmd += launch_1
        cmd += "blaster"
        cmd += " -q %s" % inFileName
        cmd += " -s %s/%s" % (cDir, bank)
        cmd += " -B %s_%s_%s" % (inFileName, acronym, bank)
        cmd += " -n %s" % blast
        if self._blastType == "ncbi": 
            cmd += " -N"
            cmd += " -p '-a 1'"
        elif self._blastType == "wu": 
            cmd += " -W"
            cmd += " -p -cpus=1"
        elif self._blastType == "blastplus":
            cmd += " -X"
            cmd += " -p '-num_threads 1'"
        cmd += " -r"
        cmd += " -v 1"
        cmd += launch_2
        
        cmd += "if not os.path.exists( \"%s/%s_%s_%s.param\" ):\n" % (cDir, inFileName, acronym, bank)
        cmd += "\tos.system( \"mv %s_%s_%s.param %s\" )\n" % (inFileName, acronym, bank, cDir)
        
        cmd += launch_1
        cmd += "matcher"
        cmd += " -m %s_%s_%s.align" % (inFileName, acronym, bank)
        #TODO: '.tab' files ('match' tables) are unused for now
#        cmd += " -q %s" % inFileName
#        cmd += " -s %s/%s" % (cDir, bank)
        cmd += " -j"
        #TODO: clean conflicts AFTER 'join' procedure instead of BEFORE ?
#            cmd += " -x"
        cmd += " -v 1"
        cmd += launch_2
        
        cmd += "if not os.path.exists( \"%s/%s_%s_%s.align.clean_match.path\" ):\n" % (cDir, inFileName, acronym, bank)
        cmd += "\tos.system( \"mv %s_%s_%s.align.clean_match.path %s\" )\n" % (inFileName, acronym, bank, cDir)
        cmd += "if not os.path.exists( \"%s/%s_%s_%s.align.clean_match.param\" ):\n" % (cDir, inFileName, acronym, bank)
        cmd += "\tos.system( \"mv %s_%s_%s.align.clean_match.param %s\" )\n" % (inFileName, acronym, bank, cDir)
        return cmd
    
    def _detectHmmProfiles(self, inFileName, launch_1, launch_2, cDir):
        #NOTE: hmmer2 no more handled
        #TODO: is it useful to prepare bank in master node ? Is it used by jobs ? As for blaster, should it be better to prepare bank on each node ?
        if not (os.path.exists("%s.h3m" % self._profilesBank) \
            and os.path.exists("%s.h3i" % self._profilesBank) \
            and os.path.exists("%s.h3f" % self._profilesBank) \
            and os.path.exists("%s.h3p" % self._profilesBank)) :
            self._log.info("prepare bank '%s'..." % self._profilesBank)
            cmd = "hmmpress"
            cmd += " -f"
            cmd += " %s/%s" % (cDir, self._profilesBank)
            cmd += " > prepareProfilesBank.log"
            process = subprocess.Popen(cmd, shell = True)
            process.communicate()
            if process.returncode != 0:
                raise Exception("ERROR when launching '%s'" % cmd)

        cmd = ""
        cmd += "from commons.core.seq.BioseqUtils import BioseqUtils\n"
        cmd += "lBioseqs = BioseqUtils.extractBioseqListFromFastaFile('%s')\n" % inFileName
        cmd += "lTranslatedBioseqs = BioseqUtils.translateBioseqListInAllFrames(lBioseqs)\n"
        cmd += "lTranslatedBioseqsWithoutStop = BioseqUtils.replaceStopCodonsByXInBioseqList(lTranslatedBioseqs)\n"
        cmd += "BioseqUtils.writeBioseqListIntoFastaFile(lTranslatedBioseqsWithoutStop, '%s_translated')\n" % inFileName
        cmd += "\n"
        
        cmd += launch_1
        cmd += "hmmscan"
        cmd += " -o %s_tr.hmmScanOut" % inFileName
        cmd += " --domtblout %s_tr.hmmScanOutTab" % inFileName
        cmd += " --noali -E %s" % self._maxProfilesEvalue
        cmd += " --cpu 1"
        cmd += " %s/%s" % (cDir, self._profilesBank)
        cmd += " %s_translated" % inFileName
        cmd += launch_2
        
        cmd += launch_1
        cmd += os.environ["REPET_PATH"] + "/bin/HmmOutput2alignAndTransformCoordInNtAndFilterScores_script.py"
        cmd += " -i %s_tr.hmmScanOutTab" % inFileName
        cmd += " -o %s_Profiles_%s.align" % (inFileName, self._profilesBank)
        cmd += " -T %s" % inFileName
        cmd += " -p hmmscan"
        cmd += " -c"
        cmd += launch_2
        
        cmd += launch_1
        cmd += "matcher"
        cmd += " -m %s_Profiles_%s.align" % (inFileName, self._profilesBank)
        cmd += " -j"
        cmd += " -E 10"
        cmd += " -L 0"
        cmd += " -v 1"
        cmd += launch_2
        
        cmd += "if not os.path.exists( \"%s/%s_Profiles_%s.align.clean_match.path\" ):\n" % (cDir, inFileName, self._profilesBank)
        cmd += "\tos.system( \"mv %s_Profiles_%s.align.clean_match.path %s\" )\n" % (inFileName, self._profilesBank, cDir)
        cmd += "if not os.path.exists( \"%s/%s_Profiles_%s.align.clean_match.param\" ):\n" % (cDir, inFileName, self._profilesBank)
        cmd += "\tos.system( \"mv %s_Profiles_%s.align.clean_match.param %s\" )\n" % (inFileName, self._profilesBank, cDir)
        return cmd
    
    def _detectTRNA(self, inFileName, launch_1, launch_2, cDir):
        cmd = ""
        cmd += launch_1
        cmd += "LaunchtRNAscanSE.py"
        cmd += " -Q"
        cmd += " -i %s" % inFileName
        cmd += " -v 1"
        cmd += launch_2
        cmd += "shutil.move('%s.tRNA.map', '%s')\n" % (inFileName, cDir)    
        return cmd
            
    def _createDirectory(self, dirName):
        self._log.info("collect results '%s'" % dirName)
        if os.path.exists(dirName):
            shutil.rmtree(dirName)
        os.mkdir(dirName)
            
    def _collectResults(self):
        if self._doTerminalRepeatSearch:
            self._collectStructFeatures("TR", "set")
        if self._doPolyAtailSearch:
            self._collectStructFeatures("polyA", "set")
        if self._doTandemRepeatSearch:
            self._collectStructFeatures("SSR", "set")
        if self._doORFSearch:
            self._collectStructFeatures("ORF", "map")
        if self._dotRNASearch:
            self._collectStructFeatures("tRNA", "map")
        if self._doHmmer:
            self._collectHomologyFeatures("Profiles", self._profilesBank)
        if self._doTEBLRn:
            self._collectHomologyFeatures("TE_BLRn", self._knownTEnuclBank)
        if self._doTEBLRtx:
            self._collectHomologyFeatures("TE_BLRtx", self._knownTEnuclBank)
        if self._doTEBLRx:
            self._collectHomologyFeatures("TE_BLRx", self._knownTEprotBank)
        if self._doHGBLRn:
            self._collectHomologyFeatures("HG_BLRn", self._HGbank)
        if self._dorDNABLRn:
            self._collectHomologyFeatures("rDNA_BLRn", self._rDNAbank)
        
    def _collectStructFeatures(self, analysis, dataType):
        self._createDirectory(analysis)
        os.chdir(analysis)
        FileUtils.catFilesByPattern("../batch_*.fa.%s.%s" % (analysis, dataType), "%s.%s.%s.tmp" % (self._projectName, analysis, dataType))
    
        if dataType == "set":
            prg = "setnum2id"
            cmd = prg
            cmd += " -i %s.%s.set.tmp" % (self._projectName, analysis)
            cmd += " -o %s.%s.set" % (self._projectName, analysis)
            cmd += " -v %i" % (self._verbosity - 3)
            process = subprocess.Popen(cmd, shell = True)
            process.communicate()
            if process.returncode != 0:
                self._logAndRaise("ERROR: '%s' return %i" % (cmd, process.returncode))
            os.remove("%s.%s.set.tmp" % (self._projectName, analysis))
        else:
            shutil.move("%s.%s.%s.tmp" % (self._projectName, analysis, dataType), "%s.%s.%s" % (self._projectName, analysis, dataType))
    
        iDb = DbFactory.createInstance()
        iDb.createTable("%s_%s_%s" % (self._projectName, analysis, dataType), dataType, "%s.%s.%s" % (self._projectName, analysis, dataType), True)
        iDb.close()

        os.chdir("..")
        if self._doClean:
            cmd = "find . -maxdepth 1 -name \"batch_*.fa.%s.%s\" -type f -delete" % (analysis, dataType)
            process = subprocess.Popen(cmd, shell = True)
            process.communicate()
            if process.returncode != 0:
                self._logAndRaise("ERROR: '%s' return %i" % (cmd, process.returncode))
    
    def _collectHomologyFeatures(self, analysis, bank):
        self._createDirectory(analysis)
        os.chdir(analysis)
        FileUtils.catFilesByPattern("../batch_*.fa_%s_%s.align.clean_match.path" % (analysis, bank),
                                     "%s_%s_%s.align.clean_match.path.tmp" % (self._projectName, analysis, bank))
        prg = "pathnum2id"
        cmd = prg
        cmd += " -i %s_%s_%s.align.clean_match.path.tmp" % (self._projectName, analysis, bank)
        cmd += " -o %s_%s_%s.align.clean_match.path" % (self._projectName, analysis, bank)
        cmd += " -v %i" % (self._verbosity - 3)
        process = subprocess.Popen(cmd, shell = True)
        process.communicate()
        if process.returncode != 0:
            self._logAndRaise("ERROR: '%s' return %i" % (cmd, process.returncode))

        iDb = DbFactory.createInstance()
        iDb.createTable("%s_%s_path" % (self._projectName, analysis), "path", "%s_%s_%s.align.clean_match.path" % (self._projectName, analysis, bank), True)
        iDb.close()
 
        os.chdir( ".." )
        if self._doClean:
            lFilesToRemove = ["%s/%s_%s_%s.align.clean_match.path.tmp" % (analysis, self._projectName, analysis, bank), \
                              "%s/%s_%s_%s.align.clean_match.tab.tmp" % (analysis, self._projectName, analysis, bank), \
                              "prepareProfilesBank.log", \
                              "prepareBanksForBlaster.log", \
                              "%s.Nstretch.map" % bank]
            FileUtils.removeFilesFromListIfExist(lFilesToRemove)
            FileUtils.removeFilesByPattern("%s_cut*" % bank)
            cmd = "find . -maxdepth 1 -name \"batch_*.fa_%s_%s.*\" -type f -delete" % (analysis, bank)
            process = subprocess.Popen(cmd, shell = True)
            process.communicate()
            if process.returncode != 0:
                self._logAndRaise("ERROR: '%s' return %i" % (cmd, process.returncode))
                
    def _checkIfFeaturesTablesExist(self):
        createdTablesNb = 0
        tableSuffix = ["_TR_set",
                       "_SSR_set",
                       "_polyA_set",
                       "_ORF_map",
                       "_Profiles_path",
                       "_TE_BLRn_path",
                       "_TE_BLRtx_path",
                       "_TE_BLRx_path",
                       "_HG_BLRn_path",
                       "_rDNA_BLRn_path",
                       "_tRNA_map"]
        iDb = DbFactory.createInstance()
        for suffix in tableSuffix:
            if iDb.doesTableExist(self._projectName + suffix):
                self._log.info("table '%s%s' exists (%i lines)" % (self._projectName, suffix, iDb.getSize(self._projectName + suffix)))
                createdTablesNb += 1
            else:
                self._log.warning("table '%s%s' doesn't exist" % (self._projectName, suffix))
        iDb.close()
        return createdTablesNb

if __name__ == "__main__":
    iLaunch = DetectTEFeatures_parallelized()
    iLaunch.setAttributesFromCmdLine()
    iLaunch.run()
