#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import os
from commons.core.LoggerFactory import LoggerFactory
from commons.core.checker.RepetException import RepetException
from commons.core.utils.RepetOptionParser import RepetOptionParser
from commons.core.utils.FileUtils import FileUtils

LOG_DEPTH = "repet.tools"

##FilterVCF filters a VCF file on indels sizes, and may remove SNPs
#
class FilterVCF(object):
    
    def __init__(self, inputVCF = "", filteredVCF = "", minInsSize = -1, maxInsSize = -1, minDelSize = -1, maxDelSize = -1, removeSNP = False, verbosity = 3):
        self._inputVCF = inputVCF
        self.setFilteredVCF(filteredVCF)
        self._minInsSize = minInsSize
        self._maxInsSize = maxInsSize
        self._minDelSize = minDelSize
        self._maxDelSize = maxDelSize
        self._removeSNP = removeSNP
        self._verbosity = verbosity
        self._log = LoggerFactory.createLogger("%s.%s" % (LOG_DEPTH, self.__class__.__name__), self._verbosity)
        
    def setAttributesFromCmdLine(self):
        usage = "FilterVCF.py [options]"
        description = "Launch filterVCF to filter a VCF file on indels sizes.\n"
        epilog = "\nExample 1: remove all SNPs, without verbosity.\n"
        epilog += "\t$ FilterVCF.py -f input.vcf -S -v 0"
        epilog += "\nExample 2: launch with ins-del >= 100 and <= 200.\n"
        epilog += "\t$ FilterVCF.py -f input.vcf -i 100 -I 200 -d 100 -D 200 -v 3\n"
        parser = RepetOptionParser(description = description, epilog = epilog, usage = usage)
        parser.add_option("-f", "--filename",   dest = "inputVCF",    action = "store",      type = "string", help = "input vcf file name [compulsory] [format: VCF]", default = "")
        parser.add_option("-o", "--out",        dest = "filteredVCF", action = "store",      type = "string", help = "output file name [optional] [default: <input>_filtered.vcf]", default = "")
        parser.add_option("-i", "--minIns",     dest = "minInsSize",  action = "store",      type = "int",    help = "minimum insertion size, -1 not to filter [optional] [default: -1]", default = -1)
        parser.add_option("-I", "--maxIns",     dest = "maxInsSize",  action = "store",      type = "int",    help = "maximum insertion size, -1 not to filter [optional] [default: -1]", default = -1)
        parser.add_option("-d", "--minDelSize", dest = "minDelSize",  action = "store",      type = "int",    help = "minimum deletion size, -1 not to filter [optional] [default: -1]", default = -1)
        parser.add_option("-D", "--maxDelSize", dest = "maxDelSize",  action = "store",      type = "int",    help = "maximum deletion size, -1 not to filter [optional] [default: -1]", default = -1)
        parser.add_option("-S", "--removeSNP",  dest = "removeSNP",   action = "store_true",                  help = "remove SNPs [optional] [default: False]", default = False)
        parser.add_option("-v", "--verbosity",  dest = "verbosity",   action = "store",      type = "int",    help = "verbosity [optional] [default: 3]", default = 3)
        options = parser.parse_args()[0]
        self._setAttributesFromOptions(options)
        
    def _setAttributesFromOptions(self, options):
        self.setInputVCF(options.inputVCF)
        self.setFilteredVCF(options.filteredVCF)
        self.setMinInsSize(options.minInsSize)
        self.setMaxInsSize(options.maxInsSize)
        self.setMinDelSize(options.minDelSize)
        self.setMaxDelSize(options.maxDelSize)
        self.setRemoveSNP(options.removeSNP)
        self.setVerbosity(options.verbosity)

    def setInputVCF(self, inputVCF):
        self._inputVCF = inputVCF
        if not self._filteredVCF:
            self.setFilteredVCF("")
        
    def setFilteredVCF(self, filteredVCF):
        if filteredVCF == "" and self._inputVCF:
            self._filteredVCF = "%s_filtered.vcf" % os.path.splitext(os.path.basename(self._inputVCF))[0]
        else:
            self._filteredVCF = filteredVCF
        
    def setMinInsSize(self, minInsSize):
        self._minInsSize = minInsSize
        
    def setMaxInsSize(self, maxInsSize):
        self._maxInsSize = maxInsSize
        
    def setMinDelSize(self, minDelSize):
        self._minDelSize = minDelSize
        
    def setMaxDelSize(self, maxDelSize):
        self._maxDelSize = maxDelSize
        
    def setRemoveSNP(self, removeSNP):
        self._removeSNP = removeSNP
        
    def setVerbosity(self, verbosity):
        self._verbosity = verbosity
        
    def _checkOptions(self):
        if self._inputVCF == "":
            self._logAndRaise("ERROR: Missing input VCF file name")
        else:
            if FileUtils.isRessourceExists(self._inputVCF):
                self._inputVCF = os.path.abspath(self._inputVCF)
            else:
                self._logAndRaise("Input VCF file '%s' does not exist!" % self._inputVCF)
                
        if self._minInsSize > self._maxInsSize:
            self._logAndRaise("Inconsistent values for minimum and maximum insertions sizes: '%d' and '%d'" % (self._minInsSize, self._maxInsSize))
            
        if self._minDelSize > self._maxDelSize:
            self._logAndRaise("Inconsistent values for minimum and maximum deletions sizes: '%d' and '%d'" % (self._minDelSize, self._maxDelSize))
            
    def _logAndRaise(self, errorMsg):
        self._log.error(errorMsg)
        raise RepetException(errorMsg)
                    
    def run(self):
        LoggerFactory.setLevel(self._log, self._verbosity)
        self._log.info("START FilterVCF")
        self._checkOptions()
        self._log.info("Input VCF file name: %s" % self._inputVCF)
        self._log.info("Output filtered VCF file name: %s" % self._filteredVCF)
        self._log.info("Minimum insertion size: %d" % self._minInsSize)
        self._log.info("Maximum insertion size: %d" % self._maxInsSize)
        self._log.info("Minimum deletion size: %d" % self._minDelSize)
        self._log.info("Maximum deletion size: %d" % self._maxDelSize)
        self._log.info("Remove SNPs: %s" % self._removeSNP)
        
        with open(self._inputVCF, "r") as f:
            filewrite = open(self._filteredVCF, "w")
            
            for line in f:
                line = line.rstrip("\n")
                
                if line and not line[0] == "#":
                    splittedLine = line.split()
                    REF = splittedLine[3]
                    ALT = splittedLine[4]
                    refLength = len(REF)
                    if "," in ALT:
                        altLength = 1
                    else:
                        altLength = len(ALT)
                    
                    if refLength > altLength: # deletion
                        refLength -= 1
                        if refLength >= self._minDelSize:
                            if self._maxDelSize == -1 or refLength <= self._maxDelSize:
                                filewrite.write("%s\n" % line)
                                
                    elif refLength < altLength: # insertion
                        altLength -= 1
                        if altLength >= self._minInsSize:
                            if self._maxInsSize == -1 or altLength <= self._maxInsSize:
                                filewrite.write("%s\n" % line)
                        
                    elif not self._removeSNP:
                        filewrite.write("%s\n" % line)
                    
                elif line:        
                    filewrite.write("%s\n" % line)
                    
        self._log.info("END FilterVCF")

if __name__ == "__main__":
    iFilterVCF = FilterVCF()
    iFilterVCF.setAttributesFromCmdLine()
    iFilterVCF.run()