#!/usr/bin/env python

import sys
import getopt
import string
from commons.core.coord.MapUtils import MapUtils
from commons.core.coord.Map import Map
from commons.core.seq.Bioseq import Bioseq


def help():
    print
    print "usage: %s [ options ]" % ( sys.argv[0].split("/")[-1] )
    print "options:"
    print "     -h: this help"
    print "     -i: name of the input file (format='fasta')"
    print "     -m: name of the file with the coordinates"
    print "     -f: format of the file with the coordinates (map, from REPET/pyramid, from Piler)"
    print "     -s: soft masking (lower case instead of N's)"
    print "     -o: name of the output file (default=inFileName+'.mask')"
    print "     -v: verbose (default=0/1)"
    print
    
    
def getDictPerSeqNameFromPyramidFile( coordFileName ):
    """
    Load the coordinates to be masked.
    @param coordFileName: name of the 'pyramid' file
    @type coordFileName: string
    @return: dictionary whose keys are sequence names and values the coordinates
    """
    dSeqname2Maps = {}
    coordFile = open( coordFileName, "r" )
    while True:
        line = coordFile.readline()
        if line == "":
            break
        tokens = line.split("\t")
        tokens = ( tokens[2], tokens[0], tokens[3], tokens[4] )
        iMap = Map()
        iMap.setFromTuple( tokens )
        if not dSeqname2Maps.has_key( iMap.seqname ):
            dSeqname2Maps[ iMap.seqname ] = [ iMap ]
        else:
            dSeqname2Maps[ iMap.seqname ].append( iMap )
    coordFile.close()
    return dSeqname2Maps


def maskSequences( inFileName, outFileName, dSeqname2Map, softMasking ):
    """
    Mask the sequences in a fasta file by the coordinates recorded in a dictionary.
    @param inFileName: name of the input fasta file
    @type inFileName: string
    @param outFileName: name of the output fasta file
    @type outFileName: string
    @param dSeqname2Map: dictionary whose keys are sequence names and values the coordinates
    @type dSeqname2Map: dictionary
    @param softMasking: replace sequences to mask by lower case instead of N's
    @param softMasking: boolean
    """
    inFile = open( inFileName, "r" )
    outFile = open( outFileName, "w" )
    bs = Bioseq()
    while True:
        bs.reset()
        bs.read( inFile )
        if bs.header == None:
            break
        tmpSeq = list( bs.sequence )
        if dSeqname2Map.has_key( bs.header ):
            for iMap in dSeqname2Map[ bs.header ]:
                if softMasking:
                    tmpSeq[ min(iMap.start,iMap.end)-1 : max(iMap.start,iMap.end) ] = map( string.lower, tmpSeq[ min(iMap.start,iMap.end)-1 : max(iMap.start,iMap.end) ] )
                else:
                    tmpSeq[ min(iMap.start,iMap.end)-1 : max(iMap.start,iMap.end) ] = (abs(iMap.end - iMap.start) + 1) * "N"
        bs.sequence = "".join( tmpSeq )
        bs.write( outFile )
    inFile.close()
    outFile.close()


def main():
    """
    Use a 'map' file to replace the sequence between two coordinates by Ns.
    """
    inFileName = ""
    coordFileName = ""
    format = ""
    softMasking = False
    outFileName = ""
    verbose = 0
    try:
        opts, args = getopt.getopt( sys.argv[1:], "hi:m:f:so:v:" )
    except getopt.GetoptError, err:
        sys.stderr.write( "%s\n" % str(err) )
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inFileName = a
        elif o == "-m":
            coordFileName = a
        elif o == "-f":
            format = a
        elif o == "-s":
            softMasking = True
        elif o == "-o":
            outFileName = a
        elif o == "-v":
            verbose = int(a)
    if inFileName == "":
        msg = "ERROR: missing input fasta file (-i)"
        sys.stderr.write( "%s\n" % msg )
        help()
        sys.exit(1)
    if format == "":
        msg = "ERROR: missing format (-f)"
        sys.stderr.write( "%s\n" % msg )
        help()
        sys.exit(1)
    if format not in [ "map", "pyramid" ]:
        msg = "ERROR: format '%s' not yet available" % ( format )
        sys.stderr.write( "%s\n" % msg )
        help()
        sys.exit(1)
    if outFileName == "":
        outFileName = "%s.mask" % ( inFileName )
        
    if verbose > 0:
        msg = "START %s" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.write( "%s\n" % msg )
        sys.stdout.flush()
        
    if format == "map":
        dSeqname2Map = MapUtils.getDictPerSeqNameFromMapFile( coordFileName )
    elif format == "pyramid":
        dSeqname2Map = getDictPerSeqNameFromPyramidFile( coordFileName )
    if verbose > 0:
        msg = "nb of input sequences to be masked: %i" % ( len(dSeqname2Map.keys()) )
        sys.stdout.write( "%s\n" % msg )
        sys.stdout.flush()
        
    if verbose > 0:
        msg = "save the masked sequences in '%s'" % ( outFileName )
        sys.stdout.write( "%s\n" % msg )
        sys.stdout.flush()
    maskSequences( inFileName, outFileName, dSeqname2Map, softMasking )
    
    if verbose > 0:
        msg = "END %s" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.write( "%s\n" % msg )
        sys.stdout.flush()
        
    return 0


if __name__ == "__main__":
    main()
