#!/usr/bin/env python

import re
import os
from commons.core.LoggerFactory import LoggerFactory
from commons.core.parsing.FastaParser import FastaParser
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser
import json
from collections import OrderedDict

DWICKERCODE = {
               "I":"RXX",
               "II":"DXX",
               "LTR":"RLX",
               "DIRS":"RYX",
               "PLE":"RPX",
               "LINE":"RIX",
               "SINE":"RSX",
               "TIR":"DTX",
               "Crypton":"DYX",
               "Helitron":"DHX",
               "Maverick":"DMX",

               "MITE":"DXX-MITE",
               "LARD":"RXX-LARD",
               "TRIM":"RXX-TRIM"
               }

LOG_DEPTH = "repet.tools"

class RenameHeaderClassif(object):

    def __init__(self, classifFileName = "", fastaFileName = "", projectName = "", outputFileName = "", isShorten = False, renameInClassif = False, verbosity = 3):
        self._classifFileName = classifFileName
        self._fastaFileName = fastaFileName
        self._isShorten = isShorten
        self._renameInClassif = renameInClassif
        self._projectName = projectName
        self._outputFileName = outputFileName
        self._verbosity = verbosity
        self._log = LoggerFactory.createLogger("%s.%s" % (LOG_DEPTH, self.__class__.__name__), self._verbosity)

    def setClassifFileName(self, fileName):
        self._classifFileName = fileName

    def setFastaFileName(self, fileName):
        self._fastaFileName = fileName

    def setOutputFileName(self, fileName):
        self._outputFileName = fileName

    def setShorten(self, isShorten):
        self._isShorten = isShorten

    def setRenameInClassif(self, renameInClassif):
        self._renameInClassif = renameInClassif

    def setProjectName(self, projectName):
        self._projectName = projectName

    def setAttributesFromCmdLine(self):
        usage = "RenameHeaderClassif.py [options]"
        description = "Shorten sequence headers (from fasta file) if possible with Wicker code according to classification.\n"
        description += "Rename headers in classif file with -r option and in json file too if it exists\n"
        epilog = "\nExample:\n"
        epilog += "\t$ RenameHeaderClassif.py -c Project.classif -i Project.fa -s -r -p ProjectName_v1\n"
        parser = RepetOptionParser(description = description, epilog = epilog, usage = usage)
        parser.add_option("-c", "--classif", dest = "classifFileName", action = "store", type = "string", help = "classification file (from PASTEC or TEclassifier.py)")
        parser.add_option("-i", "--fasta", dest = "fastaFileName", action = "store", type = "string", help = "fasta file name")
        parser.add_option("-s", "--shorten", dest = "shorten", action = "store_true", help = "to short the consensus header", default = False)
        parser.add_option("-r", "--renameInClassif", dest = "renameInClassif", action = "store_true", help = "rename the consensus header in the classification file too (<classif file name>_WickerH.classif)", default = False)
        parser.add_option("-p", "--projectName", dest = "projectName", action = "store", type = "string", help = "project name if shorten option (-s)")
        parser.add_option("-o", "--output", dest = "outputFileName", action = "store", type = "string", help = "output fasta file name (default = <fasta file name>_WickerH.fa)", default = "")
        parser.add_option("-v", "--verbosity", dest = "verbosity", action = "store", type = "int", help = "verbosity [optional] [default: 3]", default = 3)
        options = parser.parse_args()[0]
        self._setAttributesFromOptions(options)

    def _setAttributesFromOptions(self, options):
        self.setClassifFileName(options.classifFileName)
        self.setFastaFileName(options.fastaFileName)
        self.setOutputFileName(options.outputFileName)
        self.setShorten(options.shorten)
        self.setRenameInClassif(options.renameInClassif)
        self.setProjectName(options.projectName)
        self._verbosity = options.verbosity

    def _checkAttributes(self):
        if self._classifFileName == "":
            raise Exception("ERROR: No classification file specified in option!")
        elif not FileUtils.isRessourceExists(self._classifFileName):
            raise Exception("ERROR: Can't open classification file : %s" % self._classifFileName)
        if self._fastaFileName == "":
            raise Exception("ERROR: No fasta file specified in option!")
        elif not FileUtils.isRessourceExists(self._fastaFileName):
            raise Exception("ERROR: Can't open fasta file : %s" % self._fastaFileName)
        if self._isShorten == True and self._projectName == "":
            raise Exception("ERROR: With shorten option you need to specify the project name")
        if self._outputFileName == "":
            prefixe = os.path.basename(self._fastaFileName)
            prefixe = os.path.splitext(prefixe)
            self._outputFileName = "%s_WickerH%s" % (prefixe[0], prefixe[1])

    def _logAndRaise(self, errorMsg):
        self._log.error(errorMsg)
        raise Exception(errorMsg)

    def createClassifInstance(self, classifFileLine):
        lClassifItem = classifFileLine.split("\t")
        consensusName = lClassifItem[0]
        inConfuseness = lClassifItem[3]
        teClass = lClassifItem[4]
        order = lClassifItem[5]
        inCompleteness = lClassifItem[6]

        outCompleteness = self._decisionRulesForCompleteness(inCompleteness)
        outConfuseness = self._decisionRulesForConfuseness(inConfuseness)
        code = self._decisionRuleForWickerCode(teClass, order)

        iClassif = Classif(consensusName, code, outConfuseness, outCompleteness, self._projectName, self._isShorten)
        return iClassif

    def run(self):
        self._log.info("START RenameHeaderClassif")

        self._checkAttributes()
        fClassif = open(self._classifFileName, "r")
        line = fClassif.readline()
        lClassif = []
        while line:
            lClassif.append(self.createClassifInstance(line))
            line = fClassif.readline()
        fClassif.close()
        fOutput = open(self._outputFileName, "w")
        iFastaParser = FastaParser(self._fastaFileName)

        for iSequence in iFastaParser.getIterator():
            for iClassif in lClassif:
                if iClassif.getConsensusName() == iSequence.getName():
                    break
                elif "%s_reversed" % iClassif.getConsensusName() == iSequence.getName():
                    iClassif.setConsensusName(iSequence.getName())
                    break
            iSequence.setName(iClassif.createNewConsensusName())
            fOutput.write(iSequence.printFasta())
        fOutput.close()

        if self._renameInClassif:
            self._renameHeadersInClassifFile(lClassif)

        self._addWickerFieldToJsonClassif()

        self._log.info("END RenameHeaderClassif")

    def _addWickerFieldToJsonClassif(self):
        #add wicker code in json classif file <projectName>Classif.json
        classifFilePath = os.path.dirname(self._classifFileName)

        jsonClassifFileName = '%s/%sClassif.json' % (classifFilePath, self._projectName)
        dClassif = None
        if not os.path.exists(jsonClassifFileName):
            self._log.info("No json classif file provided, skipping")
            return

        with open(jsonClassifFileName, 'r') as inFile:
            dClassif = json.load(inFile, object_pairs_hook = OrderedDict)
            for key, value in dClassif.iteritems():
                teClass = value["class"]
                order = value["order"]
                code = self._decisionRuleForWickerCode(teClass, order)

                alteredODict = OrderedDict()
                alteredODict["name"] = value["name"]
                alteredODict["wCode"] = code
                alteredODict["length"] = value["length"]
                alteredODict["strand"] = value["strand"]
                alteredODict["chimeric"] = value["chimeric"]
                alteredODict["class"] = value["class"]
                alteredODict["order"] = value["order"]
                alteredODict["complete"] = value["complete"]
                alteredODict["CI"] = value["CI"]


                alteredODict["coding"] = value["coding"]
                alteredODict["struct"] = value["struct"]
                alteredODict["other"] = value["other"]

                dClassif[key] = alteredODict

            outJsonClassifFileName = "%s_WickerH.json" % os.path.splitext(os.path.basename(jsonClassifFileName))[0]
            with open(outJsonClassifFileName, 'w') as outFile:
                json.dump(dClassif, outFile)


    def _renameHeadersInClassifFile(self, lClassif):
#        the new fasta and classif files have the same name
        outputClassifFileName = "%s.classif" % os.path.splitext(os.path.basename(self._outputFileName))[0]
        fOut = open(outputClassifFileName, "w")
        f = open(self._classifFileName)
        line = f.readline()
        while line:
            consensusName = line.split("\t")[0]
            for iClassif in lClassif:
                if iClassif.getConsensusName() == consensusName or iClassif.getConsensusName() == "%s_reversed" % consensusName:
                    break
            l = [iClassif.createNewConsensusName()]
            l.extend(line.split("\t")[1:])
            fOut.write("\t".join(l))
            line = f.readline()
        f.close()
        fOut.close()

    def _decisionRulesForCompleteness(self, completeness):
        outCompleteness = None
        if completeness == "incomplete":
            outCompleteness = "incomp"
        elif completeness == "complete":
            outCompleteness = "comp"
        elif completeness == "NA":
            outCompleteness = ""
        return outCompleteness

    def _decisionRulesForConfuseness(self, inConfuseness):
        outConfuseness = None
        if inConfuseness == "ok":
            outConfuseness = ""
        elif inConfuseness == "PotentialChimeric":
            outConfuseness = "chim"
        return outConfuseness

    def _decisionRuleForWickerCode(self, teClass, order):
        code = None
        if order in DWICKERCODE.keys():
            code = DWICKERCODE[order]
        elif order == "noCat" and teClass == "noCat":
            code = "noCat"
        elif re.search("\|", order) and teClass == "noCat":
            code = "XXX"
        elif re.search("\|", order) and teClass == "NA":
            code = "nonTE"
        elif order == "noCat" or re.search("\|", order) :
            code = DWICKERCODE[teClass]
        elif order == "SSR":
            code = "SSR"
        elif order == "rDNA":
            code = "rDNA"
        elif order == "PotentialHostGene":
            code = "PotentialHostGene"
        return code

class Classif(object):

    def __init__(self, consensusName, code, outConfuseness, outCompleteness, projectName = "", isShorten = False):
        self._consensusName = consensusName
        self._code = code
        self._confusness = outConfuseness
        self._completeness = outCompleteness
        self._projectName = projectName
        self._isShorten = isShorten

    def __eq__(self, o):
        if type(o) is type(self):
            return self._consensusName == o._consensusName and self._code == o._code \
                and self._confusness == o._confusness and self._completeness == o._completeness
        return False

    def __ne__(self, o):
        return not self.__eq__(o)

    def createNewConsensusName(self):
        pastecClassif = "%s" % self._code
        if self._completeness != "":
            pastecClassif += "-%s" % self._completeness
        if self._confusness != "":
            pastecClassif += "-%s" % self._confusness
        if self._isShorten:
            pattern = "%s_[a-zA-Z0-9]+_[a-zA-Z0-9]+_[a-zA-Z0-9_]+" % self._projectName
            if re.match(pattern, self._consensusName) and not "%s_RS_" % self._projectName in self._consensusName:
                header = self.shortenConsensusName()
                header = "%s_%s" % (pastecClassif, header)
            else:
                header = "%s_%s" % (pastecClassif, self._consensusName)
        else:
            header = "%s_%s" % (pastecClassif, self._consensusName)

        return header

    def shortenConsensusName(self):
        desc = self._consensusName.split(self._projectName)[1]
        palignMeth = desc.split("_")[1]
        clustMeth = desc.split("_")[2]
        clustID = desc.split("_")[3]
        lmalignMeth = desc.split("_")[4:]
        if len(lmalignMeth) > 2:
            malignMeth = "%s%s_%s" % (lmalignMeth[0], lmalignMeth[1], lmalignMeth[2])
        else:
            malignMeth = "".join(lmalignMeth)
        consensusShorten = "%s-%s-%s%s-%s" % (self._projectName, palignMeth[0], clustMeth[0], clustID, malignMeth)

        return consensusShorten

    def getConsensusName(self):
        return self._consensusName

    def setConsensusName(self, consensusName):
        self._consensusName = consensusName

    def setProjectName(self, projectName):
        self._projectName = projectName

if __name__ == "__main__":
    iRHC = RenameHeaderClassif()
    iRHC.setAttributesFromCmdLine()
    iRHC.run()