#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from commons.core.LoggerFactory import LoggerFactory
from commons.core.utils.RepetOptionParser import RepetOptionParser
import os
from string import rstrip

CONSENSUS = "TE"
CLUSTER = "Cluster"
LOG_DEPTH = "repet.tools"
LOG_FORMAT = "%(message)s"

class giveStatsOnTEClusters(object):

    def __init__(self, clusterFileName = "", addFileName = "", globalStatPerClusterFileName = "", verbosity = 3):
        self._clusterFileName = clusterFileName
        self._addFileName = addFileName
        self._globalStatPerClusterFileName = globalStatPerClusterFileName
        self._verbosity = verbosity
        self._log = LoggerFactory.createLogger("%s.%s" % (LOG_DEPTH, self.__class__.__name__), self._verbosity, LOG_FORMAT)

    def setAttributesFromCmdLine(self):
        usage = "giveStatsOnTEClusters.py [options]"
        description = "Tool to give stats on cluster\n"
        epilog = "\nExample 1: clustering (e.g. to detect redundancy)\n"
        epilog += "\t$ giveStatsOnTEClusters.py -i filename.mcl \n"
        epilog = "\nExample 2: benchmark usage, give reference sequences name in additionnal text file \n"
        epilog += "\t$ giveStatsOnTEClusters.py -i filename.mcl -a refHeadersName.txt -v 4\n"
        parser = RepetOptionParser(description = description, epilog = epilog, usage = usage)
        parser.add_option("-i", "--cluster", dest = "clusterFileName", action = "store", type = "string", help = "cluster file name mandatory", default = "")
        parser.add_option("-a", "--addedFile", dest = "addFileName", action = "store", type = "string", help = "added file name [optional]", default = "")
        parser.add_option("-v", "--verbosity", dest = "verbosity", action = "store", type = "int", help = "verbosity [optional] [default: 3]", default = 3)
        options = parser.parse_args()[0]
        self._setAttributesFromOptions(options)

    def _setAttributesFromOptions(self, options):
        self.setClusterFileName(options.clusterFileName)
        self.setStatPerClusterFileName()
        self.setAddFileName(options.addFileName)
        self.setVerbosity(options.verbosity)

    def setClusterFileName(self, clusterFileName):
        self._clusterFileName = clusterFileName
        self.setStatPerClusterFileName()

    def setAddFileName(self, addFileName):
        self._addFileName = addFileName

    def setVerbosity(self, verbosity):
        self._verbosity = verbosity


    def setStatPerClusterFileName(self):
        self._globalStatPerClusterFileName = "%s.globalStatsPerCluster.txt" % os.path.splitext(self._clusterFileName)[0]

    def _checkOptions(self):
        pass

    def _logAndRaise(self, errorMsg):
        self._log.error(errorMsg)
        raise Exception(errorMsg)

    def run(self):
        LoggerFactory.setLevel(self._log, self._verbosity)
        self._log.info("START %s" % (type(self).__name__))
        self._log.debug("fichier de clustering: %s" % self._clusterFileName)
        self._log.debug("fichier additionnel: %s" % self._addFileName)
        self._log.debug("fichier de resultats: %s" % self._globalStatPerClusterFileName)
        lAddedHeaders = []
        i = 0
        if self._addFileName != "":
            faddedFileHeaders = open(self._addFileName, 'r')
            lAddedHeaders = faddedFileHeaders.readlines()
            while i < len(lAddedHeaders):
                lAddedHeaders[i] = rstrip(lAddedHeaders[i], '\n')
                i += 1
        fCluster = open(self._clusterFileName)
        line = fCluster.readline()
        clusterNb = 0
        clusterSeqList = line.split()
        minClusterSize = len(clusterSeqList)
        maxClusterSize = 0
        totalSeqNb = 0
        seqNbInBigClusters = 0
        dClusterSize2ClusterNb = {1:0, 2:0, 3:0}
        NbClustersTruePositif = 0
        NbClustersFalsePositif = 0
        NbClustersFalsePositifWithOnlyDenovoConsensus = 0
        NbClustersFalsePositifWithMoreThanOnebdgp = 0
        NbClustersFalseNegatif = 0
        NbClustersFalseNegatifWithOnlyOnebdgp = 0
        NbClustersFalseNegatifWithSeveralbdgp = 0
        while line:
            clusterSeqList = line.split()
            seqNb = len(clusterSeqList)
            totalSeqNb += seqNb
            lIntersection = list(set(clusterSeqList) & set(lAddedHeaders))
            nbIntersectionElements = len(lIntersection)
            if nbIntersectionElements > 0:
                if nbIntersectionElements == 1:
                    if seqNb == 1:
                        NbClustersFalseNegatif += 1
                        NbClustersFalseNegatifWithOnlyOnebdgp += 1
                    else:
                        NbClustersTruePositif += 1
                else:
                    if nbIntersectionElements == seqNb:
                        NbClustersFalseNegatif += 1
                        NbClustersFalseNegatifWithSeveralbdgp += 1
                    else:
                        NbClustersFalsePositif += 1
                        NbClustersFalsePositifWithMoreThanOnebdgp += 1
            else:
                NbClustersFalsePositif += 1
                NbClustersFalsePositifWithOnlyDenovoConsensus += 1
            if seqNb > 2:
                seqNbInBigClusters += seqNb
                dClusterSize2ClusterNb[3] += 1
            else:
                dClusterSize2ClusterNb[seqNb] += 1
            if seqNb > maxClusterSize:
                maxClusterSize = seqNb
            if seqNb < minClusterSize:
                minClusterSize = seqNb
            line = fCluster.readline()
            clusterNb += 1
        TPplusFP = NbClustersTruePositif + NbClustersFalsePositif
        if TPplusFP != 0:
            ppv = round(NbClustersTruePositif / float(TPplusFP), 2)
        else:
            ppv = 'NA'
        TPplusFN = NbClustersTruePositif + NbClustersFalseNegatif
        if TPplusFN != 0:
            sn = round(NbClustersTruePositif / float(TPplusFN), 2)
        else:
            sn = 'NA'
        TPplusFPplusFN = NbClustersTruePositif + NbClustersFalsePositif + NbClustersFalseNegatif
        if TPplusFPplusFN != 0:
            acc = round(NbClustersTruePositif / float(NbClustersTruePositif + NbClustersFalsePositif + NbClustersFalseNegatif), 2)
        else:
            acc = 'NA'
        fG = open(self._globalStatPerClusterFileName, 'w')
        fG.write("nb of sequences: %d\n" % totalSeqNb)
        fG.write("nb of clusters: %d\n" % clusterNb)
        fG.write("nb of clusters with 1 sequence: %d\n" % dClusterSize2ClusterNb[1])
        fG.write("nb of clusters with 2 sequences: %d\n" % dClusterSize2ClusterNb[2])
        fG.write("nb of clusters with >2 sequences: %d (with max seq %d)\n" % (dClusterSize2ClusterNb[3], seqNbInBigClusters))
        fG.write("nb of sequences in the largest cluster: %d\n" % maxClusterSize)
        fG.write("nb of sequences in the smallest cluster: %d\n" % minClusterSize)
        if self._addFileName != "":
            fG.write("nb true positif: %d\n" % NbClustersTruePositif)
            fG.write("nb false positif: %d\n" % NbClustersFalsePositif)
            fG.write("nb false positif with only Denovo consensus: %d\n" % NbClustersFalsePositifWithOnlyDenovoConsensus)
            fG.write("nb false positif with several bdgp: %d\n" % NbClustersFalsePositifWithMoreThanOnebdgp)
            fG.write("nb false negatif: %d\n" % NbClustersFalseNegatif)
            fG.write("nb false negatif with only 1 bdgp: %d\n" % NbClustersFalseNegatifWithOnlyOnebdgp)
            fG.write("nb false negatif with several bdgp: %d\n" % NbClustersFalseNegatifWithSeveralbdgp)
            fG.write("Positive Predictive Value (PPV) TP/(TP+FP): %s\n" % ppv)
            fG.write("Sensitivity TP/(TP+FN): %s\n" % sn)
            fG.write("Accuracy TP+TN/(TP+FP+TN+FN): %4.2f\n" % acc)
        fG.close()
        fTab = open("%s.tab" % self._globalStatPerClusterFileName, 'w')
        if self._addFileName != "":
            fTab.write("%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%4.2f\t%4.2f\t%4.2f\n" % \
                   (totalSeqNb, clusterNb, dClusterSize2ClusterNb[1], dClusterSize2ClusterNb[2], dClusterSize2ClusterNb[3], \
                    maxClusterSize, minClusterSize, NbClustersTruePositif, NbClustersFalsePositif, NbClustersFalsePositifWithOnlyDenovoConsensus, \
                    NbClustersFalsePositifWithMoreThanOnebdgp, NbClustersFalseNegatif, NbClustersFalseNegatifWithOnlyOnebdgp, NbClustersFalseNegatifWithSeveralbdgp, ppv, sn, acc))
        else:
            fTab.write("%d\t%d\t%d\t%d\t%d\t%d\t%d\n" % \
                   (totalSeqNb, clusterNb, dClusterSize2ClusterNb[1], dClusterSize2ClusterNb[2], dClusterSize2ClusterNb[3], maxClusterSize, minClusterSize))
        fTab.close()
        self._log.info("END %s" % (type(self).__name__))


if __name__ == "__main__":
    iLaunch = giveStatsOnTEClusters()
    iLaunch.setAttributesFromCmdLine()
    iLaunch.run()
