import unittest
import os
import shutil
from commons.core.sql.DbFactory import DbFactory
from commons.core.utils.FileUtils import FileUtils
from commons.tools.CreateGFF3sForClassifFeatures import CreateGFF3sForClassifFeatures

class Test_F_CreateGFF3sForClassifFeatures(unittest.TestCase):
    
    def setUp(self):
        self._cDir = os.getcwd()
        self._projectName = "Test_CGCF"
        self._TEannotProjectName = "Test_CGCF_annot"
        self._cgcfDir = "Visualization_Files"
        os.mkdir(self._projectName)
        os.chdir(self._projectName)
        self._testDir = os.getcwd()
        self._fastaFileName = "%s_LibTEs.fa" % self._projectName
        self._configFileName = "%s.cfg" % self._projectName
        self._iDb = DbFactory.createInstance()
        self._lTables = []
    
    def tearDown(self):
        for table in self._lTables:
            self._iDb.dropTable(table)
        self._iDb.close()
        os.chdir(self._cDir)
        shutil.rmtree(self._testDir)
         
    def test_run_as_script_full(self):
        self._writeFastaFile()
        cfgOptions = ["yes"] * 14
        self._writeConfigFile(cfgOptions)
        self._writeClassifFile()
        self._createDummyTR()
        self._createDummyPolyA()
        self._createDummyORF()
        self._createDummyTE_BLRn()
        self._createDummyTE_BLRtx()
        self._createDummyTE_BLRx()
        self._createDummyRDNA_BLRn()
        self._createDummyHG_BLRn()
        self._createDummyTRNA()
        self._createDummyProfiles()
        self._createDummySSR()
        self._createDummyAnnotatedCopies()
        self._createDummyOriginalCopies()
        
        cmd = "%s/commons/tools/CreateGFF3sForClassifFeatures.py" % os.environ["REPET_PATH"]
        cmd += " -C %s" % self._configFileName
        cmd += " -f %s" % self._fastaFileName
        os.system(cmd)
        
        features = ["TR", "polyA", "SSR", "rDNA_BLRn", "TE_BLRn", "TE_BLRtx", "TE_BLRx", "Profiles", "HG_BLRn", "ORF", "annotated_copies"]
        
        dExpToObs = {"%s/Tools/CreateGFF3sForClassifFeatures_exp%s.gff3" % (os.environ["REPET_DATA"], feat): "%s/%s_%s.gff3" % (self._cgcfDir, self._projectName if feat != "annotated_copies" else self._TEannotProjectName, feat)
                     for feat in features}
        
        for expFile, obsFile in dExpToObs.iteritems():
            self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
        
        oriCopDir = "%s/Original_HSP_fastaAlignment" % self._cgcfDir
        obsFiles = [f for f in os.listdir(oriCopDir) if os.path.islink(os.path.join(oriCopDir, f))]
        self.assertTrue(len(obsFiles)==2)
    
    def test_run_as_script_someFeats_noCopies_noClassif(self):
        self._writeFastaFile()
        cfgOptions = ["no"] + ["yes"] * 5 + ["no"] * 6 + ["no", "no"]
        self._writeConfigFile(cfgOptions)
        self._createDummyTR()
        self._createDummyPolyA()
        self._createDummyORF()
        self._createDummyTE_BLRn()
        self._createDummyTE_BLRtx()
        
        cmd = "%s/commons/tools/CreateGFF3sForClassifFeatures.py" % os.environ["REPET_PATH"]
        cmd += " -C %s" % self._configFileName
        cmd += " -f %s" % self._fastaFileName
        os.system(cmd)
        
        features = ["TR", "polyA", "ORF", "TE_BLRn", "TE_BLRtx"]
        
        dExpToObs = {"%s/Tools/CreateGFF3sForClassifFeatures_exp%s_noClassif.gff3" % (os.environ["REPET_DATA"], feat): "%s/%s_%s.gff3" % (self._cgcfDir, self._projectName, feat)
                     for feat in features}
        
        for expFile, obsFile in dExpToObs.iteritems():
            self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
    
    def test_run_as_instance_someFeats_noCopies(self):
        self._writeFastaFile()
        cfgOptions = ["yes"] + ["no"] * 5 + ["yes"] * 6 + ["no", "no"]
        self._writeConfigFile(cfgOptions)
        self._writeClassifFile()
        self._createDummyTE_BLRx()
        self._createDummyHG_BLRn()
        self._createDummyRDNA_BLRn()
        self._createDummyTRNA()
        self._createDummyProfiles()
        self._createDummySSR()
        
        iCGCF = CreateGFF3sForClassifFeatures(self._configFileName, self._fastaFileName)
        iCGCF.run()
        
        features = ["TE_BLRx", "HG_BLRn", "rDNA_BLRn", "tRNA", "Profiles", "SSR"]
        
        dExpToObs = {"%s/Tools/CreateGFF3sForClassifFeatures_exp%s.gff3" % (os.environ["REPET_DATA"], feat): "%s/%s_%s.gff3" % (self._cgcfDir, self._projectName, feat)
                     for feat in features}
        
        for expFile, obsFile in dExpToObs.iteritems():
            self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
    
    def test_run_as_instance_tablesDontExist(self):
        self._writeFastaFile()
        cfgOptions = ["no"] + ["yes"] * 12 + ["no"]
        self._writeConfigFile(cfgOptions)
        
        iCGCF = CreateGFF3sForClassifFeatures(self._configFileName, self._fastaFileName)
        with self.assertRaises(SystemExit):
            iCGCF.run()
    
    def test_run_as_instance_none(self):
        self._writeFastaFile()
        cfgOptions = ["no"] * 14
        self._writeConfigFile(cfgOptions)
        
        iCGCF = CreateGFF3sForClassifFeatures(self._configFileName, self._fastaFileName)
        with self.assertRaises(SystemExit):
            iCGCF.run()
    
    def test_run_as_instance_VisualizeFilesFolderExists(self):
        self._writeFastaFile()
        cfgOptions = ["no", "yes"] + ["no"] * 12
        self._writeConfigFile(cfgOptions)
        self._createDummyTR()
        
        os.mkdir(self._cgcfDir)
        
        iCGCF = CreateGFF3sForClassifFeatures(self._configFileName, self._fastaFileName, verbosity=3)
        iCGCF.run()
        
        expFile = "%s/Tools/CreateGFF3sForClassifFeatures_expTR_noClassif.gff3" % os.environ["REPET_DATA"]
        obsFile = "%s/%s_TR.gff3" % (self._cgcfDir, self._projectName)
        self.assertTrue(FileUtils.are2FilesIdentical(expFile, obsFile))
            
    def test_run_as_instance_wrong_projectName(self):
        self._projectName = "wrongName"
        self._writeFastaFile()
        cfgOptions = ["yes"] * 14
        self._writeConfigFile(cfgOptions)
        
        iCGCF = CreateGFF3sForClassifFeatures(self._configFileName, self._fastaFileName)
        with self.assertRaises(SystemExit):
            iCGCF.run()
    
    def test_run_as_instance_wrong_input_files(self):
        self._writeFastaFile()
        cfgOptions = ["yes"] * 14
        self._writeConfigFile(cfgOptions)
        wrongFasta = "wrong.fa"
        wrongCfg = "wrong.cfg"
        
        iCGCF = CreateGFF3sForClassifFeatures(wrongCfg, self._fastaFileName)
        with self.assertRaises(SystemExit):
            iCGCF.run()
            
        iCGCF = CreateGFF3sForClassifFeatures(self._configFileName, wrongFasta)
        with self.assertRaises(SystemExit):
            iCGCF.run()
        
    def _writeConfigFile(self, lOptions):
        if len(lOptions) != 14:
            print "Error with writeConfigFile call"
            raise SystemExit(1)
        with open(self._configFileName, "w") as configF:
            configF.write("[repet_env]\n")
            configF.write("repet_host: %s\n" % os.environ["REPET_HOST"])
            configF.write("repet_user: %s\n" % os.environ["REPET_USER"])
            configF.write("repet_pw: %s\n" % os.environ["REPET_PW"])    
            configF.write("repet_db: %s\n" % os.environ["REPET_DB"])
            configF.write("repet_port: %s\n" % os.environ["REPET_PORT"])
            configF.write("\n")
            configF.write("[project]\n")
            configF.write("project_name: %s\n" % self._projectName)
            configF.write("project_dir: %s\n" % os.getcwd())
            configF.write("\n")
            configF.write("[gff3_TEdenovo_options]\n")
            configF.write("add_classif_infos: %s\n" % lOptions[0])
            configF.write("TR: %s\n" % lOptions[1])
            configF.write("polyA: %s\n" % lOptions[2])
            configF.write("ORF: %s\n" % lOptions[3])
            configF.write("TE_BLRn: %s\n" % lOptions[4])
            configF.write("TE_BLRtx: %s\n" % lOptions[5])
            configF.write("TE_BLRx: %s\n" % lOptions[6])
            configF.write("HG_BLRn: %s\n" % lOptions[7])
            configF.write("rDNA_BLRn: %s\n" % lOptions[8])
            configF.write("tRNA: %s\n" % lOptions[9])
            configF.write("Profiles: %s\n" % lOptions[10])
            configF.write("SSR: %s\n" % lOptions[11])
            configF.write("\n")
            configF.write("[gff3_TEannot_options]\n")
            configF.write("project_name_teannot: %s\n" % self._TEannotProjectName)
            configF.write("annotated_copies: %s\n" % lOptions[12])
            configF.write("\n")
            configF.write("[other]\n")
            configF.write("original_HSP: %s\n" % lOptions[13])
            

    def _writeFastaFile(self):
        with open(self._fastaFileName, "w") as f:
            f.write(">DHX-incomp_%s-B-G12-Map12\n" % self._projectName)
            f.write("ATACAGCTGCGGTTAAAATAATAGCACTACTGCAGGTGGAAAGTTGATTTCCTAAAAAAA\n")
            f.write("ATACAGCTGCGGTTAAAATAATAGCACTACTGCAGGTGGAAAGTTGATTTCCTAAAAAAA\n")
            f.write(">XXX-comp_%s-L-B1-Map1\n" % self._projectName)
            f.write("TTGCTGCAAAAGTGCGCGAAATGAGGCGGTAACAAAAATAGCACTGAACACGTATTTGCT\n")
            f.write("TTGCTGCAAAAGTGCGCGAAATGAGGCGGTAACAAAAATAGCACTGAACACGTATTTGCT\n")
        
    def _writeClassifFile(self):
        TEclassifFolder = "%s_Blaster_GrpPil_Struct_Map_TEclassif" % self._projectName
        os.mkdir(TEclassifFolder)
        os.chdir(TEclassifFolder)
        os.mkdir("classifConsensus")
        os.chdir("classifConsensus")
        classifFileName = "%s_consensus_WickerH.classif" % self._projectName
        with open(classifFileName, "w") as cf:
            line = "DHX-incomp_%s-B-G12-Map12\t60\t.\tok\tII\tTIR\tincomplete\tCI=37; " % self._projectName
            line += "coding=(TE_BLRtx: Foo:Helitron: 86.84%, Bar:Helitron: 80.12%; TE_BLRx: Foo:Gypsy: 72.87%); "
            line += "struct=(TermRepeats: termLTR: 29; ORF: >30bps); "
            line += "other=(Other_profiles: PF01__OTHER_25.0: 100.00%(100.00%); rDNA_BLRn: embl|J01854|J01854 D.melanogaster 5S rRNA.: 100.00%; "
            line += "HG_BLRn: Dummy_match: 98.04%; TElength: >4000bps; SSR: (AT)6_start; SSRCoverage=0.06; polyAtail; "
            line += "helitronExtremities: Foo:classII:Helitron: (2e-39, 1, 82))\n"
            cf.write(line)
        os.chdir(self._testDir)
    
    def _createDummyTR(self):
        dummyTR = "dummyTRSetFile.set"
        dummyTRTable = "%s_TR_set" % self._projectName
        with open(dummyTR, "w") as setF:
            setF.write("1\tRep1termLTR|len=29|id=0.998125|lenTE=120\t%s_Blaster_Grouper_12_Map_12\t1\t29\n" % self._projectName)
            setF.write("1\tRep1termLTR|len=29|id=0.998125|lenTE=120\t%s_Blaster_Grouper_12_Map_12\t41\t69\n" % self._projectName)
            setF.write("2\tRep2LTR|len=12|id=0.999277|lenTE=120\t%s_Blaster_Grouper_12_Map_12\t25\t36\n" % self._projectName)
            setF.write("2\tRep2LTR|len=12|id=0.999277|lenTE=120\t%s_Blaster_Grouper_12_Map_12\t44\t57\n" % self._projectName)
        self._iDb.createTable(dummyTRTable, "set", dummyTR)
        self._lTables.append(dummyTRTable)

    def _createDummyPolyA(self):
        dummyPolyA = "dummyPolyASetFile.set"
        dummyPolyATable = "%s_polyA_set" % self._projectName
        with open(dummyPolyA, "w") as setF:
            setF.write("1\ttermPolyA|len=11|id=1\t%s_Blaster_Grouper_12_Map_12\t110\t120" % self._projectName)
        self._iDb.createTable(dummyPolyATable, "set", dummyPolyA)
        self._lTables.append(dummyPolyATable)
        
    def _createDummySSR(self):
        dummySSR = "dummySSRSetFile.set"
        dummySSRTable = "%s_SSR_set" % self._projectName
        with open(dummySSR, "w") as setF:
            setF.write("1\t(AT)6\t%s_Blaster_Grouper_12_Map_12\t1\t13" % self._projectName)
        self._iDb.createTable(dummySSRTable, "set", dummySSR)
        self._lTables.append(dummySSRTable)
        
    def _createDummyRDNA_BLRn(self):
        dummyRDNA_BLRn = "dummyRDNA_BLRnPathFile.path"
        dummyRDNA_BLRnTable = "%s_rDNA_BLRn_path" % self._projectName
        with open(dummyRDNA_BLRn, "w") as pathF:
            pathF.write("1\t%s_Blaster_Grouper_12_Map_12\t46\t165\tembl|J01854|J01854 D.melanogaster 5S rRNA.\t120\t1\t2e-57\t222\t98.33\n" % self._projectName)
        self._iDb.createTable(dummyRDNA_BLRnTable, "path", dummyRDNA_BLRn)
        self._lTables.append(dummyRDNA_BLRnTable)
        
    def _createDummyTE_BLRn(self):
        dummyTE_BLRn = "dummyTE_BLRnPathFile.path"
        dummyTE_BLRnTable = "%s_TE_BLRn_path" % self._projectName
        with open(dummyTE_BLRn, "w") as pathF:
            pathF.write("1\t%s_Blaster_Grouper_12_Map_12\t46\t165\tFoo:classII:Helitron\t120\t1\t2e-57\t222\t98.33\n" % self._projectName)
        self._iDb.createTable(dummyTE_BLRnTable, "path", dummyTE_BLRn)
        self._lTables.append(dummyTE_BLRnTable)
        
    def _createDummyTE_BLRtx(self):
        dummyTE_BLRtx = "dummyTE_BLRtxPathFile.path"
        dummyTE_BLRtxTable = "%s_TE_BLRtx_path" % self._projectName
        with open(dummyTE_BLRtx, "w") as pathF:
            pathF.write("1\t%s_Blaster_Grouper_12_Map_12\t10\t20\tFoo:Helitron\t1\t10\t1e-123\t136\t86.4\n" % self._projectName)
            pathF.write("1\t%s_Blaster_Grouper_12_Map_12\t25\t40\tFoo:Helitron\t11\t25\t1e-152\t161\t86.4\n" % self._projectName)
        self._iDb.createTable(dummyTE_BLRtxTable, "path", dummyTE_BLRtx)
        self._lTables.append(dummyTE_BLRtxTable)
        
    def _createDummyTE_BLRx(self):
        dummyTE_BLRx = "dummyTE_BLRxPathFile.path"
        dummyTE_BLRxTable = "%s_TE_BLRx_path" % self._projectName
        with open(dummyTE_BLRx, "w") as pathF:
            pathF.write("1\t%s_Blaster_Grouper_12_Map_12\t10\t20\tFoo:Gypsy\t1\t10\t1e-123\t136\t72.87\n" % self._projectName)
            pathF.write("2\t%s_Blaster_Grouper_12_Map_12\t25\t40\tBar:Gypsy\t5\t20\t1e-152\t161\t98.7\n" % self._projectName)
        self._iDb.createTable(dummyTE_BLRxTable, "path", dummyTE_BLRx)
        self._lTables.append(dummyTE_BLRxTable)
    
    def _createDummyProfiles(self):
        dummyProfiles = "dummyProfilesPathFile.path"
        dummyProfilesTable = "%s_Profiles_path" % self._projectName
        with open(dummyProfiles, "w") as pathF:
            pathF.write("1\t%s_Blaster_Grouper_12_Map_12\t10\t20\tPF01__OTHER_25.0\t1\t10\t1e-123\t136\t72.87\n" % self._projectName)
        self._iDb.createTable(dummyProfilesTable, "path", dummyProfiles)
        self._lTables.append(dummyProfilesTable)
        
    def _createDummyHG_BLRn(self):
        dummyHG_BLRn = "dummyHG_BLRnPathFile.path"
        dummyHG_BLRnTable = "%s_HG_BLRn_path" % self._projectName
        with open(dummyHG_BLRn, "w") as pathF:
            pathF.write("1\t%s_Blaster_Grouper_12_Map_12\t10\t20\tDummy_match\t1\t10\t1e-123\t136\t72.87\n" % self._projectName)
        self._iDb.createTable(dummyHG_BLRnTable, "path", dummyHG_BLRn)
        self._lTables.append(dummyHG_BLRnTable)
    
    def _createDummyORF(self):
        dummyORF = "dummyORFMapFile.map"
        dummyORFTable = "%s_ORF_map" % self._projectName
        with open(dummyORF, "w") as mapF:
            mapF.write("ORF|-2|38\t%s_Blaster_Grouper_12_Map_12\t10\t50\n" % self._projectName)
            mapF.write("ORF|-2|28\t%s_LTRharvest_Blastclust_1_Map_1\t5\t35\n" % self._projectName)
        self._iDb.createTable(dummyORFTable, "map", dummyORF)
        self._lTables.append(dummyORFTable)
    
    def _createDummyTRNA(self):
        dummyTRNA = "dummyTRNAMapFile.map"
        dummyTRNATable = "%s_tRNA_map" % self._projectName
        with open(dummyTRNA, "w") as mapF:
            mapF.write("Leu|CAG|intron=0-0|77.88\t%s_LTRharvest_Blastclust_1_Map_1\t40\t20\n" % self._projectName)
        self._iDb.createTable(dummyTRNATable, "map", dummyTRNA)
        self._lTables.append(dummyTRNATable)
    
    def _createDummyAnnotatedCopies(self):
        dummyAnnCop = "dummyAnnCopPathFile.path"
        dummyAnnCopTable = "%s_chr_allTEs_nr_noSSR_join_path" % self._TEannotProjectName
        with open(dummyAnnCop, "w") as pathF:
            pathF.write("1\tDummy_copy\t10\t20\t%s_Blaster_Grouper_12_Map_12\t1\t10\t1e-123\t136\t72.87\n" % self._TEannotProjectName)
        self._iDb.createTable(dummyAnnCopTable, "path", dummyAnnCop)
        self._lTables.append(dummyAnnCopTable)
        
    def _createDummyOriginalCopies(self):
        oriCopDir1 = "%s_Blaster_Grouper_Map" % self._projectName
        os.mkdir(oriCopDir1)
        os.chdir(oriCopDir1)
        dummyOriCop = "seqCluster12.fa.fa_aln"
        with open(dummyOriCop, "w") as faF:
            faF.write(">GrouperCluster9Mb746 dmel_chr3L {Fragment} 5096011..5098564\n")
            faF.write("-ACTG\n")
        os.chdir(self._testDir)
        oriCopDir2 = "%s_LTRharvest_Blastclust_Map" % self._projectName
        os.mkdir(oriCopDir2)
        os.chdir(oriCopDir2)
        dummyOriCop = "seqCluster1.fa.fa_aln"
        with open(dummyOriCop, "w") as faF:
            faF.write(">BlastclustCluster9Mb746 dmel_chr3L {Fragment} 5096011..5098564\n")
            faF.write("-ACTG\n")
        os.chdir(self._testDir)
    
if __name__ == "__main__":
    unittest.main()
