import os
import time
import shutil
import unittest
from commons.core.sql.DbFactory import DbFactory
from commons.core.utils.FileUtils import FileUtils
from commons.tools.DetectTEFeatures import DetectTEFeatures
from commons.tools.ListAndDropTables import ListAndDropTables
from commons.pyRepetUnit.profilesDB.ProfilesDatabankUtils import ProfilesDatabankUtils

REPET_DB_USER = os.environ["REPET_USER"]
REPET_DB_HOST = os.environ["REPET_HOST"]
REPET_DB_PW = os.environ["REPET_PW"]
REPET_DB_NAME = os.environ["REPET_DB"]
REPET_DB_PORT = os.environ["REPET_PORT"]

class Test_F_DetectTEFeatures(unittest.TestCase):
    
    def setUp(self):
        self._dataDir = os.environ["REPET_DATA"]
        self._pastecDataPath = os.path.join(self._dataDir, "PASTEC")
        self._bank = "ProfilesDB4Repet_v2.2_formatted.hmm"
        self._profileDb = os.path.join(os.environ["REPET_BANKS"], self._bank)
        
        self._repbaseNT = "DummyRepbase_nt.fa"
        self._repbaseAA = "DummyRepbase_aa.fa"

        self._repbaseNTPath = os.path.join(self._pastecDataPath, self._repbaseNT)
        self._repbaseAAPath = os.path.join(self._pastecDataPath, self._repbaseAA)
        
        self._inputFastaFileName = "consensusAllClassif.fa"
        self._inputFastaFilePath = os.path.join(self._pastecDataPath, self._inputFastaFileName)
        
        self._configFileName = "config.cfg"
        self._projectName = "test_DetectTEFeatures_%s" % time.strftime("%Y%m%d%H%M%S")
        
        self._blastType = "ncbi"
        self._verbosity = 3
        
        self._workDir = os.path.join(os.getcwd(), self._projectName)
        self._curDir = os.getcwd()
        
        self._db = DbFactory.createInstance()
        if not os.path.exists(self._workDir):
            os.makedirs(self._workDir)
    
        os.chdir(self._workDir)
        self._writeConfig()
        self._symlinkDatabanksAndInputFileInProjectDir()
        
    def tearDown(self):
        os.chdir(self._curDir)
        try:
            iLDT = ListAndDropTables()
            iLDT._action = "drop"
            iLDT._tableNames = self._projectName
            iLDT.run()
        except:pass
        try:
            shutil.rmtree(self._workDir)
        except:pass
        print

    def test_DetectTEFeatures(self):
        iDetectTEFeatures = DetectTEFeatures(fastaFileName = self._inputFastaFileName, projectName = self._projectName, configFileName = self._configFileName, doClean = True, verbosity = 3)
        iDetectTEFeatures.run()
        
        observedProfileFileName = "consensusAllClassif.fa_Profiles_ProfilesDB4Repet_v2.2_formatted.hmm.align.clean_match.path"
        self.check(observedProfileFileName)

    def test_DetectTEFeatures_onMultiHMMbanks(self):
        lSplitProfileBankNames = ProfilesDatabankUtils.splitFile(self._bank, 170)
        os.system("sed -i 's/TE_HMM_profiles: %s/TE_HMM_profiles: %s/' %s" % (self._bank, ",".join(lSplitProfileBankNames), self._configFileName))
        
        iDetectTEFeatures = DetectTEFeatures(fastaFileName = self._inputFastaFileName, projectName = self._projectName, configFileName = self._configFileName, doClean = True, verbosity = 3)
        iDetectTEFeatures.run()
        
        observedProfileFileName = "consensusAllClassif.fa_Profiles_ProfilesDB4Repet_v2.2_formatted1_ProfilesDB4Repet_v2.2_formatted2_ProfilesDB4Repet_v2.2_formatted3.hmm.align.clean_match.path"
        self.check(observedProfileFileName)
        
    def check(self, obsPathFileName):
        outputFileNames= [ "consensusAllClassif.fa_TE_BLRn_DummyRepbase_nt.fa.align.clean_match.path",
        "consensusAllClassif.fa_TE_BLRtx_DummyRepbase_nt.fa.align.clean_match.path",
        "consensusAllClassif.fa_TE_BLRx_DummyRepbase_aa.fa.align.clean_match.path",
        "consensusAllClassif.fa.ORF.map",
        "consensusAllClassif.fa.polyA.set",
        "consensusAllClassif.fa.SSR.set",
        "consensusAllClassif.fa.TR.set"]
        
        success = True
        dataPath = os.environ["REPET_DATA"] + "/Tools/DetectTEFeatures"
        for fileName in outputFileNames:
            expFileName = os.path.join(dataPath, fileName)
            print "Compare:"
            print expFileName
            print fileName
            if not FileUtils.are2FilesIdentical(expFileName, fileName):
                print "Files are different\n"
                success = False
            else:
                print "Files are identical\n"
        
        expPathFileName = os.path.join(dataPath, "test_DetectTEFeatures_Profiles_path.path")
        print "Compare:"
        print expFileName
        print fileName
        if not FileUtils.are2FilesIdentical(expPathFileName, obsPathFileName):
            print "Files are different\n"
            success = False
        else:
            print "Files are identical\n"
            
        self.assertTrue(success)
    
    def _symlinkDatabanksAndInputFileInProjectDir(self):
        os.symlink(self._profileDb, self._bank)
        os.symlink(self._repbaseNTPath, self._repbaseNT)
        os.symlink(self._repbaseAAPath, self._repbaseAA)
        os.symlink(self._inputFastaFilePath, self._inputFastaFileName)
        
    def _writeConfig(self):
        configF = open(self._configFileName, "w")
        configF.write("[repet_env]\n")
        configF.write("repet_host: %s\n" % REPET_DB_HOST)
        configF.write("repet_user: %s\n" % REPET_DB_USER)
        configF.write("repet_pw: %s\n" % REPET_DB_PW)
        configF.write("repet_db: %s\n" % REPET_DB_NAME)
        configF.write("repet_port: %s\n" % REPET_DB_PORT)
        configF.write("\n")
        configF.write("[project]\n")
        configF.write("project_name: %s\n" % self._projectName)
        configF.write("project_dir: %s\n" % self._workDir)
        configF.write("\n")
        configF.write("[detect_features]\n")
        configF.write("tmpDir: \n")
        configF.write("clean: yes\n")
        configF.write("TE_HMMER_evalue: 10\n")
        configF.write("TE_HMM_profiles: %s\n" % self._bank)
        configF.write("resources: \n") 
        configF.write("blast: %s\n" % self._blastType)
        configF.write("TE_HMMER: yes\n")
        configF.write("term_rep: yes\n")
        configF.write("polyA: yes\n")
        configF.write("tand_rep: yes\n")
        configF.write("orf: yes\n")
        configF.write("TE_BLRtx: yes\n")
        configF.write("TE_BLRn: yes\n")
        configF.write("TE_nucl_bank: %s\n" % self._repbaseNT)
        configF.write("TE_BLRx: yes\n")
        configF.write("TE_prot_bank: %s\n" % self._repbaseAA)
        configF.write("HG_BLRn: no\n")
        configF.write("HG_nucl_bank: \n") 
        configF.write("rDNA_BLRn: no\n") 
        configF.write("rDNA_bank: \n") 
        configF.write("tRNA_scan: no\n")
        configF.write("TRFmaxPeriod: 15\n")
        configF.close()
    
if __name__ == "__main__":                 
    unittest.main()