import os
import unittest
from commons.tools.FilterVCF import FilterVCF
from commons.core.utils.FileUtils import FileUtils
from commons.core.checker.RepetException import RepetException

class Test_F_FilterVCF(unittest.TestCase):
    
    def setUp(self):
        self._inputVCF = "%s/Tools/FilterVCF/input.vcf" % os.environ["REPET_DATA"]
        self._obsVCF = "input_filtered.vcf"
        self._verbosity = 3
        
    def tearDown(self):
        if os.path.exists(self._obsVCF):
            os.remove(self._obsVCF)

    def test_run_INS_DEL_100_200_SNP(self):
        expVCF = "%s/Tools/FilterVCF/expINS_DEL_100-200_SNP.vcf" % os.environ["REPET_DATA"]
        
        iFilterVCF = FilterVCF()
        iFilterVCF.setInputVCF(self._inputVCF)
        iFilterVCF.setMinInsSize(100)
        iFilterVCF.setMaxInsSize(200)
        iFilterVCF.setMinDelSize(100)
        iFilterVCF.setMaxDelSize(200)
        iFilterVCF.setVerbosity(self._verbosity)
        iFilterVCF.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCF))

    def test_run_as_script_INS_DEL_100_200_SNP(self):
        expVCF = "%s/Tools/FilterVCF/expINS_DEL_100-200_SNP.vcf" % os.environ["REPET_DATA"]
        
        cmd = "FilterVCF.py -f %s -i 100 -I 200 -d 100 -D 200 -v %s" % (self._inputVCF, self._verbosity)
        os.system(cmd)
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCF))
    
    def test_run_as_script_INS_DEL_100_200(self):
        expVCF = "%s/Tools/FilterVCF/expINS_DEL_100-200.vcf" % os.environ["REPET_DATA"]
        
        cmd = "FilterVCF.py -f %s -i 100 -I 200 -d 100 -D 200 -S -v %s" % (self._inputVCF, self._verbosity)
        os.system(cmd)
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCF))
    
    def test_run_as_script_keep_only_SNP(self):
        expVCF = "%s/Tools/FilterVCF/expSNP_only.vcf" % os.environ["REPET_DATA"]
        
        cmd = "FilterVCF.py -f %s -i 1 -I 1 -d 1 -D 1 -v %s" % (self._inputVCF, self._verbosity)
        os.system(cmd)
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCF))
    
    def test_run_as_script_keep_SNP_INS(self):
        expVCF = "%s/Tools/FilterVCF/expSNP_INS_only.vcf" % os.environ["REPET_DATA"]
        
        cmd = "FilterVCF.py -f %s -i -1 -I -1 -d 1 -D 1 -v %s" % (self._inputVCF, self._verbosity)
        os.system(cmd)
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCF))

    def test_run_inconsistent_insertions_sizes(self):
        
        iFilterVCF = FilterVCF()
        iFilterVCF.setInputVCF(self._inputVCF)
        iFilterVCF.setMinInsSize(200)
        iFilterVCF.setMaxInsSize(100)
        iFilterVCF.setMinDelSize(100)
        iFilterVCF.setMaxDelSize(200)
        iFilterVCF.setVerbosity(self._verbosity)
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            iFilterVCF.run()
        except RepetException as e:
            obsMessage = e.getMessage()
            isExceptionRaised = True
            
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, "Inconsistent values for minimum and maximum insertions sizes: '200' and '100'")

    def test_run_inconsistent_deletions_sizes(self):
        
        iFilterVCF = FilterVCF()
        iFilterVCF.setInputVCF(self._inputVCF)
        iFilterVCF.setMinInsSize(100)
        iFilterVCF.setMaxInsSize(200)
        iFilterVCF.setMinDelSize(200)
        iFilterVCF.setMaxDelSize(100)
        iFilterVCF.setVerbosity(self._verbosity)
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            iFilterVCF.run()
        except RepetException as e:
            obsMessage = e.getMessage()
            isExceptionRaised = True
            
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, "Inconsistent values for minimum and maximum deletions sizes: '200' and '100'")
        
if __name__ == "__main__":
    unittest.main()