import os
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.core.checker.RepetException import RepetException
from commons.tools.GetDeletionsFromCoverage import GetDeletionsFromCoverage

class Test_F_GetDeletionsFromCoverage(unittest.TestCase):
    
#Deletions inserted in DummyGenome_modified.fa (optimal DOC would retrieve exactly that):
#seqName    start    end    size
#DummyChr1    177515    178014    500
#DummyChr1    267548    268547    1000
#DummyChr1    370054    380053    10000
#DummyChr1    473089    523088    50000
#DummyChr1    579073    579822    750
#DummyChr1    604552    604751    200

#DummyChr2    191976    192175    200
#DummyChr2    207122    207621    500
#DummyChr2    384796    394795    10000
#DummyChr2    499878    500877    1000
#DummyChr2    529004    529753    750
#DummyChr2    543929    593928    50000
    
    def setUp(self):
        self._inputBAM = "%s/Tools/GetDeletionsFromCoverage/MutatedDummyGenome_VS_sim60x.bam" % os.environ["REPET_DATA"]
        self._inputFasta = "%s/Tools/GetDeletionsFromCoverage/DummyGenome_modified.fa" % os.environ["REPET_DATA"]
        self._obsVCF = "MutatedDummyGenome_VS_sim60x.vcf"
        self._obsVCFFileTrimmed = "MutatedDummyGenome_VS_sim60x_trimmed.vcf"
        self._verbosity = 3
        
    def tearDown(self):
        if os.path.exists(self._obsVCF):
            os.remove(self._obsVCF)
        if os.path.exists(self._obsVCFFileTrimmed):
            os.remove(self._obsVCFFileTrimmed)

    def test_run(self):
        expVCF = "%s/Tools/GetDeletionsFromCoverage/expMutatedDummyGenome_VS_sim60x.vcf" % os.environ["REPET_DATA"]
        
        iGDFC = GetDeletionsFromCoverage()
        iGDFC.setFastaFileName(self._inputFasta)
        iGDFC.setBAMFileName(self._inputBAM)
        iGDFC.setDoClean(True)
        iGDFC.setVerbosity(self._verbosity)
        iGDFC.run()
        
        os.system("grep -v '##fileDate=' %s | grep -v '##reference=' | grep -v '##assembly=' > %s" % (self._obsVCF, self._obsVCFFileTrimmed))
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCFFileTrimmed))

    def test_run_as_script(self):
        expVCF = "%s/Tools/GetDeletionsFromCoverage/expMutatedDummyGenome_VS_sim60x.vcf" % os.environ["REPET_DATA"]
        
        cmd = "GetDeletionsFromCoverage.py -i %s -b %s -c -v %s" % (self._inputFasta, self._inputBAM, self._verbosity)
        os.system(cmd)
        
        os.system("grep -v '##fileDate=' %s | grep -v '##reference=' | grep -v '##assembly=' > %s" % (self._obsVCF, self._obsVCFFileTrimmed))
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCFFileTrimmed))
        
    def test_run_as_script_other_parameters(self):
        expVCF = "%s/Tools/GetDeletionsFromCoverage/expMutatedDummyGenome_VS_sim60x_changedParams.vcf" % os.environ["REPET_DATA"]
        
        cmd = "GetDeletionsFromCoverage.py -i %s -b %s -w 100 -O 10 -s 90 -n 0.95 -c -v %s" % (self._inputFasta, self._inputBAM, self._verbosity)
        os.system(cmd)
        
        os.system("grep -v '##fileDate=' %s | grep -v '##reference=' | grep -v '##assembly=' > %s" % (self._obsVCF, self._obsVCFFileTrimmed))
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCFFileTrimmed))
        
    def test_run_as_script_other_parameters_large_window(self):
        expVCF = "%s/Tools/GetDeletionsFromCoverage/expMutatedDummyGenome_VS_sim60x_changedParams_large_window.vcf" % os.environ["REPET_DATA"]
        
        cmd = "GetDeletionsFromCoverage.py -i %s -b %s -w 1000 -O 100 -s 90 -n 0.95 -c -v %s" % (self._inputFasta, self._inputBAM, self._verbosity)
        os.system(cmd)
        
        os.system("grep -v '##fileDate=' %s | grep -v '##reference=' | grep -v '##assembly=' > %s" % (self._obsVCF, self._obsVCFFileTrimmed))
        
        self.assertTrue(FileUtils.are2FilesIdentical(expVCF, self._obsVCFFileTrimmed))
    
    def test_run_inconsistent_window(self):
        iGDFC = GetDeletionsFromCoverage()
        iGDFC.setVerbosity(self._verbosity)
        iGDFC.setBAMFileName(self._inputBAM)
        iGDFC.setFastaFileName(self._inputFasta)
        iGDFC.setDoClean(True)
        iGDFC.setWindowLength(10)
        iGDFC.setOverlap(100)
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            iGDFC.run()
        except RepetException as e:
            obsMessage = e.getMessage()
            isExceptionRaised = True
            
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, "ERROR: Windows overlaps must be lower than window size")
        
    def test_run_inconsistent_window_exact_same_boundaries(self):
        iGDFC = GetDeletionsFromCoverage()
        iGDFC.setVerbosity(self._verbosity)
        iGDFC.setBAMFileName(self._inputBAM)
        iGDFC.setFastaFileName(self._inputFasta)
        iGDFC.setDoClean(True)
        iGDFC.setWindowLength(100)
        iGDFC.setOverlap(100)
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            iGDFC.run()
        except RepetException as e:
            obsMessage = e.getMessage()
            isExceptionRaised = True
            
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, "ERROR: Windows overlaps must be lower than window size")
        
    def test_run_inconsistent_nratio(self):
        iGDFC = GetDeletionsFromCoverage()
        iGDFC.setVerbosity(self._verbosity)
        iGDFC.setBAMFileName(self._inputBAM)
        iGDFC.setFastaFileName(self._inputFasta)
        iGDFC.setDoClean(True)
        iGDFC.setThresholdN(1.1)
        
        isExceptionRaised = False
        obsMessage = ""
        try:
            iGDFC.run()
        except RepetException as e:
            obsMessage = e.getMessage()
            isExceptionRaised = True
            
        self.assertTrue(isExceptionRaised)
        self.assertEqual(obsMessage, "ERROR: maximal unknown nucleotide ratio ('-n' option) must be in [0;1]")
        
if __name__ == "__main__":
    unittest.main()