import subprocess
import unittest
import os
import glob
from commons.core.utils.FileUtils import FileUtils
from commons.core.sql.DbFactory import DbFactory
from commons.tools.PostAnalyzeTELib import PostAnalyzeTELib

class Test_F_PostAnalyzeTELib(unittest.TestCase):
    
    def setUp(self):
        self._libFileName = ""
        self._expStatFileName = "expStats.tab"
        self._obsStatFileName = ""
        self._genomeSize = 1281640
        self._pathTableName = "dummyDmelChr4_chr_allTEs_nr_noSSR_join_path"
        self._seqTableName = "dummyDmelChr4_denovoLibTEs_seq"
    
    def tearDown(self):
        lFiles = []
        if self._libFileName:
            lFiles.extend(glob.glob("%s*" % os.path.splitext(os.path.basename(self._libFileName))[0]))
        lFiles.extend(glob.glob("error.log"))
        lFiles.extend(glob.glob("blastclust.log"))
        lFiles.extend(glob.glob("expClusters.tab"))
        lFiles.extend(glob.glob("expGlobalStats.txt"))
        lFiles.extend(glob.glob(self._expStatFileName))
        lFiles.extend(glob.glob(self._obsStatFileName))
        
        for fileName in set(lFiles):
            os.remove(fileName)

    def test_run_analysis1_BlastClust(self):
        self._libFileName = "TElib.fa"
        self._writeInputFasta_analysis1(self._libFileName)
        expClusterFileName = "expClusters.tab"
        self._writeExpClusterFile_analysis1(expClusterFileName)
        expGlobalStatFileName = "expGlobalStats.txt"
        self._writeExpGlobalStats_analysis1(expGlobalStatFileName)
        self._writeExpStatsFile_analysis1(self._expStatFileName)
        obsClusterFileName = "TElib.fa_blastclust.tab"
        obsGlobalStatFileName = "TElib.fa_blastclust.globalStatsPerCluster.txt"
        self._obsStatFileName = "TElib.fa_blastclust.statsPerCluster.tab"
        
        iPATEL = PostAnalyzeTELib(analysis=1, fastaFileName=self._libFileName, doClean=True)
        iPATEL.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expClusterFileName, obsClusterFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expGlobalStatFileName, obsGlobalStatFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(self._expStatFileName, self._obsStatFileName))

    def test_run_analysis1_MCL(self):
        self._libFileName = "postAnalyzeTELibInput.fa"
        os.symlink("%s/commons/%s" % (os.environ["REPET_DATA"], self._libFileName), self._libFileName)

        expBaseFileName = "postAnalyzeTELibForMCL"
        expTabFileName = "%s/commons/exp_%s.tab" % (os.environ["REPET_DATA"], expBaseFileName)
        expStatPerCluster= "%s/commons/exp_%s.statsPerCluster.tab" % (os.environ["REPET_DATA"], expBaseFileName)
        expGlobalStatPerCluster= "%s/commons/exp_%s.globalStatsPerCluster.txt" % (os.environ["REPET_DATA"], expBaseFileName)

        obsTabFileName = "%s.tab" % os.path.splitext(self._libFileName)[0]
        obsStatPerCluster= "%s.statsPerCluster.tab" %  os.path.splitext(self._libFileName)[0]
        obsGlobalStatPerCluster= "%s.globalStatsPerCluster.txt" % os.path.splitext(self._libFileName)[0]
        
        iPATEL = PostAnalyzeTELib(analysis=1, clustMethod="MCL", fastaFileName=self._libFileName, inflate = 1.5, doClean=True, verbosity=6)
        iPATEL.run()
        self.assertTrue(FileUtils.are2FilesIdentical(expTabFileName, obsTabFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expStatPerCluster, obsStatPerCluster))
        self.assertTrue(FileUtils.are2FilesIdentical(expGlobalStatPerCluster, obsGlobalStatPerCluster))
        
    def test_run_as_script_analysis1_MCL_isClusterConsensusHeadersOption(self):
        self._libFileName = "postAnalyzeTELibInput.fa"
        os.symlink("%s/commons/%s" % (os.environ["REPET_DATA"], self._libFileName), self._libFileName)
        
        expBaseFileName = "postAnalyzeTELibForMCL"
        expTabFileName = "%s/commons/exp_%s.tab" % (os.environ["REPET_DATA"], expBaseFileName)
        expStatPerCluster= "%s/commons/exp_%s.statsPerCluster.tab" % (os.environ["REPET_DATA"], expBaseFileName)
        expGlobalStatPerCluster= "%s/commons/exp_%s.globalStatsPerCluster.txt" % (os.environ["REPET_DATA"], expBaseFileName)
        expMCLClusterFastaFileName = "%s/commons/exp_%s_MCL.fa" % (os.environ["REPET_DATA"], expBaseFileName)
        
        obsTabFileName = "%s.tab" % os.path.splitext(self._libFileName)[0]
        obsStatPerCluster= "%s.statsPerCluster.tab" %  os.path.splitext(self._libFileName)[0]
        obsGlobalStatPerCluster= "%s.globalStatsPerCluster.txt" % os.path.splitext(self._libFileName)[0]
        obsMCLClusterFastaFileName = "%s_MCL.fa" % os.path.splitext(self._libFileName)[0]

        cmd = "PostAnalyzeTELib.py -i %s -M MCL -I 1.5 -T 0.0 --isClusterConsensusHeaders -c -v 3" % self._libFileName
        process = subprocess.Popen(cmd, shell = True)
        process.communicate()
        self.assertTrue(FileUtils.are2FilesIdentical(expTabFileName, obsTabFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expStatPerCluster, obsStatPerCluster))
        self.assertTrue(FileUtils.are2FilesIdentical(expGlobalStatPerCluster, obsGlobalStatPerCluster))
        self.assertTrue(FileUtils.are2FilesIdentical(expMCLClusterFastaFileName, obsMCLClusterFastaFileName))
       
    def test_run_analysis1_MCL_isClusterConsensusHeadersOption(self):
        self._libFileName = "postAnalyzeTELibInput.fa"
        os.symlink("%s/commons/%s" % (os.environ["REPET_DATA"], self._libFileName), self._libFileName)

        expBaseFileName = "postAnalyzeTELibForMCL"
        expMCLClusterFastaFileName = "%s/commons/exp_%s_MCL.fa" % (os.environ["REPET_DATA"], expBaseFileName)
        obsMCLClusterFastaFileName = "%s_MCL.fa" % os.path.splitext(self._libFileName)[0]
        
        iPATEL = PostAnalyzeTELib(analysis=1, clustMethod="MCL", fastaFileName=self._libFileName, inflate = 1.5, doClean=True, verbosity=6, isClusterConsensusHeaders = True)
        iPATEL.run()
        self.assertTrue(FileUtils.are2FilesIdentical(expMCLClusterFastaFileName, obsMCLClusterFastaFileName))
        
    def test_run_analysis1_BlastClust_isClusterConsensusHeadersOption(self):
        self._libFileName = "postAnalyzeTELibInput.fa"
        os.symlink("%s/commons/%s" % (os.environ["REPET_DATA"], self._libFileName), self._libFileName)
        obsBlastClustClusterFastaFileName = "%s_Blastclust.fa" % os.path.basename(self._libFileName)
        expBlastClustClusterFastaFileName = "%s/commons/exp_postAnalyzeTELibForBlastClust.fa" % os.environ["REPET_DATA"]
        
        iPATEL = PostAnalyzeTELib(analysis=1, fastaFileName=self._libFileName, doClean=True, verbosity=6, isClusterConsensusHeaders = True)
        iPATEL.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expBlastClustClusterFastaFileName, obsBlastClustClusterFastaFileName))
         
    def test_run_as_script_analysis1(self):
        self._libFileName = "TElib.fa"
        with open(self._libFileName, "w") as f:
            f.write(">transib2\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">transib3\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("TGAAACTCGTCAGCGTCGTTTCCAAGTCCT\n")
            f.write(">transib4\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCAT\n")
            f.write("AAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
        expClusterFileName = "expClusters.tab"
        with open(expClusterFileName, "w") as f:
            f.write("transib4 \n")
            f.write("transib2 \n")
            f.write("transib3 \n")
        with open(self._expStatFileName, "w") as f:
            f.write("cluster\tsequencesNb\tsizeOfSmallestSeq\tsizeOfLargestSeq\taverageSize\tmedSize\n")
            f.write("1\t1\t238\t238\t238\t238\n")
            f.write("2\t1\t178\t178\t178\t178\n")
            f.write("3\t1\t90\t90\t90\t90\n")
        expGlobalStatFileName = "expGlobalStats.txt"
        with open(expGlobalStatFileName, "w") as f:
            f.write("nb of clusters: 3\n")
            f.write("nb of clusters with 1 sequence: 3\n")
            f.write("nb of clusters with 2 sequences: 0\n")
            f.write("nb of clusters with >2 sequences: 0 (0 sequences)\n")
            f.write("nb of sequences: 3\n")
            f.write("nb of sequences in the largest cluster: 1\n")
            f.write("nb of sequences in the smallest cluster: 1\n")
            f.write("size of the smallest sequence: 90\n")
            f.write("size of the largest sequence: 238\n")
            f.write("average sequences size: 168\n")
            f.write("median sequences size: 178\n")
        obsClusterFileName = "TElib.fa_blastclust.tab"
        
        obsGlobalStatFileName = "TElib.fa_blastclust.globalStatsPerCluster.txt"
        self._obsStatFileName = "TElib.fa_blastclust.statsPerCluster.tab"
        
        cmd = "PostAnalyzeTELib.py -i %s -L 98 -S 95 -b -v 3" % self._libFileName
        process = subprocess.Popen(cmd, shell = True)
        process.communicate()
      
        self.assertTrue(FileUtils.are2FilesIdentical(expClusterFileName, obsClusterFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expGlobalStatFileName, obsGlobalStatFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(self._expStatFileName, self._obsStatFileName))
       
    def test_run_analysis2(self):
        clusterFileName = "ConsensusClusters.tab"
        self._writeClusterFile_analysis2(clusterFileName)
        self._writeExpStatsFile_analysis2(self._expStatFileName)
        self._obsStatFileName = "ConsensusClusters.classifStatsPerCluster.tab"
        
        iPATEL = PostAnalyzeTELib(analysis=2, clusterFileName=clusterFileName, verbosity=3)
        iPATEL.run()
        
        os.remove(clusterFileName)
        self.assertTrue(FileUtils.are2FilesIdentical(self._expStatFileName, self._obsStatFileName))
        
    def test_run_analysis3(self):
        iDb = DbFactory.createInstance()
        iDb.createTable(self._pathTableName, "path", "%s/Tools/DmelChr4_chr_allTEs_nr_noSSR_join_path.path" % os.environ['REPET_DATA'], True)
        iDb.createTable(self._seqTableName, "seq", "%s/TEannot/DmelChr4_denovoLibTEs.fa" % os.environ['REPET_DATA'], True)
       
        expGlobalStatFileName = "expGlobalStats.txt"
        self._writeExpGlobalStats_analysis3(expGlobalStatFileName)
        self._writeExpStatsFile_analysis3(self._expStatFileName)
        obsGlobalStatFileName = "%s.globalAnnotStatsPerTE.txt" % self._pathTableName
        self._obsStatFileName = "%s.annotStatsPerTE.tab" % self._pathTableName

        iPATEL = PostAnalyzeTELib(analysis=3, pathTableName=self._pathTableName, seqTableName=self._seqTableName, genomeSize=self._genomeSize)
        iPATEL.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expGlobalStatFileName, obsGlobalStatFileName))
        os.remove(expGlobalStatFileName)
        os.remove(obsGlobalStatFileName)
        self.assertTrue(FileUtils.are2FilesIdentical(self._expStatFileName, self._obsStatFileName))
        
        iDb.dropTable(self._pathTableName)
        iDb.dropTable(self._seqTableName)
        iDb.close()

    def test_run_as_script_analysis3(self):
        iDb = DbFactory.createInstance()
        iDb.createTable(self._pathTableName, "path", "%s/Tools/DmelChr4_chr_allTEs_nr_noSSR_join_path.path" % os.environ['REPET_DATA'], True)
        iDb.createTable(self._seqTableName, "seq", "%s/TEannot/DmelChr4_denovoLibTEs.fa" % os.environ['REPET_DATA'], True)
       
        expGlobalStatFileName = "expGlobalStats.txt"
        self._writeExpGlobalStats_analysis3(expGlobalStatFileName)
        self._writeExpStatsFile_analysis3(self._expStatFileName)
        obsGlobalStatFileName = "%s.globalAnnotStatsPerTE.txt" % self._pathTableName
        self._obsStatFileName = "%s.annotStatsPerTE.tab" % self._pathTableName

        cmd = "PostAnalyzeTELib.py -a 3 -p %s -s %s -g %s -v 3" % (self._pathTableName, self._seqTableName, self._genomeSize)
        process = subprocess.Popen(cmd, shell = True)
        process.communicate()

        self.assertTrue(FileUtils.are2FilesIdentical(expGlobalStatFileName, obsGlobalStatFileName))
        os.remove(expGlobalStatFileName)
        os.remove(obsGlobalStatFileName)
        self.assertTrue(FileUtils.are2FilesIdentical(self._expStatFileName, self._obsStatFileName))
        
        iDb.dropTable(self._pathTableName)
        iDb.dropTable(self._seqTableName)
        iDb.close()

    def test_run_analysis4(self):
        iDb = DbFactory.createInstance()
        iDb.createTable(self._pathTableName, "path", "%s/Tools/DmelChr4_chr_allTEs_nr_noSSR_join_path.path" % os.environ['REPET_DATA'], True)
        iDb.createTable(self._seqTableName, "seq", "%s/TEannot/DmelChr4_denovoLibTEs.fa" % os.environ['REPET_DATA'], True)
        
        clusterFileName = "clusters.tab"
        self._writeClusterFile_analysis4(clusterFileName)
        self._writeExpStatsFile_analysis4(self._expStatFileName)
        self._obsStatFileName = "%s.annotStatsPerCluster.tab" % self._pathTableName
        
        iPATEL = PostAnalyzeTELib(analysis=4, clusterFileName=clusterFileName, pathTableName=self._pathTableName, seqTableName=self._seqTableName, genomeSize=self._genomeSize)
        iPATEL.run()
        
        self.assertTrue(FileUtils.are2FilesIdentical(self._expStatFileName, self._obsStatFileName))
        
        iDb.dropTable(self._pathTableName)
        iDb.dropTable(self._seqTableName)
        iDb.close()
        os.remove(clusterFileName)
    
    def _writeInputFasta_analysis1(self, fileName):
        with open(fileName, "w") as f:
            f.write(">DTX-incomp_DmelChr4-B-R9-Map3_reversed\n")
            f.write("CATTAGATTCAAGGCATCATGGATCAGCACATTTACACAGATATCCTGGAAAATGTGATG\n")
            f.write("CTGCCATATGCCGGGGATGAAATGCCGTTGGTTTGGACATTTCAACAGGATAACGATTCA\n")
            f.write("AAACACACGAGCAAGAAAGCTTGAAAGTGGTTTGAGCAGAAATCGATCCGAGTAATGAAA\n")
            f.write("TGGCCTGCTCTGTCATCCGACTTGAATCCAATCGAAAACCTTTGGGCGGACGTGGAAAAA\n")
            f.write(">DTX-incomp_DmelChr4-B-R10-Map3\n")
            f.write("CATTAGATTCAAGGCATCATGGATCAGCACATTTACACAGATATCCTGGAAAATGTGATG\n")
            f.write("CTGCCATATGCCGGGGATGAAATGCCGTTGGTTTGGACATTTCAACAGGATAACGATTCA\n")
            f.write("AAACACACGAGCAAGAAAGCTTGAAAGTGGTTTGAGCAGAAATCGATCCGAGTAATGAAA\n")
            f.write("TGGCCTGCTCTGTCATCCGACTTGAATCCAATCGAAAACCTTTGGGCGGACGTGGAAAAA\n")
            f.write(">PotentialHostGene-chim_DmelChr4-B-R4-Map5_reversed\n")
            f.write("TACCAAAGACACTAGAATAACAAGATGCGTAACGCCATACGATTTTTTGGCACACGATTT\n")
            f.write("TTTCGCCGTGGCTCTAGAGGTGGCTCCAGGCTCTCTCGAATTTTTGTTAGAGAGCGAGAG\n")
            f.write("AGCGGAGAGCGCTACAGCGAACAGCTCTTTTCAACGCATAAAGTGATAGCAGACAACTGT\n")

    def _writeExpClusterFile_analysis1(self, fileName):
        with open(fileName, "w") as f:
            f.write("DTX-incomp_DmelChr4-B-R10-Map3 DTX-incomp_DmelChr4-B-R9-Map3_reversed \n")
            f.write("PotentialHostGene-chim_DmelChr4-B-R4-Map5_reversed \n")
        
    def _writeExpStatsFile_analysis1(self, fileName):
    #   TODO: header in option ?
        with open(fileName, "w") as f:
            f.write("cluster\tsequencesNb\tsizeOfSmallestSeq\tsizeOfLargestSeq\taverageSize\tmedSize\n")
            f.write("1\t2\t240\t240\t240\t240\n")
            f.write("2\t1\t180\t180\t180\t180\n")
        
    def _writeExpGlobalStats_analysis1(self, fileName):
        #TODO: file or STDOUT ?
        with open(fileName, "w") as f:
            f.write("nb of clusters: 2\n")
            f.write("nb of clusters with 1 sequence: 1\n")
            f.write("nb of clusters with 2 sequences: 1\n")
            f.write("nb of clusters with >2 sequences: 0 (0 sequences)\n")
            f.write("nb of sequences: 3\n")
            f.write("nb of sequences in the largest cluster: 2\n")
            f.write("nb of sequences in the smallest cluster: 1\n")
            f.write("size of the smallest sequence: 180\n")
            f.write("size of the largest sequence: 240\n")
            f.write("average sequences size: 220\n")
            f.write("median sequences size: 240\n")

    def _writeClusterFile_analysis2(self, fileName):
        with open(fileName, "w") as f:
            f.write("DTX-incomp_DmelChr4-B-R9-Map3_reversed\tDTX-incomp_DmelChr4-B-R10-Map3_reversed\tPotentialHostGene-chim_DmelChr4-B-R4-Map5_reversed\n")
            f.write("DTX-incomp_Blc1_DmelChr4-B-R9-Map3_reversed\tDTX-incomp_Blc1_DmelChr4-B-R10-Map3\n")
            f.write("DXX-comp_DmelChr4-B-R9-Map3\tDTX-comp_DmelChr4-B-R10-Map3_reversed\tDTX-incomp_DmelChr4-B-R10-Map3_reversed\tnoCat_DmelChr4-B-G1-Map3\tnoCat_DmelChr4-B-R1-Map4\n")
            f.write("RXX-MITE_DmelChr4-B-G7-Map3\tRXX-MITE_DmelChr4-B-G5-Map3\tRXX-MITE_DmelChr4-B-G2-Map3\tRXX-MITE_DmelChr4-B-G23-Map3\tRXX-MITE_DmelChr4-B-G6-Map3\n")
        
    def _writeExpStatsFile_analysis2(self, fileName):
        #TODO: header in option ?
        with open(fileName, "w") as f:
            f.write("cluster\tnoCat\tPotentialChimeric\tcomp\tincomp\tclassifs (nbTEs)\n")
            f.write("1\t0\t1\t0\t2\tDTX (2)\tPotentialHostGene (1)\n")
            f.write("2\t0\t0\t0\t2\tDTX (2)\n")
            f.write("3\t2\t0\t2\t1\tDTX (2)\tDXX (1)\n")
            f.write("4\t0\t0\t0\t0\tMITE (5)\n")

    def _writeExpStatsFile_analysis3(self, fileName):
        #TODO: header in option ?
        with open(fileName, "w") as f:
            f.write("TE\tlength\tcovg\tfrags\tfullLgthFrags\tcopies\tfullLgthCopies\tmeanId\tsdId\tminId\tq25Id\tmedId\tq75Id\tmaxId\tmeanLgth\tsdLgth\tminLgth\tq25Lgth\tmedLgth\tq75Lgth\tmaxLgth\tmeanLgthPerc\tsdLgthPerc\tminLgthPerc\tq25LgthPerc\tmedLgthPerc\tq75LgthPerc\tmaxLgthPerc\n")
            f.write("DmelChr4-B-G1-Map3_NoCat\t542\t3701\t12\t4\t10\t4\t95.72\t1.75\t92.38\t94.66\t95.81\t96.96\t98.61\t370.10\t185.25\t72\t184.00\t442.00\t535.00\t541\t68.28\t34.18\t13.28\t33.95\t81.55\t98.71\t99.82\n")
            f.write("DmelChr4-B-G11-Map20_classII-TIR-incomp\t1240\t8216\t27\t0\t22\t0\t88.80\t7.02\t71.90\t85.57\t88.94\t94.12\t97.83\t375.00\t394.51\t46\t97.00\t196.00\t561.00\t1164\t30.24\t31.82\t3.71\t7.82\t15.81\t45.24\t93.87\n")
            f.write("DmelChr4-B-G7-Map3_classII-TIR-incomp\t1944\t15212\t49\t1\t42\t1\t89.44\t6.96\t74.20\t84.25\t90.69\t95.69\t98.69\t382.36\t415.40\t31\t99.00\t214.50\t583.00\t1946\t19.67\t21.37\t1.59\t5.09\t11.03\t29.99\t100.10\n")
            f.write("DmelChr4-B-G9-Map3_NoCat\t1590\t11564\t24\t0\t21\t1\t92.03\t7.70\t75.00\t90.00\t95.11\t97.56\t99.80\t550.67\t517.31\t26\t82.00\t264.00\t1089.00\t1590\t34.63\t32.53\t1.64\t5.16\t16.60\t68.49\t100.00\n")
            f.write("DmelChr4-B-P0.0-Map3_classII-TIR-incomp\t1042\t4001\t13\t3\t11\t3\t85.11\t10.46\t71.43\t76.53\t85.06\t94.97\t100.00\t366.36\t429.68\t58\t93.00\t129.00\t1013.00\t1042\t35.16\t41.24\t5.57\t8.93\t12.38\t97.22\t100.00\n")
            f.write("DmelChr4-B-R1-Map4_NoCat\t2367\t66031\t484\t0\t361\t0\t77.84\t5.36\t67.74\t74.17\t77.64\t80.65\t100.00\t182.91\t234.46\t32\t88.00\t123.00\t220.00\t2210\t7.73\t9.91\t1.35\t3.72\t5.20\t9.29\t93.37\n")
            f.write("DmelChr4-B-R12-Map3_NoCat\t2284\t4938\t3\t2\t3\t2\t99.26\t1.02\t98.08\t98.08\t99.82\t99.87\t99.87\t1646.00\t1109.38\t365\t365.00\t2284.00\t2289.00\t2289\t72.07\t48.57\t15.98\t15.98\t100.00\t100.22\t100.22\n")
            f.write("DmelChr4-B-R19-Map4_NoCat\t705\t3328\t10\t3\t10\t3\t88.51\t13.98\t66.09\t72.61\t96.43\t98.19\t98.86\t332.80\t301.67\t28\t28.00\t269.00\t700.00\t702\t47.21\t42.79\t3.97\t3.97\t38.16\t99.29\t99.57\n")
            f.write("DmelChr4-B-R2-Map6_NoCat\t4638\t20539\t34\t2\t29\t3\t80.93\t9.05\t67.26\t75.61\t78.69\t84.21\t99.76\t708.24\t1485.59\t36\t63.00\t89.00\t145.00\t4655\t15.27\t32.03\t0.78\t1.36\t1.92\t3.13\t100.37\n")
            f.write("DmelChr4-B-R4-Map5_NoCat\t1067\t7292\t35\t1\t28\t1\t86.50\t6.60\t73.42\t82.93\t87.81\t91.67\t97.04\t260.54\t266.97\t32\t80.00\t159.00\t422.00\t1043\t24.42\t25.02\t3.00\t7.50\t14.90\t39.55\t97.75\n")
            f.write("DmelChr4-B-R9-Map3_NoCat\t714\t5453\t19\t2\t16\t2\t81.18\t12.88\t65.63\t70.27\t75.29\t94.33\t97.85\t340.81\t208.78\t63\t216.00\t297.00\t506.00\t711\t47.73\t29.24\t8.82\t30.25\t41.60\t70.87\t99.58\n")

        
    def _writeExpGlobalStats_analysis3(self, fileName):
        with open(fileName, "w") as f:
            f.write("nb of sequences: 11\n")
            f.write("nb of matched sequences: 11\n")
            f.write("cumulative coverage: 150275 bp\n")
            f.write("coverage percentage: 11.73%\n")
            f.write("\n")
            f.write("total nb of TE fragments: 710\n")
            f.write("total nb full-length fragments: 18 (2.54%)\n")
            f.write("total nb of TE copies: 553\n")
            f.write("total nb full-length copies: 20 (3.62%)\n")
            f.write("families with full-length fragments: 8 (72.73%)\n")
            f.write(" with only one full-length fragment: 2\n")
            f.write(" with only two full-length fragments: 3\n")
            f.write(" with only three full-length fragments: 2\n")
            f.write(" with more than three full-length fragments: 1\n")
            f.write("families with full-length copies: 9 (81.82%)\n")
            f.write(" with only one full-length copy: 3\n")
            f.write(" with only two full-length copies: 2\n")
            f.write(" with only three full-length copies: 3\n")
            f.write(" with more than three full-length copies: 1\n")
            f.write("mean of median identity of all families: 88.30 +- 8.33\n")
            f.write("mean of median length percentage of all families: 30.83 +- 32.30\n")
            
    def _writeClusterFile_analysis4(self, fileName):
        with open(fileName, "w") as f:
            f.write("DmelChr4-B-R1-Map4_NoCat\tDmelChr4-B-R2-Map6_NoCat\tDmelChr4-B-R4-Map5_NoCat\n")
            f.write("DmelChr4-B-G7-Map3_classII-TIR-incomp\tDmelChr4-B-P0.0-Map3_classII-TIR-incomp\n")

    def _writeExpStatsFile_analysis4(self, fileName):
        with open(fileName, "w") as f:
            f.write("Cluster\tcovg\tfrags\tcopies\n")
            f.write("1\t93862\t553\t418\n")
            f.write("2\t19213\t62\t53\n")
            
    def _writeConfigFile(self, configFileName):
        with open(configFileName, "w") as fHandle:
            fHandle.write("[repet_env]\n")
            fHandle.write("repet_host: %s\n" % os.environ["REPET_HOST"])
            fHandle.write("repet_user: %s\n" % os.environ["REPET_USER"])
            fHandle.write("repet_pw: %s\n" % os.environ["REPET_PW"])
            fHandle.write("repet_db: %s\n" % os.environ["REPET_DB"])
            fHandle.write("repet_port: 3306\n")
            fHandle.write("[analysis1]\n")
            fHandle.write("fasta_name: %s\n" % self._expStatFileName)
            
            fHandle.write("[analysis2]\n")
            fHandle.write("clusterFileName: %s\n" % self._expStatFileName)
            
            fHandle.write("[analysis3]\n")
            fHandle.write("pathTableName: %s\n" % self._pathTableName)
            fHandle.write("seqTableName: %s\n" % self._seqTableName)
            fHandle.write("genomeSize: %s\n" % self._genomeSize)

if __name__ == "__main__":
    unittest.main()