import os
import time
import shutil
import unittest
import subprocess
from commons.core.utils.FileUtils import FileUtils
from commons.core.checker.RepetException import RepetException
from devTools.CompareTwoFastaFiles import CompareTwoFastaFiles
from commons.tools.RemoveRedundancyBasedOnCI import RemoveRedundancyBasedOnCI

class Test_F_RemoveRedundancyBasedOnCI(unittest.TestCase):
    
    def setUp(self):
        self._projectName = "test%s" % time.strftime("%d%H%M%S")
        self._inputFastaName = "%s.fa" % self._projectName
        self._inputClassifName = "%s.classif" %  self._projectName
        self._expFastaName = "exp%s_withoutRedundancy.fa" % self._projectName
        self._obsFileName = "%s_withoutRedundancy.fa" % self._projectName
        self._iRRBOC = RemoveRedundancyBasedOnCI(self._inputFastaName, self._inputClassifName, projectName = self._projectName, doClean = True)
        
    def tearDown(self):
        os.remove(self._inputFastaName)
        os.remove(self._inputClassifName)
        FileUtils.removeFilesByPattern("*withoutRedundancy.fa")
        
    def test_run_as_class(self):
        self._writeInFile()
        self._writeClassifFile()
        self._writeExpFile()
        self._iRRBOC.run()
        
        iCompare = CompareTwoFastaFiles()
        iCompare._testFileName = self._obsFileName
        iCompare._refFileName = self._expFastaName
        hasException = False
        try:
            iCompare.run()
        except RepetException:
            hasException = True
        os.remove(self._expFastaName)
        
        self.assertFalse(hasException)
        
    def test_run_as_class_with_config_wublast(self):
        configFileName = "test.cfg"
        with open(configFileName, "w") as f:
            f.write("[project]\n")
            f.write("project_name: %s\n" % self._projectName)
            f.write("[detect_features]\n")
            f.write("blast: wu\n")
            f.write("[classif_consensus]\n")
            f.write("min_redundancy_identity: 95\n")
            f.write("min_redundancy_coverage: 98\n")
            f.write("clean: yes\n")
        self._writeInFile()
        self._writeClassifFile()
        self._writeExpFile()
        self._iRRBOC.setConfigFileName(configFileName)
        self._iRRBOC.run()
        os.remove(configFileName)
        
        iCompare = CompareTwoFastaFiles()
        iCompare._testFileName = self._obsFileName
        iCompare._refFileName = self._expFastaName
        hasException = False
        try:
            iCompare.run()
        except RepetException:
            hasException = True
        os.remove(self._expFastaName)
        
        self.assertFalse(hasException)
        
    def test_run_as_class_with_config_bad_blast(self):
        configFileName = "test.cfg"
        with open(configFileName, "w") as f:
            f.write("[project]\n")
            f.write("project_name: %s\n" % self._projectName)
            f.write("[detect_features]\n")
            f.write("blast: badblast\n")
            f.write("[classif_consensus]\n")
            f.write("min_redundancy_identity: 95\n")
            f.write("min_redundancy_coverage: 98\n")
            f.write("clean: yes\n")
        self._writeInFile()
        self._writeClassifFile()
        self._iRRBOC.setConfigFileName(configFileName)
        
        hasException = False
        try:
            self._iRRBOC.run()
        except Exception:
            hasException = True
            os.chdir("..")
            shutil.rmtree(self._projectName)
        os.remove(configFileName)
        self.assertTrue(hasException)
        
    def test_run_as_class_bug(self):
        self._writeInFile_realData()
        self._writeClassifFile_realData()
        self._writeExpFile_realData()
        self._iRRBOC.run()
        
        iCompare = CompareTwoFastaFiles()
        iCompare._testFileName = self._obsFileName
        iCompare._refFileName = self._expFastaName
        hasException = False
        try:
            iCompare.run()
        except RepetException:
            hasException = True
        os.remove(self._expFastaName)
        
        self.assertFalse(hasException)
        
    def test_run_as_class_with_config_without_redundancy(self):
        configFileName = "test.cfg"
        self._writeConfigFile(configFileName)
        self._writeInFile()
        self._writeClassifFile()
        self._iRRBOC.setConfigFileName(configFileName)
        self._iRRBOC.run()
        os.remove(configFileName)
        
        iCompare = CompareTwoFastaFiles()
        iCompare._testFileName = self._obsFileName
        iCompare._refFileName = self._inputFastaName
        hasException = False
        try:
            iCompare.run()
        except RepetException:
            hasException = True
        
        self.assertFalse(hasException)
        
    def test_run_as_script_only_ok(self):
        outFileName = "RRBOCOutOk.fa"
        self._writeInFileOk()
        self._writeExpFileOk()
        self._writeClassifFileOk()
        
        cmd = "RemoveRedundancyBasedOnCI.py"
        cmd += " -c %s" % self._inputClassifName
        cmd += " -f %s" % self._inputFastaName
        cmd += " -P titi" 
        cmd += " -I 95"
        cmd += " -L 99"
        cmd += " --clean"
        cmd += " -o %s" % outFileName
        cmd += " -v 0"
        cmd += " -t 'wu'"
        process = subprocess.Popen(cmd, shell=True)
        process.communicate()
        iCompare = CompareTwoFastaFiles()
        iCompare._testFileName = outFileName
        iCompare._refFileName = self._expFastaName
        hasException = False
        try:
            iCompare.run()
        except RepetException:
            hasException = True
        os.remove(self._expFastaName)
        os.remove(outFileName)
        
        self.assertFalse(hasException)
        
    def test_run_as_script_only_chim_without_redundancy(self):
        outFileName = "RRBOCOutChim.fa"
        self._writeInFileChim()
        self._writeExpFileChim()
        self._writeClassifFileChim()
        
        cmd = "RemoveRedundancyBasedOnCI.py"
        cmd += " -c %s" % self._inputClassifName
        cmd += " -f %s" % self._inputFastaName
        cmd += " -P titi" 
        cmd += " -I 101"
        cmd += " -L 98"
        cmd += " --clean"
        cmd += " -o %s" % outFileName
        cmd += " -v 0"
        process = subprocess.Popen(cmd, shell=True)
        process.communicate()
        
        iCompare = CompareTwoFastaFiles()
        iCompare._testFileName = outFileName
        iCompare._refFileName = self._expFastaName
        hasException = False
        try:
            iCompare.run()
        except RepetException:
            hasException = True
        os.remove(self._expFastaName)
        os.remove(outFileName)
        
        self.assertFalse(hasException)
        
    def _writeConfigFile(self, configFileName):
        f = open(configFileName, "w")
        f.write("[project]\n")
        f.write("project_name: %s\n" % self._projectName)
        f.write("[detect_features]\n")
        f.write("blast: ncbi\n")
        f.write("[classif_consensus]\n")
        f.write("min_redundancy_identity: 95\n")
        f.write("min_redundancy_coverage: 101\n")#unrealistic
        f.write("clean: yes\n")
        f.close()
        
    def _writeClassifFile(self):
        with open(self._inputClassifName, "w") as f:
            f.write("transib1\t2167\t+\tok\tII\tTIR\tcomplete\tCI=55; coding=(TE_BLRtx: Transib1:classII:TIR: 100.00%; TE_BLRx: TRANSIB1p:classII:TIR: 63.81%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.16<0.95)\n")
            f.write("transib2\t2844\t+\tok\tII\tTIR\tcomplete\tCI=55; coding=(TE_BLRtx: TRANSIB2:classII:TIR: 100.00%; TE_BLRx: TRANSIB2p:classII:TIR: 100.00%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib3\t2883\t+\tok\tII\tTIR\tcomplete\tCI=55; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib4\t2883\t+\tok\tII\tTIR\tincomplete\tCI=55; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib7\t2883\t+\tok\tII\tTIR\tcomplete\tCI=20; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib8\t2883\t+\tok\tII\tTIR\tcomplete\tCI=20; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("Rt1a\t2656\t+\tPotentialChimeric\tII\tLTR\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib5\t2656\t+\tok\tII\tLTR\tincomplete\tCI=35; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib6\t2656\t+\tok\tII\tLTR\tincomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("header\t2656\t+\tok\tII\tLINE\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("BS31790\t2656\t+\tPotentialChimeric\tII\tLINE\tincomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("X-element\t4740\t+\tPotentialChimeric\tI\tLINE\tcomplete\tCI=69; coding=(TE_BLRtx: BS2:classI:LINE: 99.60%; TE_BLRx: BS3_DM-orf2p:classI:LINE: 98.14%; profiles: Exo_endo_phos_EN_11.0: 100.00%, zf-CCHC_GAG_17.9: 100.00%, RVT_1_RT_32.0: 100.00%); struct=(TElength: >700bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps GAG, >1000bps EN RT; SSR: (A)25_end); other=(SSRCoverage=0.06<0.95)\n")
            f.write("header2\t2656\t+\tok\tII\tSINE\tincomplete\tCI=45; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("header3\t2656\t+\tok\tII\tMITE\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("HELITRON2\t11435\t+\tok\tII\tHelitron\tcomplete\tCI=60; coding=(TE_BLRtx: HELITRONY2:classII:Helitron: 40.68%; TE_BLRx: ATRPA2Hp:classII:Helitron: 100.00%); struct=(TElength: >10000bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps, >1000bps); other=(SSRCoverage=0.13<0.95)\n")
            f.write("HELITRON3\t11435\t+\tok\tII\tHelitron\tcomplete\tCI=30; coding=(TE_BLRtx: HELITRONY2:classII:Helitron: 40.68%; TE_BLRx: ATRPA2Hp:classII:Helitron: 100.00%); struct=(TElength: >10000bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps, >1000bps); other=(SSRCoverage=0.13<0.95)\n")
            f.write("HELITRON4\t11435\t+\tok\tII\tHelitron\tincomplete\tCI=30; coding=(TE_BLRtx: HELITRONY2:classII:Helitron: 40.68%; TE_BLRx: ATRPA2Hp:classII:Helitron: 100.00%); struct=(TElength: >10000bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps, >1000bps); other=(SSRCoverage=0.13<0.95)\n")
            f.write("ACASINE\t334\t.\tok\tnoCat\tnoCat\tNA\tCI=NA; struct=(TElength: <700bps; TermRepeats: no termTIR, no termLTR; SSR: (ACCTTT)3_end; SSRCoverage=0.05<0.95)\n")
            f.write("ACASINE2\t62\t.\tok\tnoCat\tnoCat\tNA\tCI=NA; struct=(TElength: <700bps; TermRepeats: no termTIR, no termLTR; SSR: (ACCTTT)3_end; SSRCoverage=0.05<0.95)\n")
        
    def _writeInFile(self):
        with open(self._inputFastaName, "w") as f:
            f.write(">transib1\n")#removed, because length in classif file is not real
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">transib2\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">transib3\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("TGAAACTCGTCAGCGTCGTTTCCAAGTCCT\n")
            f.write(">transib4\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCAT\n")
            f.write("AAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write(">transib7\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCC\n")
            f.write(">transib8\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">Rt1a\n")#removed, because length in classif file is not real
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">transib5\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write(">transib6\n")
            f.write("TGGCATCTGGACAG\n")
            f.write(">header\n")
            f.write("CAATAGGCATGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATG\n")
            f.write("TCCGCCTTAATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAACCCTCCT\n")
            f.write("CGCGCGCGGGGTGCGTCTCACGCATCTGATTCCAAGTATAAGCCATGTTTCCAGGGTCCA\n")
            f.write("CGCATCACGTCATAGTAAGAGT\n")
            f.write(">BS31790\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">X-element\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write(">header2\n")
            f.write("TTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGAATAAAAAATGATTATTTG\n")
            f.write("CATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGGGATTCGTTTCATTCACCG\n")
            f.write(">header3\n")
            f.write("CACTGGCCTCGATGCTAAGACATACGGAAGTTTTTACCATGTAAAACTTAGAGTTGCGGT\n")
            f.write("ATGTAACATCGTCAGAGATAGGAGCCCGTAGGTGCCTGCAGCACGAAGGGAACTGTAGCC\n")
            f.write("AGTGTAGAGCTATGGATGATCGATAGTAACCAATATCGTTTGGCT\n")
            f.write(">HELITRON2\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">HELITRON3\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">HELITRON4\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">ACASINE\n")
            f.write("TGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATGTCCGCCTTA\n")
            f.write("ATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAA\n")
            f.write(">ACASINE2\n")
            f.write("TGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATGTCCGCCTTA\n")
            f.write("ATCAC\n")

    def _writeExpFile(self):
        with open(self._expFastaName, "w") as f:
            f.write(">transib3\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("TGAAACTCGTCAGCGTCGTTTCCAAGTCCT\n")
            f.write(">transib4\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCAT\n")
            f.write("AAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write(">transib6\n")
            f.write("TGGCATCTGGACAG\n")
            f.write(">header\n")
            f.write("CAATAGGCATGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATG\n")
            f.write("TCCGCCTTAATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAACCCTCCT\n")
            f.write("CGCGCGCGGGGTGCGTCTCACGCATCTGATTCCAAGTATAAGCCATGTTTCCAGGGTCCA\n")
            f.write("CGCATCACGTCATAGTAAGAGT\n")
            f.write(">BS31790\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">X-element\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write(">header3\n")
            f.write("CACTGGCCTCGATGCTAAGACATACGGAAGTTTTTACCATGTAAAACTTAGAGTTGCGGT\n")
            f.write("ATGTAACATCGTCAGAGATAGGAGCCCGTAGGTGCCTGCAGCACGAAGGGAACTGTAGCC\n")
            f.write("AGTGTAGAGCTATGGATGATCGATAGTAACCAATATCGTTTGGCT\n")
            f.write(">HELITRON2\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">ACASINE2\n")
            f.write("TGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATGTCCGCCTTA\n")
            f.write("ATCAC\n")
            
    def _writeClassifFileOk(self):
        with open(self._inputClassifName, "w") as f:
            f.write("transib1\t2167\t+\tok\tII\tTIR\tcomplete\tCI=55; coding=(TE_BLRtx: Transib1:classII:TIR: 100.00%; TE_BLRx: TRANSIB1p:classII:TIR: 63.81%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.16<0.95)\n")
            f.write("transib2\t2844\t+\tok\tII\tTIR\tcomplete\tCI=55; coding=(TE_BLRtx: TRANSIB2:classII:TIR: 100.00%; TE_BLRx: TRANSIB2p:classII:TIR: 100.00%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib3\t2883\t+\tok\tII\tTIR\tcomplete\tCI=55; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib4\t2883\t+\tok\tII\tTIR\tincomplete\tCI=55; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib7\t2883\t+\tok\tII\tTIR\tcomplete\tCI=20; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib8\t2883\t+\tok\tII\tTIR\tcomplete\tCI=20; coding=(TE_BLRtx: TRANSIB3:classII:TIR: 100.00%; TE_BLRx: Transib1_DPp:classII:TIR: 98.85%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR; ORF: >1000bps); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib5\t2656\t+\tok\tII\tLTR\tincomplete\tCI=35; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("transib6\t2656\t+\tok\tII\tLTR\tincomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("header\t2656\t+\tok\tII\tLINE\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("header2\t2656\t+\tok\tII\tSINE\tincomplete\tCI=45; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("header3\t2656\t+\tok\tII\tMITE\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("HELITRON2\t11435\t+\tok\tII\tHelitron\tcomplete\tCI=60; coding=(TE_BLRtx: HELITRONY2:classII:Helitron: 40.68%; TE_BLRx: ATRPA2Hp:classII:Helitron: 100.00%); struct=(TElength: >10000bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps, >1000bps); other=(SSRCoverage=0.13<0.95)\n")
            f.write("HELITRON3\t11435\t+\tok\tII\tHelitron\tcomplete\tCI=30; coding=(TE_BLRtx: HELITRONY2:classII:Helitron: 40.68%; TE_BLRx: ATRPA2Hp:classII:Helitron: 100.00%); struct=(TElength: >10000bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps, >1000bps); other=(SSRCoverage=0.13<0.95)\n")
            f.write("HELITRON4\t11435\t+\tok\tII\tHelitron\tincomplete\tCI=30; coding=(TE_BLRtx: HELITRONY2:classII:Helitron: 40.68%; TE_BLRx: ATRPA2Hp:classII:Helitron: 100.00%); struct=(TElength: >10000bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps, >1000bps); other=(SSRCoverage=0.13<0.95)\n")
            f.write("ACASINE\t334\t.\tok\tnoCat\tnoCat\tNA\tCI=NA; struct=(TElength: <700bps; TermRepeats: no termTIR, no termLTR; SSR: (ACCTTT)3_end; SSRCoverage=0.05<0.95)\n")
            f.write("ACASINE2\t62\t.\tok\tnoCat\tnoCat\tNA\tCI=NA; struct=(TElength: <700bps; TermRepeats: no termTIR, no termLTR; SSR: (ACCTTT)3_end; SSRCoverage=0.05<0.95)\n")
        
    def _writeInFileOk(self):
        with open(self._inputFastaName, "w") as f:
            f.write(">transib1\n")#removed, because length in classif file is not real
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">transib2\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">transib3\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("TGAAACTCGTCAGCGTCGTTTCCAAGTCCT\n")
            f.write(">transib4\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCAT\n")
            f.write("AAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write(">transib7\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCC\n")
            f.write(">transib8\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">transib5\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write(">transib6\n")
            f.write("TGGCATCTGGACAG\n")
            f.write(">header\n")
            f.write("CAATAGGCATGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATG\n")
            f.write("TCCGCCTTAATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAACCCTCCT\n")
            f.write("CGCGCGCGGGGTGCGTCTCACGCATCTGATTCCAAGTATAAGCCATGTTTCCAGGGTCCA\n")
            f.write("CGCATCACGTCATAGTAAGAGT\n")
            f.write(">header2\n")
            f.write("TTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGAATAAAAAATGATTATTTG\n")
            f.write("CATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGGGATTCGTTTCATTCACCG\n")
            f.write(">header3\n")
            f.write("CACTGGCCTCGATGCTAAGACATACGGAAGTTTTTACCATGTAAAACTTAGAGTTGCGGT\n")
            f.write("ATGTAACATCGTCAGAGATAGGAGCCCGTAGGTGCCTGCAGCACGAAGGGAACTGTAGCC\n")
            f.write("AGTGTAGAGCTATGGATGATCGATAGTAACCAATATCGTTTGGCT\n")
            f.write(">HELITRON2\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">HELITRON3\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">HELITRON4\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">ACASINE\n")
            f.write("TGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATGTCCGCCTTA\n")
            f.write("ATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAA\n")
            f.write(">ACASINE2\n")
            f.write("TGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATGTCCGCCTTA\n")
            f.write("ATCAC\n")
            
    def _writeExpFileOk(self):
        with open(self._expFastaName, "w") as f:
            f.write(">transib3\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("TGAAACTCGTCAGCGTCGTTTCCAAGTCCT\n")
            f.write(">transib4\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCAT\n")
            f.write("AAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write(">transib6\n")
            f.write("TGGCATCTGGACAG\n")
            f.write(">header\n")
            f.write("CAATAGGCATGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATG\n")
            f.write("TCCGCCTTAATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAACCCTCCT\n")
            f.write("CGCGCGCGGGGTGCGTCTCACGCATCTGATTCCAAGTATAAGCCATGTTTCCAGGGTCCA\n")
            f.write("CGCATCACGTCATAGTAAGAGT\n")
            f.write(">header3\n")
            f.write("CACTGGCCTCGATGCTAAGACATACGGAAGTTTTTACCATGTAAAACTTAGAGTTGCGGT\n")
            f.write("ATGTAACATCGTCAGAGATAGGAGCCCGTAGGTGCCTGCAGCACGAAGGGAACTGTAGCC\n")
            f.write("AGTGTAGAGCTATGGATGATCGATAGTAACCAATATCGTTTGGCT\n")
            f.write(">HELITRON2\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTCTC\n")
            f.write(">ACASINE2\n")
            f.write("TGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATGTCCGCCTTA\n")
            f.write("ATCAC\n")

    def _writeClassifFileChim(self):
        with open(self._inputClassifName, "w") as f:
            f.write("Rt1a\t2656\t+\tPotentialChimeric\tII\tLTR\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("BS31790\t2656\t+\tPotentialChimeric\tII\tLINE\tincomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("X-element\t4740\t+\tPotentialChimeric\tI\tLINE\tcomplete\tCI=69; coding=(TE_BLRtx: BS2:classI:LINE: 99.60%; TE_BLRx: BS3_DM-orf2p:classI:LINE: 98.14%; profiles: Exo_endo_phos_EN_11.0: 100.00%, zf-CCHC_GAG_17.9: 100.00%, RVT_1_RT_32.0: 100.00%); struct=(TElength: >700bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps GAG, >1000bps EN RT; SSR: (A)25_end); other=(SSRCoverage=0.06<0.95)\n")
        
    def _writeInFileChim(self):
        with open(self._inputFastaName, "w") as f:
            f.write(">Rt1a\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">BS31790\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">X-element\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")

    def _writeExpFileChim(self):
        with open(self._expFastaName, "w") as f:
            f.write(">Rt1a\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">BS31790\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">X-element\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")

    def _writeClassifFile_realData(self):
        with open(self._inputClassifName, "w") as f:
            f.write("fTest12185414_Blaster_Piler_1.0_Map_9\t1137\t-\tok\tII\tTIR\tcomplete\tCI=50; coding=(TE_BLRtx: PROTOP:classII:TIR: 6.70%, PROTOP_A:classII:TIR: 66.43%, PROTOP_B:classII:TIR: 6.42%); struct=(TElength: >1000bps; TermRepeats: termTIR: 52); other=(HG_BLRn: FBtr0089196_Dmel_r4.3: 51.19%; SSRCoverage=0.22)\n")
            f.write("fTest12185414_Blaster_Grouper_11_Map_20\t1240\t+\tok\tNA\tPotentialHostGene\tNA\tCI=100; coding=(HG_BLRn: FBtr0089196_Dmel_r4.3: 95.65%); other=(TE_BLRtx: PROTOP:classII:TIR: 12.03%, PROTOP_A:classII:TIR: 49.14%; TermRepeats: termTIR: 49; SSRCoverage=0.25)\n")
            
    def _writeInFile_realData(self):
        with open(self._inputFastaName, "w") as f:
            f.write(">fTest12185414_Blaster_Grouper_11_Map_20\n")
            f.write("TACCAAAGACACTACAATAACAAGATGCGTAACGGCCATACATTGGTTTGACCAAAGACC\n")
            f.write("CTAGAATAACAAGATGCGTAACGCCATACGATTTTTTGGCACACGATTTTTTGGCCGTGG\n")
            f.write("CTCTAGAGGTGGCTCCAGGCTCTCTCGAATTTTTGTTAGAGAGCGAGAGAGCGAAGAGCG\n")
            f.write("CTACAGCGAACAGCTCTTTTCAACGCACAAAGTGATAGCAGACAACTGTATGTGTGCACA\n")
            f.write("CGTATGCTCATGCATTGTAAATTTGACAAAATATGCCCTTCACCTTAGAAGTTCTTAGAC\n")
            f.write("TTTAAATCTATATTATTTTTGATCAATTGGCACCATGCGAAAATTCTATTGGCACCATGC\n")
            f.write("GATTCTTGTTTTGCATTGCCTTAACGTTATTATTATTTGAAAATAGATTAGAAATAGCCA\n")
            f.write("AATCTATGTACATATTATCACAAAAATAAATTTCAAAAATGACTTTATATAAGAATATTT\n")
            f.write("GTCATTAGAGTATTCATCTTGCGGCGTGTGAAAAATTAATAAGGCAATGATTGTTGAGTG\n")
            f.write("CTTGTGTCCGCACTTCGTGCCTCAAGATATGAACAAAGCAAAGACACTAGAATAATTCTA\n")
            f.write("GTGTATTACTTTGATATTACTTTTGCAATAAACAGTTATCATATTTTTATGAAATTTATG\n")
            f.write("AAATTACAGTAGTTATAATAATTTCTATTGTACTTCCTTTAATTATTTAGTATATTTATT\n")
            f.write("AAATCATTTGACTTAATATGATGTAACATTAACATTAAAAGTGTTTCAAAAAAAATATTT\n")
            f.write("CGCTTTTAAAAAATTGTCAGATGAGAGACAAATTAGAATTAAACATAAATATAAATGTGT\n")
            f.write("AAACGGTAGCTAATTCGAGCGGCGATTTTAACAAACAAATTTTAAAAAGCTTTAAAATTA\n")
            f.write("TAATAGTCAGGGCGCGAATTTTTAAAAATTTTTTTATTTTATCATATTGCTAGGAAATTG\n")
            f.write("GCAAAAACTACCCTAATATGTACCATGTAAATTCGTTTCTTCGATCAGAATTGATTTCGG\n")
            f.write("CCCCGAAAATCGTCTTCTAGCACAACACGCACACTTATACGCGTTCTCGTCTCTTGTTTT\n")
            f.write("TACTCACACAAGCAAGCAAATTATATTTTTAGATTTCTTACGCTCTCAGCGGGAGTGAGC\n")
            f.write("GGAAAGAGAGTAATTTTGGCCGTCACCAAAAAAGTGGCTGCATAGTGCCAAACCAATGTA\n")
            f.write("TGGCCGTTACGCATCTTGTTATTCTAGTGTCTTTGGTTTG\n")
            f.write(">fTest12185414_Blaster_Piler_1.0_Map_9\n")
            f.write("TGTTTGACCAAAGACCCTAGAATAACAAGATGCGTAACGCCATACGATTTTTTGGCACAC\n")
            f.write("GATTTTTTGGCCGTGGCTCTAGAGGTGGCTCCAGGCTCTCTCGAATTTTTGTTAGAGAGC\n")
            f.write("GAGAGAGCGAAGAGCGCTACAGCGAACAGCTCTTTTCAACGCACAAAGTGATAGCAGACA\n")
            f.write("ACTGTATGTGTGCACACGTATGCTCGCTCATGCATTGTAAATTTGACAAAATATGCCCTT\n")
            f.write("CACCTTAGAAGTTCTTAGACTTTAAATCTATATTATTTTTGATCAATTGGCACCATGCGC\n")
            f.write("TAAAAAGATTCTTGTTTTGCATTGCCTTAACGTTATTATTATTTGAAAATAGATTAGAAA\n")
            f.write("TAGCCAAATCTATGTACATATTATCACAAAAATAAATTTCAAAAATGACTTTATATAAGA\n")
            f.write("ATATTTGTCATTAGAGTATTCATCTTGCGGCGTGTGAAAAATTAATAAGGCAATGATTGT\n")
            f.write("TGAGTGCTTGTGTCCGCACTTCGTGCCTCAAGATATGAACAAAGCAAAGACACTAGAATA\n")
            f.write("ATTCTAGTGTCTTTGATATTACTTTTGCAATAAACAGTTATCATATTTAATTATTTAGTA\n")
            f.write("TATTTATTAAATCATTTGACTTAATATGATGTAACATTAACATTAAAAGTGTTTCAAAAA\n")
            f.write("AAAAATATTTCGCTTTCTTAAAAAATTGTCAGATGAGAGACAAATTAGAATTAAACATAA\n")
            f.write("ATATAAATGTGTAAACGGTAGCTAATTCGAGCGGCGATTTTAACAAACAAATTTAAAAAG\n")
            f.write("CTTTAAAATTATAATAGTCAGGGCGCGAATTTTTAAAAATTTTTTTATTTTATCATATTG\n")
            f.write("CTAGGAAATTGGCAAAAACTACCCTAATATGTACCATGTAAATTCGTTTCTTCGATCAGA\n")
            f.write("ATTGATTTCGGTCCCGAAAATCGTCTTCTAGCACAACACGCACACTTATACGCGTTCTCG\n")
            f.write("TCTCTTGTTTTTACTCACACAAGCAAGCAAATTATATTTTTAGATTTCTTACGCTCTCAG\n")
            f.write("CGGGAGTGAGCGGAAAGAGAGTAATTTTGGCCGTCACCAAAAAAGTGGCTGCATAGTGCC\n")
            f.write("AAACCAATGTATGGCCGTTACGCATCTTGTTATTCTAGTGTCTTTGGTTTTTAAACT\n")

    def _writeExpFile_realData(self):
        with open(self._expFastaName, "w") as f:
            f.write(">fTest12185414_Blaster_Grouper_11_Map_20\n")
            f.write("TACCAAAGACACTACAATAACAAGATGCGTAACGGCCATACATTGGTTTGACCAAAGACC\n")
            f.write("CTAGAATAACAAGATGCGTAACGCCATACGATTTTTTGGCACACGATTTTTTGGCCGTGG\n")
            f.write("CTCTAGAGGTGGCTCCAGGCTCTCTCGAATTTTTGTTAGAGAGCGAGAGAGCGAAGAGCG\n")
            f.write("CTACAGCGAACAGCTCTTTTCAACGCACAAAGTGATAGCAGACAACTGTATGTGTGCACA\n")
            f.write("CGTATGCTCATGCATTGTAAATTTGACAAAATATGCCCTTCACCTTAGAAGTTCTTAGAC\n")
            f.write("TTTAAATCTATATTATTTTTGATCAATTGGCACCATGCGAAAATTCTATTGGCACCATGC\n")
            f.write("GATTCTTGTTTTGCATTGCCTTAACGTTATTATTATTTGAAAATAGATTAGAAATAGCCA\n")
            f.write("AATCTATGTACATATTATCACAAAAATAAATTTCAAAAATGACTTTATATAAGAATATTT\n")
            f.write("GTCATTAGAGTATTCATCTTGCGGCGTGTGAAAAATTAATAAGGCAATGATTGTTGAGTG\n")
            f.write("CTTGTGTCCGCACTTCGTGCCTCAAGATATGAACAAAGCAAAGACACTAGAATAATTCTA\n")
            f.write("GTGTATTACTTTGATATTACTTTTGCAATAAACAGTTATCATATTTTTATGAAATTTATG\n")
            f.write("AAATTACAGTAGTTATAATAATTTCTATTGTACTTCCTTTAATTATTTAGTATATTTATT\n")
            f.write("AAATCATTTGACTTAATATGATGTAACATTAACATTAAAAGTGTTTCAAAAAAAATATTT\n")
            f.write("CGCTTTTAAAAAATTGTCAGATGAGAGACAAATTAGAATTAAACATAAATATAAATGTGT\n")
            f.write("AAACGGTAGCTAATTCGAGCGGCGATTTTAACAAACAAATTTTAAAAAGCTTTAAAATTA\n")
            f.write("TAATAGTCAGGGCGCGAATTTTTAAAAATTTTTTTATTTTATCATATTGCTAGGAAATTG\n")
            f.write("GCAAAAACTACCCTAATATGTACCATGTAAATTCGTTTCTTCGATCAGAATTGATTTCGG\n")
            f.write("CCCCGAAAATCGTCTTCTAGCACAACACGCACACTTATACGCGTTCTCGTCTCTTGTTTT\n")
            f.write("TACTCACACAAGCAAGCAAATTATATTTTTAGATTTCTTACGCTCTCAGCGGGAGTGAGC\n")
            f.write("GGAAAGAGAGTAATTTTGGCCGTCACCAAAAAAGTGGCTGCATAGTGCCAAACCAATGTA\n")
            f.write("TGGCCGTTACGCATCTTGTTATTCTAGTGTCTTTGGTTTG\n")

if __name__ == "__main__":
    unittest.main()