import os
import time
import unittest
from commons.core.sql.DbMySql import DbMySql
from commons.core.sql.TableSeqAdaptator import TableSeqAdaptator
from commons.core.sql.TablePathAdaptator import TablePathAdaptator
from commons.core.seq.Bioseq import Bioseq
from commons.core.coord.Set import Set
from commons.tools.GetSequencesFromAnnotations import GetSequencesFromAnnotations
from commons.core.coord.Path import Path
from commons.core.utils.FileUtils import FileUtils

class Test_GetSequencesFromAnnotations( unittest.TestCase ):
    
    def setUp( self ):
        self._i = GetSequencesFromAnnotations()
        self._uniqId = "%s_%s" % ( time.strftime("%Y%m%d%H%M%S") , os.getpid() )
        
        self._configFile = "%s/dummyConfigFile_%s" % ( os.getcwd(), self._uniqId )
        cfgFileHandler = open( self._configFile, "w" )
        cfgFileHandler.write( "[repet_env]\n" )
        cfgFileHandler.write( "repet_host: %s\n" % ( os.environ["REPET_HOST"] ) )
        cfgFileHandler.write( "repet_user: %s\n" % ( os.environ["REPET_USER"] ) )
        cfgFileHandler.write( "repet_pw: %s\n" % ( os.environ["REPET_PW"] ) )
        cfgFileHandler.write( "repet_db: %s\n" % ( os.environ["REPET_DB"] ) )
        cfgFileHandler.write( "repet_port: %s\n" % ( os.environ["REPET_PORT"] ) )
        cfgFileHandler.close()
        self._i._configFile = self._configFile
        self._db = DbMySql( cfgFileName=self._configFile )
        self._dummyAnnotTable = "dummyAnnotTable_%s" % ( self._uniqId )
        self._db.createTable(self._dummyAnnotTable, "path", overwrite = True)
        self._i._inCoord = self._dummyAnnotTable
        self._coordAdaptator = TablePathAdaptator( self._db, self._dummyAnnotTable )
        
        self._dummyGenomeTable = "dummyGenomeTable_%s" % ( self._uniqId )
        self._db.createTable(self._dummyGenomeTable, "seq", overwrite = True)
        self._i._inGenome = self._dummyGenomeTable
        self._tgS = TableSeqAdaptator( self._db, self._dummyGenomeTable )
        
        self._dummyRefseqTable = "dummyRefSeqTable_%s" % ( self._uniqId )
        self._db.createTable(self._dummyRefseqTable, "seq", overwrite = True)
        self._i._inRefseqs = self._dummyRefseqTable
        self._trS = TableSeqAdaptator( self._db, self._dummyRefseqTable )
        
        self._i._iDb = self._db
        
        
    def tearDown( self ):
        self._i = None
        self._uniqId = ""
        self._db.dropTable( self._dummyAnnotTable )
        self._db.dropTable( self._dummyGenomeTable )
        self._db.dropTable( self._dummyRefseqTable )
        self._db.close()
        self._db = None
        if os.path.exists( self._configFile ):
            os.remove( self._configFile )
        self._configFile = ""
        
        
    def test_saveMatches( self ):
        p = Path()
        p.setFromTuple( ("1","chr1","1","10","TE1","1","10","1e-90","130","99.2") )  # single match
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","2","6","TE1","1","7","1e-70","53","97.2") )  # chain, part 1
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","8","12","TE1","6","10","1e-70","55","97.8") )  # chain, part 2
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("3","chr2","1","6","TE1","31","36","1e-40","43","92.8") )  # match, other query
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("4","chr1","1","10","TE2","8","12","1e-65","135","96.5") )  # match, other subject
        self._coordAdaptator.insert( p )
        self._i._coordAdaptator = self._coordAdaptator
        
        s = Bioseq()
        s.header = "chr1"
        s.sequence = "AAAAATTTTTCCCCCCCCCC"  # 20 bp
        self._tgS.insert( s )
        s.header = "chr2"
        s.sequence = "AAAAACCCCCGGGGGTTTTTTTTTT"  # 25 bp
        self._tgS.insert( s )
        self._i._genomeAdaptator = self._tgS
        
        expFile = "dummyExpFile_%s"  %( self._uniqId )
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">TE1::1 chr1 1..10\n" )
        expFileHandler.write( "AAAAATTTTT\n" )
        expFileHandler.write( ">TE1::2 chr1 2..6\n" )
        expFileHandler.write( "AAAAT\n" )
        expFileHandler.write( ">TE1::2 chr1 8..12\n" )
        expFileHandler.write( "TTTCC\n" )
        expFileHandler.write( ">TE1::3 chr2 1..6\n" )
        expFileHandler.write( "AAAAAC\n" )
        expFileHandler.close()
        
        obsFile = "TE1_matches.fa"
        self._i._outFile = obsFile
        
        self._i.saveMatches( "TE1", 20 )
        
        self.assertTrue( os.path.exists( obsFile ) )
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        
        for f in [ expFile, obsFile ]:
            if os.path.exists( f ):
                os.remove( f )
                
                
    def test_saveMatches_minProp( self ):
        p = Path()
        p.setFromTuple( ("1","chr1","1","10","TE1","1","10","1e-90","130","99.2") )  # single match
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","2","6","TE1","1","7","1e-70","53","97.2") )  # chain, part 1
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","8","12","TE1","6","10","1e-70","55","97.8") )  # chain, part 2
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("3","chr2","1","6","TE1","31","36","1e-40","43","92.8") )  # match, other query
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("4","chr1","1","10","TE2","8","12","1e-65","135","96.5") )  # match, other subject
        self._coordAdaptator.insert( p )
        self._i._coordAdaptator = self._coordAdaptator
        
        s = Bioseq()
        s.header = "chr1"
        s.sequence = "AAAAATTTTTCCCCCCCCCC"  # 20 bp
        self._tgS.insert( s )
        s.header = "chr2"
        s.sequence = "AAAAACCCCCGGGGGTTTTTTTTTT"  # 25 bp
        self._tgS.insert( s )
        self._i._genomeAdaptator = self._tgS
        
        s.header = "TE1"
        s.sequence = "GCAGTAGCGA"  # 10 bp
        self._trS.insert( s )
        self._i._refseqsAdaptator = self._trS
        
        expFile = "dummyExpFile_%s"  %( self._uniqId )
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">TE1::1 chr1 1..10\n" )
        expFileHandler.write( "AAAAATTTTT\n" )
        expFileHandler.close()
        
        obsFile = "TE1_matches.fa"
        self._i._outFile = obsFile
        
        self._i._minProp = 0.9
        self._i.saveMatches( "TE1", 10 )
        
        self.assertTrue( os.path.exists( obsFile ) )
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        
        for f in [ expFile, obsFile ]:
            if os.path.exists( f ):
                os.remove( f )
                
                
    def test_saveMatches_minProp_andOutputAnnot( self ):
        p = Path()
        p.setFromTuple( ("1","chr1","1","10","TE1","1","10","1e-90","130","99.2") )  # single match
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","2","6","TE1","1","7","1e-70","53","97.2") )  # chain, part 1
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","8","12","TE1","6","10","1e-70","55","97.8") )  # chain, part 2
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("3","chr2","1","6","TE1","31","36","1e-40","43","92.8") )  # match, other query
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("4","chr1","1","10","TE2","8","12","1e-65","135","96.5") )  # match, other subject
        self._coordAdaptator.insert( p )
        self._i._coordAdaptator = self._coordAdaptator
        
        s = Bioseq()
        s.header = "chr1"
        s.sequence = "AAAAATTTTTCCCCCCCCCC"  # 20 bp
        self._tgS.insert( s )
        s.header = "chr2"
        s.sequence = "AAAAACCCCCGGGGGTTTTTTTTTT"  # 25 bp
        self._tgS.insert( s )
        self._i._genomeAdaptator = self._tgS
        
        s.header = "TE1"
        s.sequence = "GCAGTAGCGA"  # 10 bp
        self._trS.insert( s )
        self._i._refseqsAdaptator = self._trS
        
        expFile = "dummyExpFile"
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">TE1::1 chr1 1..10\n" )
        expFileHandler.write( "AAAAATTTTT\n" )
        expFileHandler.close()
        
        expPathFile = "dummyExpPathFile_1"
        expPathFileHandler = open( expPathFile, "w" )
        expPathFileHandler.write( "1\tchr1\t1\t10\tTE1\t1\t10\t1e-90\t130\t99.200000\n" )
        expPathFileHandler.close()
        
        obsFile = "TE1_matches.fa"
        self._i._outFile = obsFile
        
        obsPathFile = "dummyObsPathFile_1"
        self._i._outCoordFile = obsPathFile
        self._i._outCoordFileHandler = open( self._i._outCoordFile, "w" )
        
        self._i._getAnnot = True
        self._i._minProp = 0.9
        self._i.saveMatches( "TE1", 10 )
        
        self.assertTrue( os.path.exists( obsFile ) )
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        self.assertTrue( FileUtils.are2FilesIdentical( expPathFile, obsPathFile ) )
        
        for f in [ expFile, obsFile, expPathFile, obsPathFile ]:
            if os.path.exists( f ):
                os.remove( f )
                
                
    def test_saveChains( self ):
        p = Path()
        p.setFromTuple( ("1","chr1","1","10","TE1","1","10","1e-90","130","99.2") )  # single match
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","2","6","TE1","1","7","1e-70","53","97.2") )  # chain, part 1
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","8","12","TE1","6","10","1e-70","55","97.8") )  # chain, part 2
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("3","chr2","1","6","TE1","1","6","1e-40","43","92.8") )  # match, other query
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("4","chr1","1","10","TE2","8","12","1e-65","135","96.5") )  # match, other subject
        self._coordAdaptator.insert( p )
        self._i._coordAdaptator = self._coordAdaptator
        
        s = Bioseq()
        s.header = "chr1"
        s.sequence = "AAAAATTTTTCCCCCCCCCC"  # 20 bp
        self._tgS.insert( s )
        s.header = "chr2"
        s.sequence = "AAAAACCCCCGGGGGTTTTTTTTTT"  # 25 bp
        self._tgS.insert( s )
        self._i._genomeAdaptator = self._tgS
        
        expFile = "dummyExpFile_%s"  %( self._uniqId )
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">TE1::1 chr1 1..10\n" )
        expFileHandler.write( "AAAAATTTTT\n" )
        expFileHandler.write( ">TE1::2 chr1 2..6,8..12\n" )
        expFileHandler.write( "AAAATTTTCC\n" )
        expFileHandler.write( ">TE1::3 chr2 1..6\n" )
        expFileHandler.write( "AAAAAC\n" )
        expFileHandler.close()
        
        obsFile = "TE1_chains.fa"
        self._i._outFile = obsFile
        
        self._i.saveChains( "TE1", 10 )
        
        self.assertTrue( os.path.exists( obsFile ) )
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        
        for f in [ expFile, obsFile ]:
            if os.path.exists( f ):
                os.remove( f )
                
                
    def test_saveChains_andOutputAnnot( self ):
        p = Path()
        p.setFromTuple( ("1","chr1","1","10","TE1","1","10","1e-90","130","99.2") )  # single match
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","2","6","TE1","1","7","1e-70","53","97.2") )  # chain, part 1
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","8","12","TE1","6","10","1e-70","55","97.8") )  # chain, part 2
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("3","chr2","1","6","TE1","1","6","1e-40","43","92.8") )  # match, other query
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("4","chr1","1","10","TE2","8","12","1e-65","135","96.5") )  # match, other subject
        self._coordAdaptator.insert( p )
        self._i._coordAdaptator = self._coordAdaptator
        
        s = Bioseq()
        s.header = "chr1"
        s.sequence = "AAAAATTTTTCCCCCCCCCC"  # 20 bp
        self._tgS.insert( s )
        s.header = "chr2"
        s.sequence = "AAAAACCCCCGGGGGTTTTTTTTTT"  # 25 bp
        self._tgS.insert( s )
        self._i._genomeAdaptator = self._tgS
        
        expFile = "dummyExpFile"
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">TE1::1 chr1 1..10\n" )
        expFileHandler.write( "AAAAATTTTT\n" )
        expFileHandler.write( ">TE1::2 chr1 2..6,8..12\n" )
        expFileHandler.write( "AAAATTTTCC\n" )
        expFileHandler.write( ">TE1::3 chr2 1..6\n" )
        expFileHandler.write( "AAAAAC\n" )
        expFileHandler.close()
        
        expPathFile = "dummyExpPathFile"
        expPathFileHandler = open( expPathFile, "w" )
        expPathFileHandler.write( "1\tchr1\t1\t10\tTE1\t1\t10\t1e-90\t130\t99.200000\n" )
        expPathFileHandler.write( "2\tchr1\t2\t6\tTE1\t1\t7\t1e-70\t53\t97.200000\n" )
        expPathFileHandler.write( "2\tchr1\t8\t12\tTE1\t6\t10\t1e-70\t55\t97.800000\n" )
        expPathFileHandler.write( "3\tchr2\t1\t6\tTE1\t1\t6\t1e-40\t43\t92.800000\n" )
        expPathFileHandler.close()
        
        obsFile = "TE1_chains.fa"
        self._i._outFile = obsFile
        
        obsPathFile = "dummyObsPathFile"
        self._i._outCoordFile = obsPathFile
        
        self._i._getAnnot = True
        self._i.saveChains( "TE1", 10 )
        
        self.assertTrue( os.path.exists( obsFile ) )
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        self.assertTrue( FileUtils.are2FilesIdentical( expPathFile, obsPathFile ) )
        
        for f in [ expFile, obsFile, expPathFile, obsPathFile ]:
            if os.path.exists( f ):
                os.remove( f )
                
                
    def test_zRunAsScript_pathFormat_withTables_chains( self ):
        cDir = os.getcwd()
        
        p = Path()
        p.setFromTuple( ("1","chr1","1","10","TE1","1","10","1e-90","130","99.2") )  # single match
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","2","6","TE1","1","7","1e-70","53","97.2") )  # chain, part 1
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("2","chr1","8","12","TE1","6","10","1e-70","55","97.8") )  # chain, part 2
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("3","chr2","1","6","TE1","31","36","1e-40","43","92.8") )  # match, other query
        self._coordAdaptator.insert( p )
        p.setFromTuple( ("4","chr1","1","10","TE2","8","12","1e-65","135","96.5") )  # match, other subject
        self._coordAdaptator.insert( p )
        self._i._pathA = self._coordAdaptator
        
        s = Bioseq()
        s.header = "chr1"
        s.sequence = "AAAAATTTTTCCCCCCCCCC"  # 20 bp
        self._tgS.insert( s )
        s.header = "chr2"
        s.sequence = "AAAAACCCCCGGGGGTTTTTTTTTT"  # 25 bp
        self._tgS.insert( s )
        self._i._genomeAdaptator = self._tgS
        
        s.header = "TE1"
        s.sequence = "GCAGTAGCGACGCATCGCAGCAGCTACG"
        self._trS.insert( s )
        self._i._refseqsAdaptator = self._trS
        
        cmd = "python ../GetSequencesFromAnnotations.py"
        cmd += " -i %s" % ( self._dummyAnnotTable )
        cmd += " -g %s" % ( self._dummyGenomeTable )
        cmd += " -s %s"  %( self._dummyRefseqTable )
        cmd += " -r %s" % ( "TE1" )
        cmd += " -d %s" % ( "chains" )
        cmd += " -C %s" % ( self._configFile )
        cmd += " -v 0"
        exitStatus = os.system( cmd )
        self.assertTrue( exitStatus == 0 )
        
        for f in [ "TE1_matches.fa", "TE1_chains.fa" ]:
            if os.path.exists( f ):
                os.remove( f )
        os.chdir( cDir )
        
        
    def test_zRunAsScript_pathFormat_withFiles_chains( self ):
        cDir = os.getcwd()
        
        dummyAnnotFile = "dummyAnnotFile.path"
        p = Path()
        p.setFromTuple( ("1","chr1","1","10","TE1","1","10","1e-90","130","99.2") )  # single match
        p.save( dummyAnnotFile )
        p.setFromTuple( ("2","chr1","2","6","TE1","1","7","1e-70","53","97.2") )  # chain, part 1
        p.save( dummyAnnotFile )
        p.setFromTuple( ("2","chr1","8","12","TE1","6","10","1e-70","55","97.8") )  # chain, part 2
        p.save( dummyAnnotFile )
        p.setFromTuple( ("3","chr2","1","6","TE1","31","36","1e-40","43","92.8") )  # match, other query
        p.save( dummyAnnotFile )
        p.setFromTuple( ("4","chr1","1","10","TE2","8","12","1e-65","135","96.5") )  # match, other subject
        p.save( dummyAnnotFile )
        
        dummyGenomeFile = "dummyGenomeFile.fa"
        if os.path.exists( dummyGenomeFile ):
            os.remove( dummyGenomeFile )
        s = Bioseq( "chr1", "AAAAATTTTTCCCCCCCCCC" )  # 20 bp
        s.save( dummyGenomeFile )
        s = Bioseq( "chr2", "AAAAACCCCCGGGGGTTTTTTTTTT" )  # 25 bp
        s.save( dummyGenomeFile )
        
        dummyRefseqFile = "dummyRefseqFile.fa"
        if os.path.exists( dummyRefseqFile ):
            os.remove( dummyRefseqFile )
        s = Bioseq( "TE1", "GCAGTAGCGACGCATCGCAGCAGCTACG" )
        s.save( dummyRefseqFile )
        
        cmd = "GetSequencesFromAnnotations.py"
        cmd += " -i %s" % ( dummyAnnotFile )
        cmd += " -g %s" % ( dummyGenomeFile )
        cmd += " -s %s"  %( dummyRefseqFile )
        cmd += " -r %s" % ( "TE1" )
        cmd += " -d %s" % ( "chains" )
        cmd += " -C %s" % ( self._configFile )
        cmd += " -v 2"
        exitStatus = os.system( cmd )
        self.assertTrue( exitStatus == 0 )
        
        for f in [ dummyAnnotFile, dummyGenomeFile, dummyRefseqFile, "TE1_matches.fa", "TE1_chains.fa" ]:
            if os.path.exists( f ):
                os.remove( f )
        os.chdir( cDir )
        
        
    def test_zRunAsScript_setFormat_withFiles_chains_withRefseqs( self ):
        cDir = os.getcwd()
        
        dummyAnnotFile = "dummyAnnotFile.set"
        if os.path.exists( dummyAnnotFile ):
            os.remove( dummyAnnotFile )
        iSet = Set()
        iSet.setFromTuple( ("1","TE1","chr1","1","5") )  # single match
        iSet.save( dummyAnnotFile )
        iSet.setFromTuple( ("2","TE2","chr1","11","15") )  # chain, part 1
        iSet.save( dummyAnnotFile )
        iSet.setFromTuple( ("2","TE2","chr1","18","20") )  # chain, part 2
        iSet.save( dummyAnnotFile )
        
        dummyGenomeFile = "dummyGenomeFile.fa"
        if os.path.exists( dummyGenomeFile ):
            os.remove( dummyGenomeFile )
        iBs = Bioseq( "chr1", "AAAAATTTTTGGGGGAAGGG" )  # 20 bp
        iBs.save( dummyGenomeFile )
        iBs = Bioseq( "chr2", "CCCCCCCCCCCCCCCCCCCCCCCCC" )  # 25 bp
        iBs.save( dummyGenomeFile )
        
        dummyRefseqFile = "dummyRefseqFile.fa"
        if os.path.exists( dummyRefseqFile ):
            os.remove( dummyRefseqFile )
        s = Bioseq( "TE1", "GCAGTAGCGACGCATCGCAGCAGCTACG" )
        s.save( dummyRefseqFile )
        s = Bioseq( "TE2", "TTAGCGTGACTTTCAGTGCATGTGT" )
        s.save( dummyRefseqFile )
        
        expFile = "dummyExpFile.fa"
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">TE2::2 chr1 11..15,18..20\n" )
        expFileHandler.write( "GGGGGGGG\n" )
        expFileHandler.close()
        
        obsFile = "TE2_chains.fa"
        if os.path.exists( obsFile ):
            os.remove( obsFile )
            
        cmd = "python ../GetSequencesFromAnnotations.py"
        cmd += " -i %s" % ( dummyAnnotFile )
        cmd += " -f %s" % ( "set" )
        cmd += " -g %s" % ( dummyGenomeFile )
        cmd += " -s %s"  %( dummyRefseqFile )
        cmd += " -r %s" % ( "TE2" )
        cmd += " -d %s" % ( "chains" )
        cmd += " -C %s" % ( self._configFile )
        cmd += " -v %i" % ( 0 )
        os.system( cmd )
        
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        
        self._db.dropTable("dummyGenomeFile_fa")
        for f in [ dummyAnnotFile, dummyGenomeFile, dummyRefseqFile, expFile, obsFile ]:
            if os.path.exists( f ):
                os.remove( f )
        os.chdir( cDir )
        
        
    def test_zRunAsScript_setFormat_withFiles_chains_withoutRefseqFile( self ):
        cDir = os.getcwd()
        
        dummyAnnotFile = "dummyAnnotFile.set"
        if os.path.exists( dummyAnnotFile ):
            os.remove( dummyAnnotFile )
        iSet = Set()
        iSet.setFromTuple( ("1","Mb1Gr1Cl0","chr1","1","5") )  # single match
        iSet.save( dummyAnnotFile )
        iSet.setFromTuple( ("2","Mb2Gr1Cl0","chr1","11","15") )  # chain, part 1
        iSet.save( dummyAnnotFile )
        iSet.setFromTuple( ("2","Mb2Gr1Cl0","chr1","18","20") )  # chain, part 2
        iSet.save( dummyAnnotFile )
        
        dummyGenomeFile = "dummyGenomeFile.fa"
        if os.path.exists( dummyGenomeFile ):
            os.remove( dummyGenomeFile )
        iBs = Bioseq( "chr1", "AAAAATTTTTGGGGGAAGGG" )  # 20 bp
        iBs.save( dummyGenomeFile )
        iBs = Bioseq( "chr2", "CCCCCCCCCCCCCCCCCCCCCCCCC" )  # 25 bp
        iBs.save( dummyGenomeFile )
        
        expFile = "dummyExpFile.fa"
        expFileHandler = open( expFile, "w" )
        expFileHandler.write( ">Mb2Gr1Cl0::2 chr1 11..15,18..20\n" )
        expFileHandler.write( "GGGGGGGG\n" )
        expFileHandler.close()
        
        obsFile = "Mb2Gr1Cl0_chains.fa"
        if os.path.exists( obsFile ):
            os.remove( obsFile )
            
        cmd = "python ../GetSequencesFromAnnotations.py"
        cmd += " -i %s" % ( dummyAnnotFile )
        cmd += " -f %s" % ( "set" )
        cmd += " -g %s" % ( dummyGenomeFile )
        cmd += " -r %s" % ( "Mb2Gr1Cl0" )
        cmd += " -d %s" % ( "chains" )
        cmd += " -C %s" % ( self._configFile )
        cmd += " -v %i" % ( 0 )
        os.system( cmd )
        
        self.assertTrue( FileUtils.are2FilesIdentical( expFile, obsFile ) )
        
        self._db.dropTable("dummyGenomeFile_fa")
        for f in [ dummyAnnotFile, dummyGenomeFile, expFile, obsFile ]:
            if os.path.exists( f ):
                os.remove( f )
        os.chdir( cDir )
        
if __name__ == "__main__":
        unittest.main()