import os
import unittest
from commons.core.utils.FileUtils import FileUtils
from commons.tools.RemoveRedundancyBasedOnCI import RemoveRedundancyBasedOnCI

class Test_RemoveRedundancyBasedOnCI(unittest.TestCase):
    
    def setUp(self):
        self._projectName = "test"
    
    def test_splitFastaFileByChimericTag(self):
        inputClassifName = "%s.classif" % self._projectName
        self._writeClassifFile(inputClassifName)
        inputFastaName = "%s.fa" % self._projectName
        self._writeFastaFile(inputFastaName)
        expChimFileName = "expChim.fa"
        self._writeExpChimFile(expChimFileName)
        expNotChimFileName = "expNotChim.fa"
        self._writeExpNotChimFile(expNotChimFileName)
        obsChimFileName = "%s_potentialChimeric.fa" % self._projectName
        obsNotChimFileName = "%s_notPotentialChimeric.fa" % self._projectName
        
        iRRBOC = RemoveRedundancyBasedOnCI(inputFastaName, inputClassifName, projectName = self._projectName)
        iRRBOC.splitFastaFileByChimericTag()
        
        self.assertTrue(FileUtils.are2FilesIdentical(expChimFileName, obsChimFileName))
        self.assertTrue(FileUtils.are2FilesIdentical(expNotChimFileName, obsNotChimFileName))
        os.remove(inputClassifName)
        os.remove(inputFastaName)
        os.remove(expChimFileName)
        os.remove(expNotChimFileName)
        os.remove(obsChimFileName)
        os.remove(obsNotChimFileName)
        
    def test_createHeader2CIAndLengthDictFromClassifFile(self):
        inputClassifName = "%s.classif" % self._projectName
        self._writeClassifFile(inputClassifName)
        expDHeader2CIAndLength = {'Rt1a': (55, 2656),
                                  'header': (55, 2656),
                                  'BS31790': (-1, 2656),
                                  'X-element': (69, 4740),
                                  'header2': (45, 2656),
                                  'header3': (55, 2656)}
        expDHeader2IsKept = {'Rt1a': True,
                             'header': True,
                             'BS31790': True,
                             'X-element': True,
                             'header2': True,
                             'header3': True}
        iRRBOC = RemoveRedundancyBasedOnCI(inputClassifName = inputClassifName, projectName = self._projectName)
        iRRBOC.createHeader2CIAndLengthDictFromClassifFile()
        obsDHeader2CIAndLength = iRRBOC.getHeader2CIAndLengthDict()
        obsDHeader2IsKept = iRRBOC.getHeader2IsKeptDict()
        os.remove(inputClassifName)
        self.assertEquals(expDHeader2CIAndLength, obsDHeader2CIAndLength)
        self.assertEquals(expDHeader2IsKept, obsDHeader2IsKept)
        
    def test_createFastaAccordingToHeadersDict(self):
        dHeader2IsKept = {'Rt1a': True,
                             'header': True,
                             'BS31790': False,
                             'X-element': True,
                             'header2': False,
                             'header3': True}
        inputFastaName = "%s.fa" % self._projectName
        self._writeFastaFile(inputFastaName)
        expFileName = "exp_withoutRedundancy.fa"
        self._writeExpFileWithoutRdd(expFileName)
        obsFileName = "%s_withoutRedundancy.fa" % self._projectName
        iRRBOC = RemoveRedundancyBasedOnCI(inputFastaName, projectName = self._projectName)
        iRRBOC.setHeader2IsKeptDict(dHeader2IsKept)
        iRRBOC.createFastaAccordingToHeadersDict()
        self.assertTrue(FileUtils.are2FilesIdentical(expFileName, obsFileName))
        os.remove(inputFastaName)
        os.remove(expFileName)
        os.remove(obsFileName)

    def test_findSequencesNotKeptFromTabFileAndCIAndLength_keep_all(self):
        dHeader2CIAndLength = {"case1Q_118_40" : (40, 118),
                               "case1S_120_50" : (50, 120),
                               "case1bisQ_120_40" : (40, 120),
                               "case1bisS_118_50" : (50, 118)}
        inFileName = "%s_notPotentialChimeric.fa.align.match.tab" % self._projectName
        self._writeTabFile_keep_all(inFileName)
        dHeader2IsKept =    {"case1Q_118_40" : True,
                             "case1S_120_50" : True,
                             "case1bisQ_120_40" : True,
                             "case1bisS_118_50" : True}
        expDHeader2IsKept = {"case1Q_118_40" : True,
                             "case1S_120_50" : True,
                             "case1bisQ_120_40" : True,
                             "case1bisS_118_50" : True}
        iRRBOC = RemoveRedundancyBasedOnCI()
        iRRBOC.setHeader2IsKeptDict(dHeader2IsKept)
        iRRBOC.setHeader2CIAndLengthDict(dHeader2CIAndLength)
        iRRBOC.findSequencesNotKeptFromTabFileAndCIAndLength(inFileName)
        obsDHeader2IsKept = iRRBOC.getHeader2IsKeptDict()
        self.assertEquals(expDHeader2IsKept, obsDHeader2IsKept)
        os.remove(inFileName)

    def test_findSequencesNotKeptFromTabFileAndCIAndLength(self):
        dHeader2CIAndLength = {"case1Q_118_40" : (40, 118),
                               "case1S_120_50" : (50, 120),
                               "case1bisQ_120_40" : (40, 120),
                               "case1bisS_118_50" : (50, 118),
                               "case2Q_118_40" : (40, 118),
                               "case2S_120_40" : (40, 120),
                               "case2bisQ_120_40" : (40, 120),
                               "case2bisS_120_40" : (40, 120),
                               "case3Q_120_40" : (40, 120),
                               "case3S_118_40" : (40, 118),
                               "case7Q_120_50" : (50, 120),
                               "case7S_200_40" : (40, 200),
                               "case7bisQ_200_40" : (40, 200),
                               "case7bisS_118_50" : (50, 118),
                               "case4Q_120_50" : (50, 120),
                               "case4S_118_40" : (40, 118),
                               "case4bisQ_118_50" : (50, 118),
                               "case4bisS_120_40" : (40, 120),
                               "case5Q_200_40" : (40, 120),
                               "case5S_118_40" : (40, 118),
                               "case5bisQ_120_40" : (40, 120),
                               "case5bisS_120_40" : (40, 120),
                               "case5bisDummyQ_120_40" : (40, 120),
                               "case5bisDummyS_120_40" : (40, 120),
                               "case6Q_118_40" : (40, 118),
                               "case6S_120_40" : (40, 120),
                               "case6DummyQ_118_40" : (40, 118),
                               "case6DummyS_120_40" : (40, 120)}
        inFileName = "%s_notPotentialChimeric.fa.align.match.tab" % self._projectName
        self._writeTabFile(inFileName)
        dHeader2IsKept =    {"case1Q_118_40" : True,
                             "case1S_120_50" : True,
                             "case1bisQ_120_40" : True,
                             "case1bisS_118_50" : True,
                             "case2Q_118_40" : True,
                             "case2S_120_40" : True,
                             "case2bisQ_120_40" : True,
                             "case2bisS_120_40" : True,
                             "case3Q_120_40" : True,
                             "case3S_118_40" : True,
                             "case7Q_120_50" : True,
                             "case7S_200_40" : True,
                             "case7bisQ_200_40" : True,
                             "case7bisS_118_50" : True,
                             "case4Q_120_50" : True,
                             "case4S_118_40" : True,
                             "case4bisQ_118_50" : True,
                             "case4bisS_120_40" : True,
                             "case5Q_200_40" : True,
                             "case5S_118_40" : True,
                             "case5bisQ_120_40" : True,
                             "case5bisS_120_40" : True,
                             "case5bisDummyQ_120_40" : True,
                             "case5bisDummyS_120_40" : True,
                             "case6Q_118_40" : True,
                             "case6S_120_40" : True,
                             "case6DummyQ_118_40" : True,
                             "case6DummyS_120_40" : True}
        expDHeader2IsKept = {"case1Q_118_40" : False,
                             "case1S_120_50" : True,
                             "case1bisQ_120_40" : False,
                             "case1bisS_118_50" : True,
                             "case2Q_118_40" : False,
                             "case2S_120_40" : True,
                             "case2bisQ_120_40" : False,
                             "case2bisS_120_40" : True,
                             "case3Q_120_40" : True,
                             "case3S_118_40" : False,
                             "case7Q_120_50" : True,
                             "case7S_200_40" : True,
                             "case7bisQ_200_40" : True,
                             "case7bisS_118_50" : True,
                             "case4Q_120_50" : True,
                             "case4S_118_40" : False,
                             "case4bisQ_118_50" : True,
                             "case4bisS_120_40" : False,
                             "case5Q_200_40" : True,
                             "case5S_118_40" : False,
                             "case5bisQ_120_40" : False,
                             "case5bisS_120_40" : True,
                             "case5bisDummyQ_120_40" : True,
                             "case5bisDummyS_120_40" : False,
                             "case6Q_118_40" : False,
                             "case6S_120_40" : True,
                             "case6DummyQ_118_40" : False,
                             "case6DummyS_120_40" : True}
        iRRBOC = RemoveRedundancyBasedOnCI()
        iRRBOC.setHeader2IsKeptDict(dHeader2IsKept)
        iRRBOC.setHeader2CIAndLengthDict(dHeader2CIAndLength)
        iRRBOC.findSequencesNotKeptFromTabFileAndCIAndLength(inFileName)
        obsDHeader2IsKept = iRRBOC.getHeader2IsKeptDict()
        self.assertEquals(expDHeader2IsKept, obsDHeader2IsKept)
        os.remove(inFileName)
        
    def _writeTabFile(self, fileName):
        with open(fileName, "w") as f:
            f.write("query.name\tquery.start\tquery.end\tquery.length\tquery.length.%\tmatch.length.%\tsubject.name\tsubject.start\tsubject.end\tsubject.length\tsubject.length.%\tE.value\tScore\tIdentity\tpath\n")
            f.write("case1Q_118_40\t1\t118\t118\t0.983333\t1\tcase1S_120_50\t1\t120\t120\t1\t7e-67\t234\t100\t1\n")
            f.write("case1bisQ_120_40\t1\t118\t118\t0.983333\t1\tcase1bisS_118_50\t1\t118\t118\t1\t7e-67\t234\t100\t2\n")
            f.write("case2Q_118_40\t1\t118\t118\t1\t1\tcase2S_120_40\t1\t120\t120\t1\t7e-67\t234\t100\t3\n")
            f.write("case2bisQ_120_40\t1\t118\t118\t0.983333\t1\tcase2bisS_120_40\t1\t120\t120\t1\t7e-67\t234\t100\t4\n")
            f.write("case3Q_120_40\t1\t118\t118\t0.983333\t1\tcase3S_118_40\t1\t118\t118\t1\t7e-67\t234\t100\t5\n")
            f.write("case7Q_120_50\t1\t118\t118\t0.983333\t1\tcase7S_200_40\t1\t100\t200\t0.5\t7e-67\t234\t100\t6\n")
            f.write("case7bisQ_200_40\t1\t100\t200\t0.5\t1\tcase7bisS_118_50\t1\t118\t118\t1\t7e-67\t234\t100\t7\n")
            f.write("case4Q_120_50\t1\t120\t120\t1\t1\tcase4S_118_40\t1\t118\t118\t0.983333\t7e-67\t234\t100\t1\n")
            f.write("case4bisQ_118_50\t1\t118\t118\t1\t1\tcase4bisS_120_40\t1\t118\t118\t0.983333\t7e-67\t234\t100\t2\n")
            f.write("case5Q_200_40\t1\t100\t100\t0.5\t1\tcase5S_118_40\t1\t118\t118\t1\t7e-67\t234\t100\t3\n")
            f.write("case5bisQ_120_40\t1\t120\t120\t1\t1\tcase5bisS_120_40\t1\t118\t118\t0.983333\t7e-67\t234\t100\t4\n") #like case2bis
            f.write("case5bisDummyQ_120_40\t1\t120\t120\t0.5\t1\tcase5bisDummyS_120_40\t1\t118\t118\t0.983333\t7e-67\t234\t100\t4\n") #Is it possible ?
            f.write("case6Q_118_40\t1\t118\t118\t1\t1\tcase6S_120_40\t1\t118\t118\t0.983333\t7e-67\t234\t100\t5\n") #like case2
            f.write("case6DummyQ_118_40\t1\t118\t118\t0.5\t1\tcase6DummyS_120_40\t1\t118\t118\t0.983333\t7e-67\t234\t100\t5\n") #Is it possible ?
        
    def _writeTabFile_keep_all(self, fileName):
        with open(fileName, "w") as f:
            f.write("query.name\tquery.start\tquery.end\tquery.length\tquery.length.%\tmatch.length.%\tsubject.name\tsubject.start\tsubject.end\tsubject.length\tsubject.length.%\tE.value\tScore\tIdentity\tpath\n")
            f.write("case1Q_118_40\t1\t118\t118\t0.983333\t1\tcase1S_120_50\t1\t120\t120\t1\t7e-67\t234\t94\t1\n")
            f.write("case1bisQ_120_40\t1\t118\t118\t0.973333\t1\tcase1bisS_118_50\t1\t118\t118\t1\t7e-67\t234\t100\t2\n")
       
    def _writeClassifFile(self, inputClassifName):
        with open(inputClassifName, "w") as f:
            f.write("Rt1a\t2656\t+\tPotentialChimeric\tII\tLTR\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("header\t2656\t+\tok\tII\tLINE\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("BS31790\t2656\t+\tok\tnoCat\tnoCat\tNA\tCI=NA; struct=(SSRCoverage=0.12<0.95)\n")
            f.write("X-element\t4740\t+\tPotentialChimeric\tI\tLINE\tcomplete\tCI=69; coding=(TE_BLRtx: BS2:classI:LINE: 99.60%; TE_BLRx: BS3_DM-orf2p:classI:LINE: 98.14%; profiles: Exo_endo_phos_EN_11.0: 100.00%, zf-CCHC_GAG_17.9: 100.00%, RVT_1_RT_32.0: 100.00%); struct=(TElength: >700bps; TermRepeats: no termLTR, no termTIR; ORF: >1000bps GAG, >1000bps EN RT; SSR: (A)25_end); other=(SSRCoverage=0.06<0.95)\n")
            f.write("header2\t2656\t+\tok\tII\tSINE\tincomplete\tCI=45; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
            f.write("header3\t2656\t+\tok\tII\tMITE\tcomplete\tCI=55; coding=(TE_BLRtx: Transib4:classII:TIR: 96.72%; TE_BLRx: Transib4p:classII:TIR: 98.25%); struct=(TElength: >700bps; TermRepeats: no termLTR, termTIR); other=(SSRCoverage=0.12<0.95)\n")
    
    def _writeFastaFile(self, inputFastaName):
        with open(inputFastaName, "w") as f:
            f.write(">Rt1a\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">header\n")
            f.write("CAATAGGCATGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATG\n")
            f.write("TCCGCCTTAATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAACCCTCCT\n")
            f.write("CGCGCGCGGGGTGCGTCTCACGCATCTGATTCCAAGTATAAGCCATGTTTCCAGGGTCCA\n")
            f.write("CGCATCACGTCATAGTAAGAGT\n")
            f.write(">BS31790\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">X-element\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write(">header2\n")
            f.write("TTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGAATAAAAAATGATTATTTG\n")
            f.write("CATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGGGATTCGTTTCATTCACCG\n")
            f.write(">header3\n")
            f.write("CACTGGCCTCGATGCTAAGACATACGGAAGTTTTTACCATGTAAAACTTAGAGTTGCGGT\n")
            f.write("ATGTAACATCGTCAGAGATAGGAGCCCGTAGGTGCCTGCAGCACGAAGGGAACTGTAGCC\n")
            f.write("AGTGTAGAGCTATGGATGATCGATAGTAACCAATATCGTTTGGCT\n")
            
    def _writeExpChimFile(self, inputFastaName):
        with open(inputFastaName, "w") as f:
            f.write(">Rt1a\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">X-element\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            
    def _writeExpNotChimFile(self, inputFastaName):
        with open(inputFastaName, "w") as f:
            f.write(">header\n")
            f.write("CAATAGGCATGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATG\n")
            f.write("TCCGCCTTAATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAACCCTCCT\n")
            f.write("CGCGCGCGGGGTGCGTCTCACGCATCTGATTCCAAGTATAAGCCATGTTTCCAGGGTCCA\n")
            f.write("CGCATCACGTCATAGTAAGAGT\n")
            f.write(">BS31790\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">header2\n")
            f.write("TTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGAATAAAAAATGATTATTTG\n")
            f.write("CATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGGGATTCGTTTCATTCACCG\n")
            f.write(">header3\n")
            f.write("CACTGGCCTCGATGCTAAGACATACGGAAGTTTTTACCATGTAAAACTTAGAGTTGCGGT\n")
            f.write("ATGTAACATCGTCAGAGATAGGAGCCCGTAGGTGCCTGCAGCACGAAGGGAACTGTAGCC\n")
            f.write("AGTGTAGAGCTATGGATGATCGATAGTAACCAATATCGTTTGGCT\n")

    def _writeExpFileWithoutRdd(self, inputFastaName):
        with open(inputFastaName, "w") as f:
            f.write(">Rt1a\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("ATAAAAAATGATTATTTGCATGAAATGGGGATGAGAGAGAAGGAAAGAGTTTCATCCTGG\n")
            f.write("GATTCGTTTCATTCACCGGATCTCTTGCGTCCGCCTCCGCCGTGCGACCTCCGCATTC\n")
            f.write(">header\n")
            f.write("CAATAGGCATGAGGGTAACCCGAGGCCAGCTTCGGCGCGTAGTTCTGGGTCTTTCCTATG\n")
            f.write("TCCGCCTTAATCACTGTAAGCGACTGCGTTTGCGAAAGGGCGCGCAGGACCAACCCTCCT\n")
            f.write("CGCGCGCGGGGTGCGTCTCACGCATCTGATTCCAAGTATAAGCCATGTTTCCAGGGTCCA\n")
            f.write("CGCATCACGTCATAGTAAGAGT\n")
            f.write(">X-element\n")
            f.write("GGCCAGTCACAATGGGGGTTTCACTGGTGTGTCATGCACATTTAATAGGGGTAAGACTGA\n")
            f.write("\n")
            f.write(">header3\n")
            f.write("CACTGGCCTCGATGCTAAGACATACGGAAGTTTTTACCATGTAAAACTTAGAGTTGCGGT\n")
            f.write("ATGTAACATCGTCAGAGATAGGAGCCCGTAGGTGCCTGCAGCACGAAGGGAACTGTAGCC\n")
            f.write("AGTGTAGAGCTATGGATGATCGATAGTAACCAATATCGTTTGGCT\n")

if __name__ == "__main__":
    unittest.main()